@@ -29,19 +29,26 @@
 #include "gstrans.h"
 #include "gxdownscale.h"
 
+#include "gdevkrnlsclass.h" /* 'standard' built in subclasses, currently First/Last Page and obejct filter */
+
 /*#define DEBUGGING_HACKS*/
 
 /* GC information */
 static
-ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev)
+ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev);
     if (PRINTER_IS_CLIST(pdev))
-        ENUM_PREFIX(st_device_clist, 0);
+        ENUM_PREFIX(st_device_clist, 2);
     else
-        ENUM_PREFIX(st_device_forward, 0);
+        ENUM_PREFIX(st_device_forward, 2);
+    break;
+case 0:ENUM_RETURN(gx_device_enum_ptr(pdev->parent));
+case 1:ENUM_RETURN(gx_device_enum_ptr(pdev->child));
 ENUM_PTRS_END
 static
 RELOC_PTRS_WITH(device_printer_reloc_ptrs, gx_device_printer *pdev)
 {
+    pdev->parent = gx_device_reloc_ptr(pdev->parent, gcst);
+    pdev->child = gx_device_reloc_ptr(pdev->child, gcst);
     if (PRINTER_IS_CLIST(pdev))
         RELOC_PREFIX(st_device_clist);
     else
@@ -76,17 +83,31 @@ static void prn_print_page_in_background(void *data);
 static void prn_finish_bg_print(gx_device_printer *ppdev);
 
 /* ------ Open/close ------ */
-
 /* Open a generic printer device. */
 /* Specific devices may wish to extend this. */
 int
 gdev_prn_open(gx_device * pdev)
 {
-    gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+    gx_device_printer * ppdev;
     int code;
+    bool update_procs = false;
+
+    code = install_internal_subclass_devices(&pdev, &update_procs);
+    if (code < 0)
+        return code;
+
+    ppdev = (gx_device_printer *)pdev;
 
     ppdev->file = NULL;
     code = gdev_prn_allocate_memory(pdev, NULL, 0, 0);
+    if (update_procs) {
+        if (pdev->ObjectHandlerPushed) {
+            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_obj_filter_device.procs);
+            pdev = pdev->parent;
+        }
+        if (pdev->PageHandlerPushed)
+            gx_copy_device_procs(&pdev->parent->procs, &pdev->procs, (gx_device_procs *)&gs_flp_device.procs);
+    }
     if (code < 0)
         return code;
     if (ppdev->OpenOutputFile)
@@ -104,7 +125,14 @@ prn_finish_bg_print(gx_device_printer *ppdev)
     /* close and unlink the files and free the device and its private allocator	*/
     if (ppdev->bg_print.device != NULL) {
         int closecode;
+        gx_device_printer *bgppdev = (gx_device_printer *)ppdev->bg_print.device;
         gx_semaphore_wait(ppdev->bg_print.sema);
+        /* If numcopies > 1, then the bg_print->device will have closed and reopened
+         * the output file, so the pointer in the original device is now stale,
+         * so copy it back.
+         * If numcopies == 1, this is pointless, but benign.
+         */
+        ppdev->file = bgppdev->file;
         closecode = gdev_prn_close_printer((gx_device *)ppdev);
         if (ppdev->bg_print.return_code == 0)
             ppdev->bg_print.return_code = closecode;	/* return code here iff there wasn't another error */
@@ -237,7 +265,6 @@ open_c:
                       false, /* do_not_open_or_close_bandfiles */
                       (ppdev->bandlist_memory == 0 ? pdev->memory->non_gc_memory:
                        ppdev->bandlist_memory),
-                      ppdev->free_up_bandlist_memory,
                       ppdev->clist_disable_mask,
                       ppdev->page_uses_transparency);
     code = (*gs_clist_device_procs.open_device)( (gx_device *)pcldev );
@@ -294,20 +321,10 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
         ppdev->buffer_space = 0;
         was_command_list = true;
 
-        if (ppdev->bg_print.ocfile) {
-            (void)ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.ocfile, ppdev->bg_print.ocfname, true);
-        }
-        if (ppdev->bg_print.obfile) {
-            (void)ppdev->bg_print.oio_procs->fclose(ppdev->bg_print.obfile, ppdev->bg_print.obfname, true);
-        }
-        ppdev->bg_print.ocfile = ppdev->bg_print.obfile = NULL;
-        if (ppdev->bg_print.ocfname) {
-            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.ocfname, "gdev_prn_tear_down(ocfname)");
-        }
-        if (ppdev->bg_print.obfname) {
-            gs_free_object(ppdev->memory->non_gc_memory, ppdev->bg_print.obfname, "gdev_prn_tear_down(obfname)");
-        }
-        ppdev->bg_print.ocfname = ppdev->bg_print.obfname = NULL;
+        prn_finish_bg_print(ppdev);
+        
+        gs_free_object(pcldev->memory->non_gc_memory, pcldev->cache_chunk, "free tile cache for clist");
+        pcldev->cache_chunk = 0;
 
         rc_decrement(pcldev->icc_cache_cl, "gdev_prn_tear_down");
         pcldev->icc_cache_cl = NULL;
@@ -408,9 +425,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
         space_params = ppdev->space_params;
         space_params.BufferSpace = 0;
         (*ppdev->printer_procs.get_space_params)(ppdev, &space_params);
-        if (ppdev->is_async_renderer && space_params.band.BandBufferSpace != 0)
-            space_params.BufferSpace = space_params.band.BandBufferSpace;
-        else if (space_params.BufferSpace == 0) {
+        if (space_params.BufferSpace == 0) {
             if (space_params.band.BandBufferSpace > 0)
                 space_params.BufferSpace = space_params.band.BandBufferSpace;
             else {
@@ -583,29 +598,6 @@ gdev_prn_free_memory(gx_device *pdev)
     return 0;
 }
 
-/* ------------- Stubs related only to async rendering ------- */
-
-int	/* rets 0 ok, -ve error if couldn't start thread */
-gx_default_start_render_thread(gdev_prn_start_render_params *params)
-{
-    return gs_error_unknownerror;
-}
-
-/* Open the renderer's copy of a device. */
-/* This is overriden in gdevprna.c */
-int
-gx_default_open_render_device(gx_device_printer *pdev)
-{
-    return gs_error_unknownerror;
-}
-
-/* Close the renderer's copy of a device. */
-int
-gx_default_close_render_device(gx_device_printer *pdev)
-{
-    return gdev_prn_close( (gx_device *)pdev );
-}
-
 /* ------ Get/put parameters ------ */
 
 int
@@ -672,7 +664,7 @@ gdev_prn_get_param(gx_device *dev, char *Param, void *list)
     if (strcmp(Param, "pageneutralcolor") == 0) {
         return param_write_bool(plist, "pageneutralcolor", &pageneutralcolor);
     }
-    return gs_error_undefined;
+    return gx_default_get_param(dev, Param, list);
 }
 
 /* Get parameters.  Printer devices add several more parameters */
@@ -810,10 +802,11 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
             if ((bls.size > 1) && (bls.data[0] == 'm' ||
                  (clist_io_procs_file_global != NULL && bls.data[0] == 'f')))
                 break;
-            /* falls through */
+            /* fall through */
         default:
             ecode = code;
             param_signal_error(plist, param_name, ecode);
+            /* fall through */
         case 1:
             bls.data = 0;
             break;
@@ -830,10 +823,11 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
                 code = validate_output_file(&ofs, pdev->memory);
             if (code >= 0)
                 break;
-            /* falls through */
+            /* fall through */
         default:
             ecode = code;
             param_signal_error(plist, param_name, ecode);
+            /* fall through */
         case 1:
             ofs.data = 0;
             break;
@@ -979,7 +973,7 @@ gx_default_get_space_params(const gx_device_printer *printer_dev,
 /* background printing, i.e., thread safe and does not change the device.        */
 /* If the printer device is in 'saved_pages' mode, then background printing is   */
 /* irrelevant and is ignored. In this case, pages are saved to the list.         */
-static int	/* 0 ok, -ve error, or 1 if successfully upgraded to buffer_page */
+static int	/* 0 ok, -ve error                                               */
 gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seekable, bool bg_print_ok)
 {
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
@@ -999,13 +993,7 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
         if ((code = gdev_prn_open_printer_seekable(pdev, 1, seekable)) < 0)
             return code;
 
-        /* If copypage request, try to do it using buffer_page */
-        if ( !flush &&
-             (*ppdev->printer_procs.buffer_page)(ppdev, ppdev->file, num_copies) >= 0) {
-            upgraded_copypage = true;
-            flush = true;
-
-        } else if (num_copies > 0) {
+        if (num_copies > 0) {
             int threads_enabled = 0;
             int print_foreground = 1;		/* default to foreground printing */
 
@@ -1022,10 +1010,7 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 threads_enabled = 0;	/* and allow current page to try foreground */
             }
             /* Use 'while' instead of 'if' to avoid nesting */
-            while (ppdev->bg_print_requested && threads_enabled &&
-                   /* FIXME: Don't allow bg_print if multiple rendering threads are used.  */
-                   /* TEMPORARY WORK AROUND FOR BUG 695711 */
-                   ppdev->num_render_threads_requested == 0) {
+            while (ppdev->bg_print_requested && threads_enabled) {
                 gx_device *ndev;
                 gx_device_printer *npdev;
                 gx_device_clist_reader *crdev = (gx_device_clist_reader *)ppdev;
@@ -1055,8 +1040,11 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 crdev->page_info.cfile = crdev->page_info.bfile = NULL;
 
                 if (ppdev->bg_print.sema == NULL)
-                    if (((ppdev->bg_print.sema = gx_semaphore_alloc(ppdev->memory->non_gc_memory)) == NULL))
+                {
+                    ppdev->bg_print.sema = gx_semaphore_label(gx_semaphore_alloc(ppdev->memory->non_gc_memory), "BGPrint");
+                    if (ppdev->bg_print.sema == NULL)
                         break;			/* couldn't create the semaphore */
+                }
 
                 ndev = setup_device_and_mem_for_thread(pdev->memory->thread_safe_memory, pdev, true, NULL);
                 if (ndev == NULL) {
@@ -1069,12 +1057,13 @@ gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seeka
                 npdev->num_render_threads_requested = ppdev->num_render_threads_requested;
 
                 /* Now start the thread to print the page */
-                if ((code == gp_thread_start(prn_print_page_in_background,
-                                             (void *)&(ppdev->bg_print),
-                                             &(ppdev->bg_print.thread_id))) < 0) {
+                if ((code = gp_thread_start(prn_print_page_in_background,
+                                            (void *)&(ppdev->bg_print),
+                                            &(ppdev->bg_print.thread_id))) < 0) {
                     /* Did not start cleanly - clean up is in print_foreground block below */
                     break;
                 }
+                gp_thread_label(ppdev->bg_print.thread_id, "BG print thread");
                 /* Page was succesfully started in bg_print mode */
                 print_foreground = 0;
                 /* Now we need to set up the next page so it will use new clist files */
@@ -1201,18 +1190,6 @@ gx_default_print_page_copies(gx_device_printer * pdev, FILE * prn_stream,
     return (*pdev->printer_procs.print_page) (pdev, prn_stream);
 }
 
-/*
- * Buffer a (partial) rasterized page & optionally print result multiple times.
- * The default implementation returns error, since the driver needs to override
- * this (in procedure vector) in configurations where this call may occur.
- */
-int
-gx_default_buffer_page(gx_device_printer *pdev, FILE *prn_stream,
-                       int num_copies)
-{
-    return gs_error_unknownerror;
-}
-
 /*
  * Print a page in the background. When printing is complete,
  * post the return code and signal the foreground (semaphore).
@@ -1326,6 +1303,7 @@ gdev_prn_open_printer_seekable(gx_device *pdev, bool binary_mode,
         }
     }
     ppdev->file_is_new = true;
+
     return 0;
 }
 int
@@ -1369,6 +1347,7 @@ gx_page_info_color_usage(const gx_device *dev,
     start = y / band_height;
     end = (y + height + band_height - 1) / band_height;
     if (crdev->color_usage_array == NULL) {
+        return -1;
     }
     for (i = start; i < end; ++i) {
         or |= crdev->color_usage_array[i].or;
@@ -1486,8 +1465,13 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
      */
     gs_deviceinitialmatrix(target, &mdev->initial_matrix);
     if (plane_index >= 0) {
-        gx_device_plane_extract *edev =
-            gs_alloc_struct(mem, gx_device_plane_extract,
+        gx_device_plane_extract *edev;
+
+        /* Guard against potential NULL dereference in gs_alloc_struct */
+        if (!mem)
+            return_error(gs_error_undefined);
+
+        edev = gs_alloc_struct(mem, gx_device_plane_extract,
                             &st_device_plane_extract, "create_buf_device");
 
         if (edev == 0) {
