@@ -529,21 +529,30 @@ bits_extract_plane(const bits_plane_t *dest /*write*/,
             break;
         }
         default: {
-            sample_load_declare_setup(sptr, sbit, source_row, source_bit,
-                                      source_depth);
-            sample_store_declare_setup(dptr, dbit, dbbyte, dest_row, dest_bit,
-                                       dest_depth);
+            const byte *sptr = source_row;
+            int sbit = source_bit;
+            byte *dptr = dest_row;
+            int dbit = dest_bit;
+            byte dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
 
-            sample_store_preload(dbbyte, dptr, dbit, dest_depth);
+            dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
             for (x = width; x > 0; --x) {
                 gx_color_index color;
                 uint pixel;
 
-                sample_load_next_any(color, sptr, sbit, source_depth);
+                if (sizeof(color) > 4) {
+                    if (sample_load_next64((uint64_t *)&color, &sptr, &sbit, source_depth) < 0)
+                        return_error(gs_error_rangecheck);
+                }
+                else {
+                    if (sample_load_next32((uint32_t *)&color, &sptr, &sbit, source_depth) < 0)
+                        return_error(gs_error_rangecheck);
+                }
                 pixel = (color >> shift) & plane_mask;
-                sample_store_next8(pixel, dptr, dbit, dest_depth, dbbyte);
+                if (sample_store_next8(pixel, &dptr, &dbit, dest_depth, &dbbyte) < 0)
+                    return_error(gs_error_rangecheck);
             }
-            sample_store_flush(dptr, dbit, dest_depth, dbbyte);
+            sample_store_flush(dptr, dbit, dbbyte);
         }
         }
     }
@@ -614,21 +623,31 @@ bits_expand_plane(const bits_plane_t *dest /*write*/,
              ++y, source_row += source->raster, dest_row += dest->raster
              ) {
             int x;
-            sample_load_declare_setup(sptr, sbit, source_row, source_bit,
-                                      source_depth);
-            sample_store_declare_setup(dptr, dbit, dbbyte, dest_row, dest_bit,
-                                       dest_depth);
+            const byte *sptr = source_row;
+            int sbit = source_bit;
+            byte *dptr = dest_row;
+            int dbit = dest_bit;
+            byte dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
 
-            sample_store_preload(dbbyte, dptr, dbit, dest_depth);
+            dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
             for (x = width; x > 0; --x) {
                 uint color;
                 gx_color_index pixel;
 
-                sample_load_next8(color, sptr, sbit, source_depth);
-                pixel = color << shift;
-                sample_store_next_any(pixel, dptr, dbit, dest_depth, dbbyte);
+                if (sample_load_next8(&color, &sptr, &sbit, source_depth) < 0)
+                    return_error(gs_error_rangecheck);
+
+                pixel = (gx_color_index)color << shift;
+                if (sizeof(pixel) > 4) {
+                    if (sample_store_next64(pixel, &dptr, &dbit, dest_depth, &dbbyte) < 0)
+                        return_error(gs_error_rangecheck);
+                }
+                else {
+                    if (sample_store_next32(pixel, &dptr, &dbit, dest_depth, &dbbyte) < 0)
+                        return_error(gs_error_rangecheck);
+                }
             }
-            sample_store_flush(dptr, dbit, dest_depth, dbbyte);
+            sample_store_flush(dptr, dbit, dbbyte);
         }
         break;
 
