@@ -49,7 +49,7 @@
 
 /* Set the bounding box for the current path. */
 int
-gs_setbbox(gs_state * pgs, floatp llx, floatp lly, floatp urx, floatp ury)
+gs_setbbox(gs_gstate * pgs, double llx, double lly, double urx, double ury)
 {
     gs_rect ubox, dbox;
     gs_fixed_rect obox, bbox;
@@ -98,12 +98,12 @@ gs_setbbox(gs_state * pgs, floatp llx, floatp lly, floatp urx, floatp ury)
 
 /* Append a list of rectangles to a path. */
 static int
-gs_rectappend_compat(gs_state * pgs, const gs_rect * pr, uint count, bool clip)
+gs_rectappend_compat(gs_gstate * pgs, const gs_rect * pr, uint count, bool clip)
 {
     bool CPSI_mode = gs_currentcpsimode(pgs->memory);
 
     for (; count != 0; count--, pr++) {
-        floatp px = pr->p.x, py = pr->p.y, qx = pr->q.x, qy = pr->q.y;
+        double px = pr->p.x, py = pr->p.y, qx = pr->q.x, qy = pr->q.y;
         int code;
 
         if (CPSI_mode) {
@@ -156,14 +156,14 @@ gs_rectappend_compat(gs_state * pgs, const gs_rect * pr, uint count, bool clip)
     return 0;
 }
 int
-gs_rectappend(gs_state * pgs, const gs_rect * pr, uint count)
+gs_rectappend(gs_gstate * pgs, const gs_rect * pr, uint count)
 {
     return gs_rectappend_compat(pgs, pr, count, false);
 }
 
 /* Clip to a list of rectangles. */
 int
-gs_rectclip(gs_state * pgs, const gs_rect * pr, uint count)
+gs_rectclip(gs_gstate * pgs, const gs_rect * pr, uint count)
 {
     int code;
     gx_path save;
@@ -185,7 +185,7 @@ gs_rectclip(gs_state * pgs, const gs_rect * pr, uint count)
 /* Fill a list of rectangles. */
 /* We take the trouble to do this efficiently in the simple cases. */
 int
-gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
+gs_rectfill(gs_gstate * pgs, const gs_rect * pr, uint count)
 {
     const gs_rect *rlist = pr;
     gx_clip_path *pcpath;
@@ -193,8 +193,8 @@ gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
     int code;
     gx_device * pdev = pgs->device;
     gx_device_color *pdc = gs_currentdevicecolor_inline(pgs);
-    const gs_imager_state *pis = (const gs_imager_state *)pgs;
-    bool hl_color_available = gx_hld_is_hl_color_available(pis, pdc);
+    const gs_gstate *pgs2 = (const gs_gstate *)pgs;
+    bool hl_color_available = gx_hld_is_hl_color_available(pgs2, pdc);
     bool hl_color = (hl_color_available &&
                 dev_proc(pdev, dev_spec_op)(pdev, gxdso_supports_hlcolor, 
                                   NULL, 0));
@@ -214,7 +214,7 @@ gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
          pdc->type == gx_dc_type_pure ||
          pdc->type == gx_dc_type_ht_binary ||
          pdc->type == gx_dc_type_ht_colored) &&
-        gs_state_color_load(pgs) >= 0 &&
+        gs_gstate_color_load(pgs) >= 0 &&
         (*dev_proc(pdev, get_alpha_bits)) (pdev, go_graphics)
         <= 1 &&
         (!pgs->overprint || !pgs->effective_overprint_mode)
@@ -251,7 +251,7 @@ gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
                 if (draw_rect.p.x <= draw_rect.q.x &&
                     draw_rect.p.y <= draw_rect.q.y) {
                     code = dev_proc(pdev, fill_rectangle_hl_color)(pdev,
-                             &draw_rect, pis, pdc, pcpath);
+                             &draw_rect, pgs2, pdc, pcpath);
                     if (code < 0)
                         return code;
                 }
@@ -272,8 +272,8 @@ gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
                 }
                 x = fixed2int(draw_rect.p.x);
                 y = fixed2int(draw_rect.p.y);
-                w = fixed2int(draw_rect.q.x - draw_rect.p.x);
-                h = fixed2int(draw_rect.q.y - draw_rect.p.y);
+                w = fixed2int(draw_rect.q.x) - x;
+                h = fixed2int(draw_rect.q.y) - y;
                 /* clients that use the "any part of pixel" rule also
                    fill 0 areas.  This is true of current graphics
                    library clients but not a general rule.  */
@@ -317,7 +317,7 @@ gs_rectfill(gs_state * pgs, const gs_rect * pr, uint count)
 /* Stroke a list of rectangles. */
 /* (We could do this a lot more efficiently.) */
 int
-gs_rectstroke(gs_state * pgs, const gs_rect * pr, uint count,
+gs_rectstroke(gs_gstate * pgs, const gs_rect * pr, uint count,
               const gs_matrix * pmat)
 {
     bool do_save = pmat != NULL || !gx_path_is_null(pgs->path);
