@@ -60,7 +60,7 @@
  * do the following:
  *   a.  Create a copy of the color space and change the copy so that it
  *       uses its alternate colorspace.
- *   b.  Create a copy of the current imager state and modify its color
+ *   b.  Create a copy of the current gs_gstate and modify its color
  *       mapping (cmap) procs to use a special set of 'capture' procs.
  *   c.  Based upon the type of color space (Separation or DeviceN) create
  *       a 'color' which consists of the desired spot color set to 100% and
@@ -111,7 +111,7 @@
 
 static void
 update_Separation_spot_equivalent_cmyk_colors(gx_device * pdev,
-                    const gs_state * pgs, const gs_color_space * pcs,
+                    const gs_gstate * pgs, const gs_color_space * pcs,
                     gs_devn_params * pdevn_params,
                     equivalent_cmyk_color_params * pparams)
 {
@@ -150,9 +150,61 @@ update_Separation_spot_equivalent_cmyk_colors(gx_device * pdev,
     }
 }
 
+/* This is used for getting the equivalent CMYK values for the spots that
+   may exist in a DeviceN output ICC profile */
+static int
+update_ICC_spot_equivalent_cmyk_colors(gx_device * pdev,
+                    const gs_gstate * pgs, const gs_color_space * pcs,
+                    gs_devn_params * pdevn_params,
+                    equivalent_cmyk_color_params * pparams)
+{
+    int i, j;
+    cmm_dev_profile_t *dev_profile;
+    int code;
+    gs_client_color client_color;
+
+
+    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    if (code < 0)
+        return code;
+    /*
+     * Check if the ICC spot names matche any of the
+     * separations for which we need an equivalent CMYK color.
+     */
+    for (i = 0; i < pdevn_params->separations.num_separations; i++) {
+        if (pparams->color[i].color_info_valid == false) {
+            const devn_separation_name * dev_sep_name =
+                            &(pdevn_params->separations.names[i]);
+            gsicc_colorname_t *name_entry;
+
+            name_entry = dev_profile->spotnames->head;
+
+            for (j = 0; j < dev_profile->device_profile[0]->num_comps; j++) {
+                client_color.paint.values[j] = 0.0;
+            }
+            for (j = 0; j < dev_profile->spotnames->count; j++) {
+                if (compare_color_names(dev_sep_name->data, dev_sep_name->size,
+                                name_entry->name, name_entry->length)) {
+                    /*
+                     * Create a copy of the color space and then modify it
+                     * to force the use of the alternate color space.
+                     */
+                    client_color.paint.values[j] = 1.0;
+                    capture_spot_equivalent_cmyk_colors(pdev, pgs, &client_color,
+                                                pcs, i, pparams);
+                    break;
+                }
+                /* Did not match, grab the next one */
+                name_entry = name_entry->next;
+            }
+        }
+    }
+    return 0;
+}
+
 static void
 update_DeviceN_spot_equivalent_cmyk_colors(gx_device * pdev,
-                    const gs_state * pgs, const gs_color_space * pcs,
+                    const gs_gstate * pgs, const gs_color_space * pcs,
                     gs_devn_params * pdevn_params,
                     equivalent_cmyk_color_params * pparams)
 {
@@ -232,20 +284,26 @@ static bool check_all_colors_known(int num_spot,
 }
 
 /* If possible, update the equivalent CMYK color for a spot color */
-void
-update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
+int
+update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_gstate * pgs,
     gs_devn_params * pdevn_params, equivalent_cmyk_color_params * pparams)
 {
     const gs_color_space * pcs;
+    cmm_dev_profile_t *dev_profile;
+    int code;
+
+    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    if (code < 0)
+        return code;
 
     /* If all of the color_info is valid then there is nothing to do. */
     if (pparams->all_color_info_valid)
-        return;
+        return 0;
 
     /* Verify that we really have some separations. */
     if (pdevn_params->separations.num_separations == 0) {
         pparams->all_color_info_valid = true;
-        return;
+        return 0;
     }
     /*
      * Verify that the given color space is a Separation or a DeviceN color
@@ -265,13 +323,25 @@ update_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
                                                 pdevn_params, pparams);
             pparams->all_color_info_valid = check_all_colors_known
                     (pdevn_params->separations.num_separations, pparams);
+        } else if (pcs->type->index == gs_color_space_index_ICC &&
+            dev_profile->spotnames != NULL) {
+            /* In this case, we are trying to set up the equivalent colors
+               for the spots in the output ICC profile */
+            code = update_ICC_spot_equivalent_cmyk_colors(pdev, pgs, pcs,
+                                                          pdevn_params,
+                                                          pparams);
+            if (code < 0)
+                return code;
+            pparams->all_color_info_valid = check_all_colors_known
+                    (pdevn_params->separations.num_separations, pparams);
         }
     }
+    return 0;
 }
 
 static void
 save_spot_equivalent_cmyk_color(int sep_num,
-                equivalent_cmyk_color_params * pparams, frac cmyk[4])
+                equivalent_cmyk_color_params * pparams, const frac cmyk[4])
 {
     pparams->color[sep_num].c = cmyk[0];
     pparams->color[sep_num].m = cmyk[1];
@@ -315,7 +385,7 @@ static const gx_color_map_procs cmap_capture_cmyk_color = {
 
 static void
 cmap_gray_capture_cmyk_color(frac gray, gx_device_color * pdc,
-        const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+        const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
     equivalent_cmyk_color_params * pparams =
             ((color_capture_device *)dev)->pequiv_cmyk_colors;
@@ -329,20 +399,20 @@ cmap_gray_capture_cmyk_color(frac gray, gx_device_color * pdc,
 
 static void
 cmap_rgb_capture_cmyk_color(frac r, frac g, frac b, gx_device_color * pdc,
-     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
     equivalent_cmyk_color_params * pparams =
             ((color_capture_device *)dev)->pequiv_cmyk_colors;
     int sep_num = ((color_capture_device *)dev)->sep_num;
     frac cmyk[4];
 
-    color_rgb_to_cmyk(r, g, b, pis, cmyk, dev->memory);
+    color_rgb_to_cmyk(r, g, b, pgs, cmyk, dev->memory);
     save_spot_equivalent_cmyk_color(sep_num, pparams, cmyk);
 }
 
 static void
 cmap_cmyk_capture_cmyk_color(frac c, frac m, frac y, frac k, gx_device_color * pdc,
-     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select,
+     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select,
      const gs_color_space *pcs)
 {
     equivalent_cmyk_color_params * pparams =
@@ -359,24 +429,32 @@ cmap_cmyk_capture_cmyk_color(frac c, frac m, frac y, frac k, gx_device_color * p
 
 static void
 cmap_rgb_alpha_capture_cmyk_color(frac r, frac g, frac b, frac alpha,
-        gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+        gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
                          gs_color_select_t select)
 {
-    cmap_rgb_capture_cmyk_color(r, g, b, pdc, pis, dev, select);
+    cmap_rgb_capture_cmyk_color(r, g, b, pdc, pgs, dev, select);
 }
 
 static void
 cmap_separation_capture_cmyk_color(frac all, gx_device_color * pdc,
-     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
-    dprintf("cmap_separation_capture_cmyk_color - this routine should not be executed\n");
+    dmprintf(pgs->memory, "cmap_separation_capture_cmyk_color - this routine should not be executed\n");
 }
 
+/* The call to this is actually going to occur if we happen to be using a 
+   named color profile and doing a replacement.  Since the destination profile
+   will have been CMYK based during the swap out to find the equivalent color, we can
+   go ahead and just grab the cmyk portion */
 static void
 cmap_devicen_capture_cmyk_color(const frac * pcc, gx_device_color * pdc,
-     const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
+     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
 {
-    dprintf("cmap_devicen_capture_cmyk_color - this routine should not be executed\n");
+    equivalent_cmyk_color_params * pparams =
+        ((color_capture_device *)dev)->pequiv_cmyk_colors;
+    int sep_num = ((color_capture_device *)dev)->sep_num;
+
+    save_spot_equivalent_cmyk_color(sep_num, pparams, pcc);
 }
 
 /*
@@ -384,23 +462,31 @@ cmap_devicen_capture_cmyk_color(const frac * pcc, gx_device_color * pdc,
  * alternate color space.
  */
 void
-capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
+capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_gstate * pgs,
     const gs_client_color * pcc, const gs_color_space * pcs,
     int sep_num, equivalent_cmyk_color_params * pparams)
 {
-    gs_imager_state temp_state = *((const gs_imager_state *)pgs);
+    gs_gstate temp_state = *((const gs_gstate *)pgs);
     color_capture_device temp_device = { 0 };
     gx_device_color dev_color;
-    gsicc_rendering_intents_t rendering_intent;
-    int code;
+    gsicc_rendering_param_t render_cond;
     cmm_dev_profile_t *dev_profile;
     cmm_profile_t *curr_output_profile;
-    cmm_dev_profile_t temp_profile;
+    cmm_dev_profile_t temp_profile = {	/* Initialize to 0's/NULL's */
+                          { 0 } /* device_profile[] */, 0 /* proof_profile */,
+                          0 /* link_profile */, 0 /* oi_profile */, 
+                          0 /* postren_profile */,
+                          { {0} } /* rendercond[] */, 0 /* devicegraytok */, 
+                          0 /* graydection */, 0 /* pageneutralcolor */, 
+                          0 /* usefastcolor */, 0 /* supports_devn */,
+                          0 /* sim_overprint */, 0 /* spotnames */, 
+                          0 /* prebandthreshold */, 0 /* memory */, 
+                          { 0 } /* rc_header */
+                          };
 
-    code = dev_proc(pdev, get_profile)(pdev, &dev_profile);
+    dev_proc(pdev, get_profile)(pdev, &dev_profile);
     gsicc_extract_profile(pdev->graphics_type_tag,
-                          dev_profile, &(curr_output_profile),
-                          &rendering_intent);
+                          dev_profile, &(curr_output_profile), &render_cond);
     /*
      * Create a temp device.  The primary purpose of this device is pass the
      * separation number and a pointer to the original device's equivalent
@@ -410,22 +496,36 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     temp_device.color_info = pdev->color_info;
     temp_device.sep_num = sep_num;
     temp_device.pequiv_cmyk_colors = pparams;
-    temp_device.memory = pgs->memory; 
+    temp_device.memory = pgs->memory;
 
     temp_profile.usefastcolor = false;  /* This avoids a few headaches */
+    temp_profile.prebandthreshold = true;
     temp_profile.supports_devn = false;
-    temp_profile.device_profile[0] = curr_output_profile;
-    temp_profile.device_profile[1] = NULL;
-    temp_profile.device_profile[2] = NULL;
-    temp_profile.device_profile[2] = NULL;
-    temp_profile.link_profile = NULL;
-    temp_profile.proof_profile = NULL;
-    temp_profile.oi_profile = NULL;
+    temp_profile.rendercond[0] = render_cond;
+    temp_profile.rendercond[1] = render_cond;
+    temp_profile.rendercond[2] = render_cond;
+    temp_profile.rendercond[3] = render_cond;
     temp_device.icc_struct = &temp_profile;
+
+    /* The equivalent CMYK colors are used for
+       the preview of the composite image and as such should not rely
+       upon the output profile if our output profile is a DeviceN profile.
+       For example, if our output profile was CMYK + OG then if we use
+       the device profile here we may not get CMYK values that
+       even come close to the color we want to simulate.  So, for this
+       case we go ahead and use the default CMYK profile and let the user
+       beware.  Note that the actual separations will be correct however
+       for the CMYK + OG values. */
+
+    if (curr_output_profile->data_cs == gsNCHANNEL) {
+        temp_profile.device_profile[0] = temp_state.icc_manager->default_cmyk;
+    } else {
+        temp_profile.device_profile[0] = curr_output_profile;
+    }
     set_dev_proc(&temp_device, get_profile, gx_default_get_profile);
 
     /*
-     * Create a temp copy of the imager state.  We do this so that we
+     * Create a temp copy of the gs_gstate.  We do this so that we
      * can modify the color space mapping (cmap) procs.  We use our
      * replacment procs to capture the color.  The installation of a
      * Separation or DeviceN color space also sets a use_alt_cspace flag
@@ -435,6 +535,14 @@ capture_spot_equivalent_cmyk_colors(gx_device * pdev, const gs_state * pgs,
     temp_state.color_component_map.use_alt_cspace = true;
 
     /* Now capture the color */
-    pcs->type->remap_color (pcc, pcs, &dev_color, &temp_state, 
+    pcs->type->remap_color (pcc, pcs, &dev_color, &temp_state,
                             (gx_device *)&temp_device, gs_color_select_texture);
 }
+
+/* Used for detecting if we are trying to find the equivalant color during
+   named color replacement */
+bool
+named_color_equivalent_cmyk_colors(const gs_gstate * pgs)
+{
+    return pgs->cmap_procs == &cmap_capture_cmyk_color;
+}
