@@ -34,7 +34,7 @@
 /* ------ Transparency-related graphics state elements ------ */
 
 int
-gs_setblendmode(gs_state *pgs, gs_blend_mode_t mode)
+gs_setblendmode(gs_gstate *pgs, gs_blend_mode_t mode)
 {
 #ifdef DEBUG
     if (gs_debug_c('v')) {
@@ -52,20 +52,20 @@ gs_setblendmode(gs_state *pgs, gs_blend_mode_t mode)
     /* Compatible is now specified to be the same.                        */
     if (mode == BLEND_MODE_Compatible)
         mode = BLEND_MODE_Normal;
-    if (mode < 0 || mode > MAX_BLEND_MODE)
+    if ((int)mode < 0 || (int)mode > MAX_BLEND_MODE)
         return_error(gs_error_rangecheck);
     pgs->blend_mode = mode;
     return 0;
 }
 
 gs_blend_mode_t
-gs_currentblendmode(const gs_state *pgs)
+gs_currentblendmode(const gs_gstate *pgs)
 {
     return pgs->blend_mode;
 }
 
 int
-gs_setopacityalpha(gs_state *pgs, floatp alpha)
+gs_setopacityalpha(gs_gstate *pgs, double alpha)
 {
     if_debug2m('v', pgs->memory, "[v](0x%lx)opacity.alpha = %g\n", (ulong)pgs, alpha);
     pgs->opacity.alpha = (alpha < 0.0 ? 0.0 : alpha > 1.0 ? 1.0 : alpha);
@@ -73,13 +73,13 @@ gs_setopacityalpha(gs_state *pgs, floatp alpha)
 }
 
 float
-gs_currentopacityalpha(const gs_state *pgs)
+gs_currentopacityalpha(const gs_gstate *pgs)
 {
     return pgs->opacity.alpha;
 }
 
 int
-gs_setshapealpha(gs_state *pgs, floatp alpha)
+gs_setshapealpha(gs_gstate *pgs, double alpha)
 {
     if_debug2m('v', pgs->memory, "[v](0x%lx)shape.alpha = %g\n", (ulong)pgs, alpha);
     pgs->shape.alpha = (alpha < 0.0 ? 0.0 : alpha > 1.0 ? 1.0 : alpha);
@@ -87,13 +87,13 @@ gs_setshapealpha(gs_state *pgs, floatp alpha)
 }
 
 float
-gs_currentshapealpha(const gs_state *pgs)
+gs_currentshapealpha(const gs_gstate *pgs)
 {
     return pgs->shape.alpha;
 }
 
 int
-gs_settextknockout(gs_state *pgs, bool knockout)
+gs_settextknockout(gs_gstate *pgs, bool knockout)
 {
     if_debug2m('v', pgs->memory, "[v](0x%lx)text_knockout = %s\n",
               (ulong)pgs, (knockout ? "true" : "false"));
@@ -102,7 +102,7 @@ gs_settextknockout(gs_state *pgs, bool knockout)
 }
 
 bool
-gs_currenttextknockout(const gs_state *pgs)
+gs_currenttextknockout(const gs_gstate *pgs)
 {
     return pgs->text_knockout;
 }
@@ -115,11 +115,11 @@ gs_currenttextknockout(const gs_state *pgs)
    be entered as compositor actions in the pattern clist */
 
 static int
-check_for_nontrans_pattern(gs_state *pgs, unsigned char *comp_name)
+check_for_nontrans_pattern(gs_gstate *pgs, unsigned char *comp_name)
 {
     gx_device * dev = pgs->device;
-    bool is_patt_clist = (strcmp("pattern-clist",dev->dname) == 0);
-    bool is_patt_acum = (strcmp("pattern accumulator",dev->dname) == 0);
+    bool is_patt_clist = gx_device_is_pattern_clist(dev);
+    bool is_patt_acum = gx_device_is_pattern_accum(dev);
 
     /* Check if we are collecting data for a pattern that has no
        transparency.  In that case, we need to ignore the state changes */
@@ -155,24 +155,33 @@ check_for_nontrans_pattern(gs_state *pgs, unsigned char *comp_name)
  * compositor device.
  */
 static int
-gs_state_update_pdf14trans(gs_state * pgs, gs_pdf14trans_params_t * pparams)
+gs_gstate_update_pdf14trans(gs_gstate * pgs, gs_pdf14trans_params_t * pparams)
 {
-    gs_imager_state * pis = (gs_imager_state *)pgs;
     gx_device * dev = pgs->device;
     gx_device *pdf14dev = NULL;
     int code;
+    int curr_num = dev->color_info.num_components;
 
     /*
      * Send the PDF 1.4 create compositor action specified by the parameters.
      */
-    code = send_pdf14trans(pis, dev, &pdf14dev, pparams, pgs->memory);
+    code = send_pdf14trans(pgs, dev, &pdf14dev, pparams, pgs->memory);
+    if (code < 0)
+        return code;
     /*
      * If we created a new PDF 1.4 compositor device then we need to install it
      * into the graphics state.
      */
-    if (code >= 0 && pdf14dev != dev) {
+    if (pdf14dev != dev) {
         gx_set_device_only(pgs, pdf14dev);
     }
+
+    /* If we had a color space change and we are in overprint, then we need to
+       update the drawn_comps */
+    if (pgs->overprint && curr_num != pdf14dev->color_info.num_components) {
+        code = gs_do_set_overprint(pgs);
+    }
+
     return code;
 }
 
@@ -188,23 +197,22 @@ gs_trans_group_params_init(gs_transparency_group_params_t *ptgp)
 }
 
 int
-gs_update_trans_marking_params(gs_state * pgs)
+gs_update_trans_marking_params(gs_gstate * pgs)
 {
     gs_pdf14trans_params_t params = { 0 };
 
     if_debug0m('v', pgs->memory, "[v]gs_update_trans_marking_params\n");
     params.pdf14_op = PDF14_SET_BLEND_PARAMS;
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 int
-gs_begin_transparency_group(gs_state *pgs,
+gs_begin_transparency_group(gs_gstate *pgs,
                             const gs_transparency_group_params_t *ptgp,
                             const gs_rect *pbbox)
 {
     gs_pdf14trans_params_t params = { 0 };
     const gs_color_space *blend_color_space;
-    gs_imager_state * pis = (gs_imager_state *)pgs;
     cmm_profile_t *profile;
 
     if (check_for_nontrans_pattern(pgs,
@@ -237,7 +245,7 @@ gs_begin_transparency_group(gs_state *pgs,
            ICC default color space in these cases. */
         blend_color_space = gs_currentcolorspace_inline(pgs);
     } else {
-        blend_color_space = cs_concrete_space(blend_color_space, pis);
+        blend_color_space = cs_concrete_space(blend_color_space, pgs);
     }
     /* Note that if the /CS parameter was NOT present in the push
        of the transparency group, then we must actually inherent
@@ -245,8 +253,10 @@ gs_begin_transparency_group(gs_state *pgs,
        target device (process color model).  Here we just want
        to set it as a unknown type for clist writing, as we will take care
        of using the parent group color space later during clist reading.
+       Also, if the group was not isolated we MUST use the parent group 
+       color space regardless of what the group color space is specified to be
        */
-    if (ptgp->ColorSpace == NULL) {
+    if (ptgp->ColorSpace == NULL || params.Isolated != true) {
         params.group_color = UNKNOWN;
         params.group_color_numcomps = 0;
     } else {
@@ -315,11 +325,11 @@ gs_begin_transparency_group(gs_state *pgs,
     }
 #endif
     params.bbox = *pbbox;
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 int
-gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
+gx_begin_transparency_group(gs_gstate * pgs, gx_device * pdev,
                                 const gs_pdf14trans_params_t * pparams)
 {
     gs_transparency_group_params_t tgp = {0};
@@ -339,9 +349,9 @@ gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
     tgp.iccprofile = pparams->iccprofile;
     tgp.icc_hashcode = pparams->icc_hash;
 
-    pis->opacity.alpha = pparams->opacity.alpha;
-    pis->shape.alpha = pparams->shape.alpha;
-    pis->blend_mode = pparams->blend_mode;
+    pgs->opacity.alpha = pparams->opacity.alpha;
+    pgs->shape.alpha = pparams->shape.alpha;
+    pgs->blend_mode = pparams->blend_mode;
     bbox = pparams->bbox;
 #ifdef DEBUG
     if (gs_debug_c('v')) {
@@ -349,7 +359,7 @@ gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
             GS_COLOR_SPACE_TYPE_NAMES
         };
         dmlprintf6(pdev->memory, "[v](0x%lx)gx_begin_transparency_group [%g %g %g %g] Num_grp_clr_comp = %d\n",
-                  (ulong)pis, bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y,
+                  (ulong)pgs, bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y,
                         pparams->group_color_numcomps);
         if (tgp.ColorSpace)
             dmprintf1(pdev->memory, "     CS = %s",
@@ -363,32 +373,32 @@ gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
     }
 #endif
     if (dev_proc(pdev, begin_transparency_group) != 0)
-        return (*dev_proc(pdev, begin_transparency_group)) (pdev, &tgp,
-                                                        &bbox, pis, NULL);
+        return (*dev_proc(pdev, begin_transparency_group)) (pdev, &tgp, &bbox, pgs,
+                                                            NULL);
     else
         return 0;
 }
 
 int
-gs_end_transparency_group(gs_state *pgs)
+gs_end_transparency_group(gs_gstate *pgs)
 {
     gs_pdf14trans_params_t params = { 0 };
 
-    if (pgs->is_gstate && check_for_nontrans_pattern(pgs,
+    if (check_for_nontrans_pattern(pgs,
                   (unsigned char *)"gs_end_transparency_group")) {
         return(0);
     }
     if_debug0m('v', pgs->memory, "[v]gs_end_transparency_group\n");
     params.pdf14_op = PDF14_END_TRANS_GROUP;  /* Other parameters not used */
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 int
-gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev)
+gx_end_transparency_group(gs_gstate * pgs, gx_device * pdev)
 {
-    if_debug0m('v', pis->memory, "[v]gx_end_transparency_group\n");
+    if_debug0m('v', pgs->memory, "[v]gx_end_transparency_group\n");
     if (dev_proc(pdev, end_transparency_group) != 0)
-        return (*dev_proc(pdev, end_transparency_group)) (pdev, pis);
+        return (*dev_proc(pdev, end_transparency_group)) (pdev, pgs);
     else
         return 0;
 }
@@ -396,10 +406,9 @@ gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev)
 /* Commands for handling q softmask Q in graphic states */
 
 int
-gs_push_transparency_state(gs_state *pgs)
+gs_push_transparency_state(gs_gstate *pgs)
 {
     gs_pdf14trans_params_t params = { 0 };
-    gs_imager_state * pis = (gs_imager_state *)pgs;
     int code;
 
     if (check_for_nontrans_pattern(pgs,
@@ -410,7 +419,7 @@ gs_push_transparency_state(gs_state *pgs)
        that we need to watch for end transparency
        soft masks when we are at this graphic state
        level */
-    /* pis->trans_flags.xstate_pending = true; */
+    /* pgs->trans_flags.xstate_pending = true; */
     /* Actually I believe the above flag is not
        needed.  We really should be watching for
        the softmask even at the base level.  What
@@ -421,10 +430,10 @@ gs_push_transparency_state(gs_state *pgs)
        We will need to send a push state to save
        the current soft mask, so that we can
        restore it later */
-    if (pis->trans_flags.xstate_change) {
+    if (pgs->trans_flags.xstate_change) {
         if_debug0m('v', pgs->memory, "[v]gs_push_transparency_state sending\n");
         params.pdf14_op = PDF14_PUSH_TRANS_STATE;
-        code = gs_state_update_pdf14trans(pgs, &params);
+        code = gs_gstate_update_pdf14trans(pgs, &params);
         if (code < 0)
             return(code);
     } else {
@@ -434,10 +443,9 @@ gs_push_transparency_state(gs_state *pgs)
 }
 
 int
-gs_pop_transparency_state(gs_state *pgs, bool force)
+gs_pop_transparency_state(gs_gstate *pgs, bool force)
 {
     gs_pdf14trans_params_t params = { 0 };
-    gs_imager_state * pis = (gs_imager_state *)pgs;
     int code;
 
     if (check_for_nontrans_pattern(pgs,
@@ -448,10 +456,10 @@ gs_pop_transparency_state(gs_state *pgs, bool force)
        an active softmask for the graphic state.  We
        need to communicate to the compositor to pop
        the softmask */
-    if ( pis->trans_flags.xstate_change || force) {
+    if ( pgs->trans_flags.xstate_change || force) {
         if_debug0m('v', pgs->memory, "[v]gs_pop_transparency_state sending\n");
         params.pdf14_op = PDF14_POP_TRANS_STATE;
-        code = gs_state_update_pdf14trans(pgs, &params);
+        code = gs_gstate_update_pdf14trans(pgs, &params);
         if ( code < 0 )
             return (code);
     } else {
@@ -463,21 +471,21 @@ gs_pop_transparency_state(gs_state *pgs, bool force)
 }
 
 int
-gx_pop_transparency_state(gs_imager_state * pis, gx_device * pdev)
+gx_pop_transparency_state(gs_gstate * pgs, gx_device * pdev)
 {
-    if_debug0m('v', pis->memory, "[v]gx_pop_transparency_state\n");
+    if_debug0m('v', pgs->memory, "[v]gx_pop_transparency_state\n");
     if (dev_proc(pdev, pop_transparency_state) != 0)
-        return (*dev_proc(pdev, pop_transparency_state)) (pdev, pis);
+        return (*dev_proc(pdev, pop_transparency_state)) (pdev, pgs);
     else
         return 0;
 }
 
 int
-gx_push_transparency_state(gs_imager_state * pis, gx_device * pdev)
+gx_push_transparency_state(gs_gstate * pgs, gx_device * pdev)
 {
-    if_debug0m('v', pis->memory, "[v]gx_push_transparency_state\n");
+    if_debug0m('v', pgs->memory, "[v]gx_push_transparency_state\n");
     if (dev_proc(pdev, push_transparency_state) != 0)
-        return (*dev_proc(pdev, push_transparency_state)) (pdev, pis);
+        return (*dev_proc(pdev, push_transparency_state)) (pdev, pgs);
     else
         return 0;
 }
@@ -486,7 +494,7 @@ gx_push_transparency_state(gs_imager_state * pis, gx_device * pdev)
  * Handler for identity mask transfer functions.
  */
 static int
-mask_transfer_identity(floatp in, float *out, void *proc_data)
+mask_transfer_identity(double in, float *out, void *proc_data)
 {
     *out = (float) in;
     return 0;
@@ -499,6 +507,7 @@ gs_trans_mask_params_init(gs_transparency_mask_params_t *ptmp,
     ptmp->ColorSpace = 0;
     ptmp->subtype = subtype;
     ptmp->Background_components = 0;
+    ptmp->Matte_components = 0;
     ptmp->GrayBackground = 0.0;
     ptmp->TransferFunction = mask_transfer_identity;
     ptmp->TransferFunction_data = 0;
@@ -507,13 +516,14 @@ gs_trans_mask_params_init(gs_transparency_mask_params_t *ptmp,
 }
 
 int
-gs_begin_transparency_mask(gs_state * pgs,
+gs_begin_transparency_mask(gs_gstate * pgs,
                            const gs_transparency_mask_params_t * ptmp,
                            const gs_rect * pbbox, bool mask_is_image)
 {
     gs_pdf14trans_params_t params = { 0 };
     gs_pdf14trans_params_t params_color = { 0 };
     const int l = sizeof(params.Background[0]) * ptmp->Background_components;
+    const int m = sizeof(params.Matte[0]) * ptmp->Matte_components;
     int i, code;
     gs_color_space *blend_color_space;
     gsicc_manager_t *icc_manager = pgs->icc_manager;
@@ -527,6 +537,8 @@ gs_begin_transparency_mask(gs_state * pgs,
     params.subtype = ptmp->subtype;
     params.Background_components = ptmp->Background_components;
     memcpy(params.Background, ptmp->Background, l);
+    params.Matte_components = ptmp->Matte_components;
+    memcpy(params.Matte, ptmp->Matte, m);
     params.GrayBackground = ptmp->GrayBackground;
     params.transfer_function = ptmp->TransferFunction_data;
     params.function_is_identity =
@@ -550,60 +562,72 @@ gs_begin_transparency_mask(gs_state * pgs,
     }
     /* A new soft mask group,  make sure the profiles are set */
     if_debug0m('v', pgs->memory, "[v]pushing soft mask color sending\n");
-    params_color.pdf14_op = PDF14_PUSH_SMASK_COLOR;
-    code = gs_state_update_pdf14trans(pgs, &params_color);
-    if (code < 0)
-        return(code);
-    blend_color_space = gs_cspace_new_DeviceGray(pgs->memory);
-    blend_color_space->cmm_icc_profile_data = pgs->icc_manager->default_gray;
-    rc_increment(blend_color_space->cmm_icc_profile_data);
-    if_debug8m('v', pgs->memory, "[v](0x%lx)gs_begin_transparency_mask [%g %g %g %g]\n\
-      subtype = %d  Background_components = %d %s\n",
-              (ulong)pgs, pbbox->p.x, pbbox->p.y, pbbox->q.x, pbbox->q.y,
-              (int)ptmp->subtype, ptmp->Background_components,
-              (ptmp->TransferFunction == mask_transfer_identity ? "no TR" :
-               "has TR"));
-    /* Sample the transfer function */
-    for (i = 0; i < MASK_TRANSFER_FUNCTION_SIZE; i++) {
-        float in = (float)(i * (1.0 / (MASK_TRANSFER_FUNCTION_SIZE - 1)));
-        float out;
+    if (params.subtype != TRANSPARENCY_MASK_None) {
+        params_color.pdf14_op = PDF14_PUSH_SMASK_COLOR;
+        code = gs_gstate_update_pdf14trans(pgs, &params_color);
+        if (code < 0)
+            return(code);
+        blend_color_space = gs_cspace_new_DeviceGray(pgs->memory);
+        blend_color_space->cmm_icc_profile_data = pgs->icc_manager->default_gray;
+        rc_increment(blend_color_space->cmm_icc_profile_data);
+        if_debug9m('v', pgs->memory, "[v](0x%lx)gs_begin_transparency_mask [%g %g %g %g]\n\
+          subtype = %d  Background_components = %d, Matte_components = %d, %s\n",
+                  (ulong)pgs, pbbox->p.x, pbbox->p.y, pbbox->q.x, pbbox->q.y,
+                  (int)ptmp->subtype, ptmp->Background_components,
+                  ptmp->Matte_components,
+                  (ptmp->TransferFunction == mask_transfer_identity ? "no TR" :
+                   "has TR"));
+        /* Sample the transfer function */
+        for (i = 0; i < MASK_TRANSFER_FUNCTION_SIZE; i++) {
+            float in = (float)(i * (1.0 / (MASK_TRANSFER_FUNCTION_SIZE - 1)));
+            float out;
 
-        ptmp->TransferFunction(in, &out, ptmp->TransferFunction_data);
-        params.transfer_fn[i] = (byte)floor((double)(out * 255 + 0.5));
+            ptmp->TransferFunction(in, &out, ptmp->TransferFunction_data);
+            params.transfer_fn[i] = (byte)floor((double)(out * 255 + 0.5));
+        }
+        /* Note:  This function is called during the c-list writer side. */
+        if ( blend_color_space->cmm_icc_profile_data != NULL ) {
+        /* Blending space is ICC based.  If we are doing c-list rendering we will
+           need to write this color space into the clist. */
+            params.group_color = ICC;
+            params.group_color_numcomps =
+                    blend_color_space->cmm_icc_profile_data->num_comps;
+            /* Get the ICC profile */
+            /* We don't reference count this - see comment in
+             * pdf14_update_device_color_procs_pop_c()
+             */
+            params.iccprofile = blend_color_space->cmm_icc_profile_data;
+            params.icc_hash = blend_color_space->cmm_icc_profile_data->hashcode;
+        } else {
+            params.group_color = GRAY_SCALE;
+            params.group_color_numcomps = 1;  /* Need to check */
+        }
+        /* Explicitly decrement the profile data since blend_color_space may not
+         * be an ICC color space object.
+         */
+        rc_decrement(blend_color_space->cmm_icc_profile_data, "gs_begin_transparency_mask");
+        rc_decrement_only_cs(blend_color_space, "gs_begin_transparency_mask");
     }
-    /* Note:  This function is called during the c-list writer side. */
-    if ( blend_color_space->cmm_icc_profile_data != NULL ) {
-    /* Blending space is ICC based.  If we are doing c-list rendering we will
-       need to write this color space into the clist. */
-        params.group_color = ICC;
-        params.group_color_numcomps =
-                blend_color_space->cmm_icc_profile_data->num_comps;
-        /* Get the ICC profile */
-        params.iccprofile = blend_color_space->cmm_icc_profile_data;
-        params.icc_hash = blend_color_space->cmm_icc_profile_data->hashcode;
-        rc_increment(params.iccprofile);
-    } else {
-        params.group_color = GRAY_SCALE;
-        params.group_color_numcomps = 1;  /* Need to check */
-    }
-    rc_decrement_only_cs(blend_color_space, "gs_begin_transparency_mask");
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 /* This occurs on the c-list reader side */
 
 int
-gx_begin_transparency_mask(gs_imager_state * pis, gx_device * pdev,
+gx_begin_transparency_mask(gs_gstate * pgs, gx_device * pdev,
                                 const gs_pdf14trans_params_t * pparams)
 {
     gx_transparency_mask_params_t tmp;
     const int l = sizeof(pparams->Background[0]) * pparams->Background_components;
+    const int m = sizeof(pparams->Matte[0]) * pparams->Matte_components;
 
     tmp.group_color = pparams->group_color;
     tmp.subtype = pparams->subtype;
     tmp.group_color_numcomps = pparams->group_color_numcomps;
     tmp.Background_components = pparams->Background_components;
     memcpy(tmp.Background, pparams->Background, l);
+    tmp.Matte_components = pparams->Matte_components;
+    memcpy(tmp.Matte, pparams->Matte, m);
     tmp.GrayBackground = pparams->GrayBackground;
     tmp.function_is_identity = pparams->function_is_identity;
     tmp.idle = pparams->idle;
@@ -619,29 +643,28 @@ gx_begin_transparency_mask(gs_imager_state * pis, gx_device * pdev,
         tmp.icc_hashcode = 0;
     }
     memcpy(tmp.transfer_fn, pparams->transfer_fn, size_of(tmp.transfer_fn));
-    if_debug9m('v', pis->memory,
+    if_debug10m('v', pgs->memory,
                "[v](0x%lx)gx_begin_transparency_mask [%g %g %g %g]\n"
-               "      subtype = %d  Background_components = %d  Num_grp_clr_comp = %d %s\n",
-              (ulong)pis, pparams->bbox.p.x, pparams->bbox.p.y,
+               "      subtype = %d  Background_components = %d Matte_components = %d Num_grp_clr_comp = %d %s\n",
+              (ulong)pgs, pparams->bbox.p.x, pparams->bbox.p.y,
               pparams->bbox.q.x, pparams->bbox.q.y,
-              (int)tmp.subtype, tmp.Background_components,
+              (int)tmp.subtype, tmp.Background_components, tmp.Matte_components,
               tmp.group_color_numcomps,
               (tmp.function_is_identity ? "no TR" :
                "has TR"));
     if (dev_proc(pdev, begin_transparency_mask) != 0)
         return (*dev_proc(pdev, begin_transparency_mask))
-                        (pdev, &tmp, &(pparams->bbox), pis, NULL);
+                        (pdev, &tmp, &(pparams->bbox), pgs, NULL);
     else
         return 0;
 }
 
 int
-gs_end_transparency_mask(gs_state *pgs,
+gs_end_transparency_mask(gs_gstate *pgs,
                          gs_transparency_channel_selector_t csel)
 {
     gs_pdf14trans_params_t params = { 0 };
     gs_pdf14trans_params_t params_color = { 0 };
-    gs_imager_state * pis = (gs_imager_state *)pgs;
     int code;
 
     if (check_for_nontrans_pattern(pgs,
@@ -649,40 +672,40 @@ gs_end_transparency_mask(gs_state *pgs,
         return(0);
     }
     /* If we have done a q then set a flag to watch for any Qs */
-   /* if (pis->trans_flags.xstate_pending)
-        pis->trans_flags.xstate_change = true; */
+   /* if (pgs->trans_flags.xstate_pending)
+        pgs->trans_flags.xstate_change = true; */
     /* This should not depend upon if we have encountered a q
        operation.  We could be setting a softmask, before
        there is any q operation.  Unlikely but it could happen.
        Then if we encouter a q operation (and this flag
        is true) we will need to
        push the mask graphic state (PDF14_PUSH_TRANS_STATE). */
-    pis->trans_flags.xstate_change = true;
-    if_debug1m('v', pis->memory,
+    pgs->trans_flags.xstate_change = true;
+    if_debug1m('v', pgs->memory,
                "[v]xstate_changed set true, gstate level is %d\n", pgs->level);
-    if_debug2m('v', pis->memory,
+    if_debug2m('v', pgs->memory,
                "[v](0x%lx)gs_end_transparency_mask(%d)\n", (ulong)pgs,
                (int)csel);
     params.pdf14_op = PDF14_END_TRANS_MASK;  /* Other parameters not used */
     params.csel = csel;
     /* If this is the outer end then return us to our normal defaults */
-    if_debug0m('v', pis->memory, "[v]popping soft mask color sending\n");
+    if_debug0m('v', pgs->memory, "[v]popping soft mask color sending\n");
     params_color.pdf14_op = PDF14_POP_SMASK_COLOR;
-    code = gs_state_update_pdf14trans(pgs, &params_color);
+    code = gs_gstate_update_pdf14trans(pgs, &params_color);
     if (code < 0)
         return(code);
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 int
-gx_end_transparency_mask(gs_imager_state * pis, gx_device * pdev,
+gx_end_transparency_mask(gs_gstate * pgs, gx_device * pdev,
                                 const gs_pdf14trans_params_t * pparams)
 {
-    if_debug2m('v', pis->memory,
-               "[v](0x%lx)gx_end_transparency_mask(%d)\n", (ulong)pis,
+    if_debug2m('v', pgs->memory,
+               "[v](0x%lx)gx_end_transparency_mask(%d)\n", (ulong)pgs,
                (int)pparams->csel);
     if (dev_proc(pdev, end_transparency_mask) != 0)
-        return (*dev_proc(pdev, end_transparency_mask)) (pdev, pis);
+        return (*dev_proc(pdev, end_transparency_mask)) (pdev, pgs);
     else
         return 0;
 }
@@ -694,7 +717,7 @@ gx_end_transparency_mask(gs_imager_state * pis, gx_device * pdev,
  * and convert spot colors into process colors.
  */
 static int
-get_num_pdf14_spot_colors(gs_state * pgs)
+get_num_pdf14_spot_colors(gs_gstate * pgs)
 {
     gx_device * dev = pgs->device;
     gs_devn_params * pclist_devn_params = dev_proc(dev, ret_devn_params)(dev);
@@ -723,17 +746,19 @@ get_num_pdf14_spot_colors(gs_state * pgs)
 }
 
 int
-gs_push_pdf14trans_device(gs_state * pgs, bool is_pattern)
+gs_push_pdf14trans_device(gs_gstate * pgs, bool is_pattern)
 {
     gs_pdf14trans_params_t params = { 0 };
     cmm_profile_t *icc_profile;
-    gsicc_rendering_param_t render_cond;   
+    gsicc_rendering_param_t render_cond;
     int code;
     cmm_dev_profile_t *dev_profile;
 
     code = dev_proc(pgs->device, get_profile)(pgs->device,  &dev_profile);
-    gsicc_extract_profile(GS_UNKNOWN_TAG, dev_profile, &icc_profile, 
-                          &render_cond); 
+    if (code < 0)
+        return code;
+    gsicc_extract_profile(GS_UNKNOWN_TAG, dev_profile, &icc_profile,
+                          &render_cond);
     params.pdf14_op = PDF14_PUSH_DEVICE;
     /*
      * We really only care about the number of spot colors when we have
@@ -752,15 +777,37 @@ gs_push_pdf14trans_device(gs_state * pgs, bool is_pattern)
         params.iccprofile = pgs->icc_manager->default_rgb;
     }
     /* Note: Other parameters not used */
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
 
 int
-gs_pop_pdf14trans_device(gs_state * pgs, bool is_pattern)
+gs_pop_pdf14trans_device(gs_gstate * pgs, bool is_pattern)
 {
     gs_pdf14trans_params_t params = { 0 };
 
     params.is_pattern = is_pattern;
     params.pdf14_op = PDF14_POP_DEVICE;  /* Other parameters not used */
-    return gs_state_update_pdf14trans(pgs, &params);
+    return gs_gstate_update_pdf14trans(pgs, &params);
 }
+
+int
+gs_abort_pdf14trans_device(gs_gstate * pgs)
+{
+    gs_pdf14trans_params_t params = { 0 };
+
+    params.pdf14_op = PDF14_ABORT_DEVICE;  /* Other parameters not used */
+    return gs_gstate_update_pdf14trans(pgs, &params);
+}
+
+/* Something has gone wrong have the device clean up everything */
+
+int
+gx_abort_trans_device(gs_gstate * pgs, gx_device * pdev)
+{
+    if_debug1m('v', pgs->memory, "[v](0x%lx)gx_abort_trans_device\n", (ulong)pgs);
+    if (dev_proc(pdev, discard_transparency_layer) != 0)
+    return (*dev_proc(pdev, discard_transparency_layer)) (pdev, pgs);
+    else
+    return 0;
+}
+
