@@ -35,10 +35,6 @@
 #include "gzpath.h"
 #include "gxfcid.h"
 
-/* Define the maximum size of a full temporary bitmap when rasterizing, */
-/* in bits (not bytes). */
-static const uint MAX_TEMP_BITMAP_BITS = 80000;
-
 /* Define whether the show operation uses the character outline data, */
 /* as opposed to just needing the width (or nothing). */
 #define SHOW_USES_OUTLINE(penum)\
@@ -47,7 +43,7 @@ static const uint MAX_TEMP_BITMAP_BITS = 80000;
 /* Structure descriptors */
 public_st_gs_show_enum();
 extern_st(st_gs_text_enum);
-extern_st(st_gs_state);         /* only for testing */
+extern_st(st_gs_gstate);         /* only for testing */
 static
 ENUM_PTRS_BEGIN(show_enum_enum_ptrs)
      return ENUM_USING(st_gs_text_enum, vptr, size, index - 5);
@@ -71,7 +67,7 @@ static int continue_show_update(gs_show_enum *);
 static void show_set_scale(const gs_show_enum *, gs_log2_scale_point *log2_scale);
 static int show_cache_setup(gs_show_enum *);
 static int show_state_setup(gs_show_enum *);
-static int show_origin_setup(gs_state *, fixed, fixed, gs_show_enum * penum);
+static int show_origin_setup(gs_gstate *, fixed, fixed, gs_show_enum * penum);
 
 /* Accessors for current_char and current_glyph. */
 #define CURRENT_CHAR(penum) ((penum)->returned.current_char)
@@ -83,7 +79,7 @@ static int show_origin_setup(gs_state *, fixed, fixed, gs_show_enum * penum);
 
 /* Allocate a show enumerator. */
 gs_show_enum *
-gs_show_enum_alloc(gs_memory_t * mem, gs_state * pgs, client_name_t cname)
+gs_show_enum_alloc(gs_memory_t * mem, gs_gstate * pgs, client_name_t cname)
 {
     gs_show_enum *penum;
 
@@ -123,7 +119,7 @@ static const gs_text_enum_procs_t default_text_procs = {
 };
 
 int
-gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
+gx_default_text_begin(gx_device * dev, gs_gstate * pgs1,
                       const gs_text_params_t * text, gs_font * font,
                       gx_path * path, const gx_device_color * pdcolor,
                       const gx_clip_path * pcpath,
@@ -132,20 +128,20 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
     uint operation = text->operation;
     bool propagate_charpath = (operation & TEXT_DO_DRAW) != 0;
     int code;
-    gs_state *pgs = (gs_state *)pis;
+    gs_gstate *pgs = (gs_gstate *)pgs1;
     gs_show_enum *penum;
 
     /*
-     * For the moment, require pis to be a gs_state *, since all the
+     * For the moment, require pgs to be a gs_gstate *, since all the
      * procedures for character rendering expect it.
      */
-    if (gs_object_type(mem, pis) != &st_gs_state)
+    if (gs_object_type(mem, pgs) != &st_gs_gstate)
         return_error(gs_error_Fatal);
     penum = gs_show_enum_alloc(mem, pgs, "gx_default_text_begin");
     if (!penum)
         return_error(gs_error_VMerror);
     code = gs_text_enum_init((gs_text_enum_t *)penum, &default_text_procs,
-                             dev, pis, text, font, path, pdcolor, pcpath, mem);
+                             dev, pgs, text, font, path, pdcolor, pcpath, mem);
     if (code < 0) {
         gs_free_object(mem, penum, "gx_default_text_begin");
         return code;
@@ -180,10 +176,8 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
     penum->show_gstate =
         (propagate_charpath && (pgs->in_charpath != 0) ?
          pgs->show_gstate : pgs);
-    if((operation &
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) ==
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
-        /* This is stringwidth. */
+    if (!(~operation & (TEXT_DO_NONE | TEXT_RETURN_WIDTH))) {
+        /* This is stringwidth (or a PDF with text in rendering mode 3) . */
         gx_device_null *dev_null =
             gs_alloc_struct(mem, gx_device_null, &st_device_null,
                             "stringwidth(dev_null)");
@@ -214,7 +208,7 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
 
 /* Compute the number of characters in a text. */
 int
-gs_text_count_chars(gs_state * pgs, gs_text_params_t *text, gs_memory_t * mem)
+gs_text_count_chars(gs_gstate * pgs, gs_text_params_t *text, gs_memory_t * mem)
 {
     font_proc_next_char_glyph((*next_proc)) = pgs->font->procs.next_char_glyph;
 
@@ -248,13 +242,12 @@ gs_text_count_chars(gs_state * pgs, gs_text_params_t *text, gs_memory_t * mem)
 
 /* An auxiliary functions for pdfwrite to process type 3 fonts. */
 int
-gx_hld_stringwidth_begin(gs_imager_state * pis, gx_path **path)
+gx_hld_stringwidth_begin(gs_gstate * pgs, gx_path **path)
 {
-    gs_state *pgs = (gs_state *)pis;
-    extern_st(st_gs_state);
+    extern_st(st_gs_gstate);
     int code;
 
-    if (gs_object_type(pis->memory, pis) != &st_gs_state)
+    if (gs_object_type(pgs->memory, pgs) != &st_gs_gstate)
         return_error(gs_error_unregistered);
     code = gs_gsave(pgs);
     if (code < 0)
@@ -269,7 +262,7 @@ int
 gx_default_text_restore_state(gs_text_enum_t *pte)
 {
     gs_show_enum *penum;
-    gs_state *pgs;
+    gs_gstate *pgs;
 
     if (SHOW_IS(pte, TEXT_DO_NONE))
         return 0;
@@ -280,8 +273,8 @@ gx_default_text_restore_state(gs_text_enum_t *pte)
 /* ------ Width/cache setting ------ */
 
 static int
-    set_cache_device(gs_show_enum *penum, gs_state *pgs,
-                     floatp llx, floatp lly, floatp urx, floatp ury);
+    set_cache_device(gs_show_enum *penum, gs_gstate *pgs,
+                     double llx, double lly, double urx, double ury);
 
 /* This is the default implementation of text enumerator set_cache. */
 static int
@@ -289,7 +282,7 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
                           gs_text_cache_control_t control)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     gs_font *pfont = gs_rootfont(pgs);
 
     /* Detect zero FontMatrix now for Adobe compatibility with CET tests.
@@ -366,7 +359,7 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 /* Note that this returns 1 if the current show operation is */
 /* non-displaying (stringwidth or cshow). */
 int
-set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
+set_char_width(gs_show_enum *penum, gs_gstate *pgs, double wx, double wy)
 {
     int code;
 
@@ -440,7 +433,7 @@ compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *al
                     int *depth,
                     gs_fixed_point *subpix_origin, gs_log2_scale_point *log2_scale)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     gx_device *dev = gs_currentdevice_inline(pgs);
     int code;
 
@@ -489,8 +482,8 @@ compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *al
 /* Return 1 if we just set up a cache device. */
 /* Used by setcachedevice and setcachedevice2. */
 static int
-set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
-                 floatp urx, floatp ury)
+set_cache_device(gs_show_enum * penum, gs_gstate * pgs, double llx, double lly,
+                 double urx, double ury)
 {
     gs_glyph glyph;
 
@@ -499,7 +492,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         return 0;
     if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE)) { /* cshow */
         int code;
-        if_debug0('k', "[k]no cache: cshow");
+        if_debug0m('k', penum->memory, "[k]no cache: cshow");
         code = gs_nulldevice(pgs);
         if (code < 0)
             return code;
@@ -508,13 +501,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
     pgs->in_cachedevice = CACHE_DEVICE_NOT_CACHING;     /* disable color/gray/image operators */
     /* We can only use the cache if we know the glyph. */
     glyph = CURRENT_GLYPH(penum);
-    if (glyph == gs_no_glyph)
+    if (glyph == GS_NO_GLYPH)
         return 0;
     /* We can only use the cache if ctm is unchanged */
     /* (aside from a possible translation). */
     if (penum->can_cache <= 0 || !pgs->char_tm_valid) {
-        if_debug2('k', "[k]no cache: can_cache=%d, char_tm_valid=%d\n",
-                  penum->can_cache, (int)pgs->char_tm_valid);
+        if_debug2m('k', penum->memory, "[k]no cache: can_cache=%d, char_tm_valid=%d\n",
+                   penum->can_cache, (int)pgs->char_tm_valid);
         return 0;
     } {
         const gs_font *pfont = pgs->font;
@@ -525,7 +518,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         static const fixed max_cdim[3] =
         {
 #define max_cd(n)\
-            (fixed_1 << (arch_sizeof_short * 8 - n)) - (fixed_1 >> n) * 3
+            (fixed_1 << (ARCH_SIZEOF_SHORT * 8 - n)) - (fixed_1 >> n) * 3
             max_cd(0), max_cd(1), max_cd(2)
 #undef max_cd
         };
@@ -578,13 +571,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
             return code;
 #ifdef DEBUG
         if (gs_debug_c('k')) {
-            dlprintf6("[k]cbox=[%g %g %g %g] scale=%dx%d\n",
-                      fixed2float(cll.x), fixed2float(cll.y),
-                      fixed2float(cur.x), fixed2float(cur.y),
-                      1 << log2_scale.x, 1 << log2_scale.y);
-            dlprintf6("[p]  ctm=[%g %g %g %g %g %g]\n",
-                      pgs->ctm.xx, pgs->ctm.xy, pgs->ctm.yx, pgs->ctm.yy,
-                      pgs->ctm.tx, pgs->ctm.ty);
+            dmlprintf6(pgs->memory, "[k]cbox=[%g %g %g %g] scale=%dx%d\n",
+                       fixed2float(cll.x), fixed2float(cll.y),
+                       fixed2float(cur.x), fixed2float(cur.y),
+                       1 << log2_scale.x, 1 << log2_scale.y);
+            dmlprintf6(pgs->memory, "[p]  ctm=[%g %g %g %g %g %g]\n",
+                       pgs->ctm.xx, pgs->ctm.xy, pgs->ctm.yx, pgs->ctm.yy,
+                       pgs->ctm.tx, pgs->ctm.ty);
         }
 #endif
         cdim.x = cur.x - cll.x;
@@ -595,9 +588,9 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
             return 0;           /* much too big */
         iwidth = ((ushort) fixed2int_var(cdim.x) + 3) << log2_scale.x;
         iheight = ((ushort) fixed2int_var(cdim.y) + 3) << log2_scale.y;
-        if_debug3('k', "[k]iwidth=%u iheight=%u dev_cache %s\n",
-                  (uint) iwidth, (uint) iheight,
-                  (penum->dev_cache == 0 ? "not set" : "set"));
+        if_debug3m('k', penum->memory, "[k]iwidth=%u iheight=%u dev_cache %s\n",
+                   (uint) iwidth, (uint) iheight,
+                   (penum->dev_cache == 0 ? "not set" : "set"));
         if (penum->dev_cache == 0) {
             code = show_cache_setup(penum);
             if (code < 0)
@@ -611,7 +604,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
          * full oversampling with compression at the end.
          */
         code = gx_alloc_char_bits(dir, penum->dev_cache,
-                                (iwidth > MAX_TEMP_BITMAP_BITS / iheight &&
+                                (iwidth > MAX_CCACHE_TEMP_BITMAP_BITS / iheight &&
                                  log2_scale.x + log2_scale.y > alpha_bits ?
                                  penum->dev_cache2 : NULL),
                                 iwidth, iheight, &log2_scale, depth, &cc);
@@ -648,10 +641,10 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         /* Truncate the offsets to avoid artifacts later. */
         cc->offset.x = fixed_ceiling(-cll.x) + fixed_1;
         cc->offset.y = fixed_ceiling(-cll.y) + fixed_1;
-        if_debug4('k', "[k]width=%u, height=%u, offset=[%g %g]\n",
-                  (uint) iwidth, (uint) iheight,
-                  fixed2float(cc->offset.x),
-                  fixed2float(cc->offset.y));
+        if_debug4m('k', penum->memory, "[k]width=%u, height=%u, offset=[%g %g]\n",
+                   (uint) iwidth, (uint) iheight,
+                   fixed2float(cc->offset.x),
+                   fixed2float(cc->offset.y));
         pgs->in_cachedevice = CACHE_DEVICE_NONE; /* Provide correct grestore */
         if ((code = gs_gsave(pgs)) < 0) {
             gx_free_cached_char(dir, cc);
@@ -689,9 +682,13 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
         clip_box.q.y = int2fixed(iheight);
         if ((code = gx_clip_to_rectangle(pgs, &clip_box)) < 0)
             return code;
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
-        gx_set_device_color_1(pgs);     /* write 1's */
+        code = gx_set_device_color_1(pgs);     /* write 1's */
+        if (code < 0)
+            return code;
         gs_swapcolors_quick(pgs);
         pgs->in_cachedevice = CACHE_DEVICE_CACHING;
     }
@@ -701,7 +698,7 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 
 /* Return the cache device status. */
 gs_in_cache_device_t
-gs_incachedevice(const gs_state *pgs)
+gs_incachedevice(const gs_gstate *pgs)
 {
     return pgs->in_cachedevice;
 }
@@ -776,7 +773,7 @@ continue_show(gs_show_enum * penum)
 static int
 continue_kshow(gs_show_enum * penum)
 {   int code;
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
 
     if (pgs->font != penum->orig_font)
         gs_setfont(pgs, penum->orig_font);
@@ -792,7 +789,7 @@ continue_kshow(gs_show_enum * penum)
 static int
 show_update(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     cached_char *cc = penum->cc;
     int code;
 
@@ -843,7 +840,7 @@ show_update(gs_show_enum * penum)
             code = gs_grestore(pgs);
             if (code < 0)
                 return code;
-            code = gs_state_color_load(pgs);
+            code = gs_gstate_color_load(pgs);
             if (code < 0)
                 return code;
             return gx_image_cached_char(penum, cc);
@@ -863,9 +860,9 @@ show_update(gs_show_enum * penum)
 
 /* Move to next character */
 static inline int
-show_fast_move(gs_state * pgs, gs_fixed_point * pwxy)
+show_fast_move(gs_gstate * pgs, gs_fixed_point * pwxy)
 {
-    return gs_moveto_aux((gs_imager_state *)pgs, pgs->path,
+    return gs_moveto_aux(pgs, pgs->path,
                               pgs->current_point.x + fixed2float(pwxy->x),
                               pgs->current_point.y + fixed2float(pwxy->y));
 }
@@ -901,7 +898,7 @@ int gx_current_char(const gs_text_enum_t * pte)
 static int
 show_move(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     int code;
 
     if (SHOW_IS(penum, TEXT_REPLACE_WIDTHS)) {
@@ -921,7 +918,7 @@ show_move(gs_show_enum * penum)
          */
         if (SHOW_IS_ADD_TO_SPACE(penum)
             && (!penum->single_byte_space
-            || penum->fstack.depth <= 0)) {
+            || penum->bytes_decoded == 1)) {
             gs_char chr = gx_current_char((const gs_text_enum_t *)penum);
 
             if (chr == penum->text.space.s_char) {
@@ -953,7 +950,7 @@ show_move(gs_show_enum * penum)
         int code;
 
         if (penum->use_wxy_float)
-            code = gs_moveto_aux((gs_imager_state *)pgs, pgs->path,
+            code = gs_moveto_aux(pgs, pgs->path,
                     pgs->current_point.x + penum->wxy_float.x + fixed2float(penum->wxy.x),
                     pgs->current_point.y + penum->wxy_float.y + fixed2float(penum->wxy.y));
         else
@@ -972,7 +969,7 @@ show_move(gs_show_enum * penum)
 static int
 show_proceed(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     gs_font *pfont;
     cached_fm_pair *pair = 0;
     gs_font *rfont =
@@ -984,12 +981,12 @@ show_proceed(gs_show_enum * penum)
   (++(penum->xy_index), next_char_glyph(pte, pchr, pglyph))
     gs_char chr;
     gs_glyph glyph;
-    int code;
+    int code, start;
     cached_char *cc;
     gs_log2_scale_point log2_scale;
 
     if (penum->charpath_flag == cpm_show && SHOW_USES_OUTLINE(penum)) {
-        code = gs_state_color_load(pgs);
+        code = gs_gstate_color_load(pgs);
         if (code < 0)
             return code;
     }
@@ -1002,6 +999,7 @@ show_proceed(gs_show_enum * penum)
     if (penum->can_cache >= 0) {
         /* Loop with cache */
         for (;;) {
+            start = penum->index;
             switch ((code = get_next_char_glyph((gs_text_enum_t *)penum,
                                                 &chr, &glyph))
                     ) {
@@ -1029,12 +1027,13 @@ show_proceed(gs_show_enum * penum)
                      * Store glyph now, because pdfwrite needs it while
                      * synthezising bitmap fonts (see assign_char_code).
                      */
-                    if (glyph == gs_no_glyph) {
+                    if (glyph == GS_NO_GLYPH) {
                         glyph = (*penum->encode_char)(pfont, chr,
                                                       GLYPH_SPACE_NAME);
                         SET_CURRENT_GLYPH(penum, glyph);
                     } else
                         SET_CURRENT_GLYPH(penum, glyph);
+                    penum->bytes_decoded = penum->index - start;
                     penum->is_pure_color = gs_color_writes_pure(penum->pgs); /* Save
                                  this data for compute_glyph_raster_params to work
                                  independently on the color change in BuildChar.
@@ -1056,7 +1055,7 @@ show_proceed(gs_show_enum * penum)
                                 return code;
                         }
                         penum->pair = pair;
-                        if (glyph == gs_no_glyph) {
+                        if (glyph == GS_NO_GLYPH) {
                             cc = 0;
                             goto no_cache;
                         }
@@ -1159,7 +1158,7 @@ show_proceed(gs_show_enum * penum)
                 }
         }
         SET_CURRENT_CHAR(penum, chr);
-        if (glyph == gs_no_glyph) {
+        if (glyph == GS_NO_GLYPH) {
             glyph = (*penum->encode_char)(pfont, chr, GLYPH_SPACE_NAME);
         }
         SET_CURRENT_GLYPH(penum, glyph);
@@ -1298,7 +1297,8 @@ gx_show_text_retry(gs_text_enum_t *pte)
 static int
 show_finish(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
+    int code = 0, rcode;
 
     if ((penum->text.operation & TEXT_DO_FALSE_CHARPATH) ||
         (penum->text.operation & TEXT_DO_TRUE_CHARPATH)) {
@@ -1308,17 +1308,16 @@ show_finish(gs_show_enum * penum)
     if (penum->auto_release)
         penum->procs->release((gs_text_enum_t *)penum, "show_finish");
 
-    if((penum->text.operation &
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) ==
-         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
-        /* Save the accumulated width before returning, */
-        /* and undo the extra gsave. */
-        int code = gs_currentpoint(pgs, &penum->returned.total_width);
-        int rcode = gs_grestore(pgs);
+    if (!SHOW_IS_STRINGWIDTH(penum))
+       return 0;
 
-        return (code < 0 ? code : rcode);
-    }
-    return 0;
+    /* Save the accumulated width before returning, if we are not in PDF text rendering mode 3, */
+    /* and undo the extra gsave. */
+    if (!(penum->text.operation & TEXT_RENDER_MODE_3))
+        code = gs_currentpoint(pgs, &penum->returned.total_width);
+    rcode = gs_grestore(pgs);
+
+    return (code < 0 ? code : rcode);
 }
 
 /* Release the structure. */
@@ -1397,7 +1396,7 @@ gs_show_current_font(const gs_show_enum * penum)
 static int
 show_state_setup(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     gx_clip_path *pcpath;
     gs_font *pfont;
 
@@ -1412,7 +1411,7 @@ show_state_setup(gs_show_enum * penum)
                                 penum->text.data.d_glyph, NULL, &fidx);
             if (code < 0) { /* failed to load glyph data, reload glyph for CID 0 */
                code = ((gs_font_cid0 *)pfont)->cidata.glyph_data((gs_font_base *)pfont,
-                            (gs_glyph)(gs_min_cid_glyph + 0), NULL, &fidx);
+                            (gs_glyph)(GS_MIN_CID_GLYPH + 0), NULL, &fidx);
                if (code < 0)
                    return_error(gs_error_invalidfont);
             }
@@ -1466,7 +1465,7 @@ show_state_setup(gs_show_enum * penum)
             double fdx = pgs->char_tm.tx - pgs->ctm.tx;
             double fdy = pgs->char_tm.ty - pgs->ctm.ty;
 
-#define int_bits (arch_sizeof_int * 8 - 1)
+#define int_bits (ARCH_SIZEOF_INT * 8 - 1)
             if (!(f_fits_in_bits(fdx, int_bits) &&
                   f_fits_in_bits(fdy, int_bits))
                 )
@@ -1488,9 +1487,15 @@ show_set_scale(const gs_show_enum * penum, gs_log2_scale_point *log2_scale)
      * Decide whether to oversample.
      * We have to decide this each time setcachedevice is called.
      */
-    const gs_state *pgs = penum->pgs;
+    const gs_gstate *pgs = NULL;
 
-    if ((penum->charpath_flag == cpm_show ||
+    if (gs_object_type(penum->pgs->memory, penum) == &st_gs_show_enum) {
+        pgs = penum->pgs;
+    } else {
+            pgs = (gs_gstate *)penum->pgs;
+    }
+
+    if (pgs != NULL && (penum->charpath_flag == cpm_show ||
          penum->charpath_flag == cpm_charwidth) &&
         SHOW_USES_OUTLINE(penum)
         /* && gx_path_is_void_inline(pgs->path) */
@@ -1533,7 +1538,7 @@ show_set_scale(const gs_show_enum * penum, gs_log2_scale_point *log2_scale)
 static int
 show_cache_setup(gs_show_enum * penum)
 {
-    gs_state *pgs = penum->pgs;
+    gs_gstate *pgs = penum->pgs;
     gs_memory_t *mem = penum->memory;
     gx_device_memory *dev =
         gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
@@ -1569,7 +1574,7 @@ show_cache_setup(gs_show_enum * penum)
 /* Used before rendering characters, and for moving the origin */
 /* in setcachedevice2 when WMode=1. */
 static int
-show_origin_setup(gs_state * pgs, fixed cpt_x, fixed cpt_y, gs_show_enum * penum)
+show_origin_setup(gs_gstate * pgs, fixed cpt_x, fixed cpt_y, gs_show_enum * penum)
 {
     if (penum->charpath_flag == cpm_show) {
         /* Round the translation in the graphics state. */
