@@ -411,7 +411,7 @@ clist_delete_tile(gx_device_clist_writer * cldev, tile_slot * slot)
     if_debug2m('L', cldev->memory, "[L]deleting index=%u, offset=%lu\n",
                index, (ulong) ((byte *) slot - cldev->data));
     gx_bits_cache_free(&cldev->bits, (gx_cached_bits_head *) slot,
-                       &cldev->chunk);
+                       cldev->cache_chunk);
     table[index].offset = 0;
     /* Delete the entry from the hash table. */
     /* We'd like to move up any later entries, so that we don't need */
@@ -429,7 +429,7 @@ clist_delete_tile(gx_device_clist_writer * cldev, tile_slot * slot)
                        index, offset);
             gx_bits_cache_free(&cldev->bits,
                              (gx_cached_bits_head *) (cldev->data + offset),
-                               &cldev->chunk);
+                               cldev->cache_chunk);
             table[index].offset = 0;
         }
     }
@@ -453,11 +453,11 @@ clist_add_tile(gx_device_clist_writer * cldev, const gx_strip_bitmap * tiles,
     if (cldev->bits.csize == cldev->tile_max_count) {	/* Don't let the hash table get too full: delete an entry. */
         /* Since gx_bits_cache_alloc returns an entry to delete when */
         /* it fails, just force it to fail. */
-        gx_bits_cache_alloc(&cldev->bits, (ulong) cldev->chunk.size,
+        gx_bits_cache_alloc(&cldev->bits, (ulong) cldev->cache_chunk->size,
                             &slot_head);
         if (slot_head == 0) {	/* Wrap around and retry. */
             cldev->bits.cnext = 0;
-            gx_bits_cache_alloc(&cldev->bits, (ulong) cldev->chunk.size,
+            gx_bits_cache_alloc(&cldev->bits, (ulong) cldev->cache_chunk->size,
                                 &slot_head);
 #ifdef DEBUG
             if (slot_head == 0) {
@@ -547,7 +547,7 @@ clist_new_tile_params(gx_strip_bitmap * new_tile, const gx_strip_bitmap * tiles,
     if (tiles->num_planes != 1)
         depth /= tiles->num_planes;
     rep_width_bits = rep_width * depth;
-    max_bytes = cldev->chunk.size / (rep_width_bits * rep_height);
+    max_bytes = cldev->cache_chunk->size / (rep_width_bits * rep_height);
 
     max_bytes -= min(max_bytes, tile_overhead);
     if (max_bytes > max_tile_bytes)
@@ -647,7 +647,7 @@ clist_change_tile(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                                  * compatible with those stored in the device
                                  * (cldev->tile_params).
                                  */
-                ulong offset = (byte *) loc.tile - cldev->chunk.data;
+                ulong offset = (byte *) loc.tile - cldev->cache_chunk->data;
                 uint rsize =
                     extra + 1 + cmd_size_w(loc.index) + cmd_size_w(offset);
                 byte *dp;
@@ -734,7 +734,7 @@ clist_change_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         } else {		/* Not known yet.  Output the bits. */
             /* Note that the offset we write is the one used by */
             /* the reading phase, not the writing phase. */
-            ulong offset = (byte *) loc.tile - cldev->chunk.data;
+            ulong offset = (byte *) loc.tile - cldev->cache_chunk->data;
             uint rsize = 2 + cmd_size_w(loc.tile->width) +
             cmd_size_w(loc.tile->height) + cmd_size_w(loc.index) +
             cmd_size_w(offset);
@@ -752,7 +752,7 @@ clist_change_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                                 loc.tile->width * pdepth,
                                 loc.tile->height * loc.tile->num_planes, loc.tile->cb_raster,
                                 rsize,
-                              decompress_elsewhere | (((gx_device_printer *)cldev->target)->BLS_force_memory ? (1 << cmd_compress_cfe) : 0),
+                              decompress_elsewhere | (cldev->target->BLS_force_memory ? (1 << cmd_compress_cfe) : 0),
                                 &dp, &csize);
 
             if (code < 0)
