@@ -33,7 +33,7 @@
 #include "gxdevsop.h"
 
 #ifdef PACIFY_VALGRIND
-#include <valgrind/helgrind.h>
+#include "valgrind.h"
 #endif
 
 extern dev_proc_open_device(pattern_clist_open_device);
@@ -46,15 +46,17 @@ extern dev_proc_open_device(pattern_clist_open_device);
     writer rather than the common.   fixme: Also, if icc_cache_cl is not
     included in the writer, 64bit builds will seg fault */
 
-extern_st(st_imager_state);
+extern_st(st_gs_gstate);
 static
 ENUM_PTRS_WITH(device_clist_enum_ptrs, gx_device_clist *cdev)
     if (index < st_device_forward_max_ptrs) {
-        gs_ptr_type_t ret = ENUM_USING_PREFIX(st_device_forward, 0);
+        gs_ptr_type_t ret = ENUM_USING_PREFIX(st_device_forward, st_device_max_ptrs);
 
         return (ret ? ret : ENUM_OBJ(0));
     }
     index -= st_device_forward_max_ptrs;
+    /* RJW: We do not enumerate icc_cache_cl or icc_cache_list as they
+     * are allocated in non gc space */
     if (CLIST_IS_WRITER(cdev)) {
         switch (index) {
         case 0: return ENUM_OBJ((cdev->writer.image_enum_id != gs_no_id ?
@@ -64,10 +66,9 @@ ENUM_PTRS_WITH(device_clist_enum_ptrs, gx_device_clist *cdev)
         case 2: return ENUM_OBJ(cdev->writer.pinst);
         case 3: return ENUM_OBJ(cdev->writer.cropping_stack);
         case 4: return ENUM_OBJ(cdev->writer.icc_table);
-        case 5: return ENUM_OBJ(cdev->writer.icc_cache_cl);
         default:
-        return ENUM_USING(st_imager_state, &cdev->writer.imager_state,
-                  sizeof(gs_imager_state), index - 5);
+        return ENUM_USING(st_gs_gstate, &cdev->writer.gs_gstate,
+                  sizeof(gs_gstate), index - 5);
         }
     }
     else {
@@ -79,13 +80,11 @@ ENUM_PTRS_WITH(device_clist_enum_ptrs, gx_device_clist *cdev)
          */
 
         if (index == 0)
-            return ENUM_OBJ(cdev->reader.band_complexity_array);
-        else if (index == 1)
             return ENUM_OBJ(cdev->reader.offset_map);
-        else if (index == 2)
+        else if (index == 1)
             return ENUM_OBJ(cdev->reader.icc_table);
-        else if (index == 3)
-            return ENUM_OBJ(cdev->reader.icc_cache_cl);
+        else if (index == 2)
+            return ENUM_OBJ(cdev->reader.color_usage_array);
         else
             return 0;
     }
@@ -102,18 +101,16 @@ RELOC_PTRS_WITH(device_clist_reloc_ptrs, gx_device_clist *cdev)
         RELOC_VAR(cdev->writer.pinst);
         RELOC_VAR(cdev->writer.cropping_stack);
         RELOC_VAR(cdev->writer.icc_table);
-        RELOC_VAR(cdev->writer.icc_cache_cl);
-        RELOC_USING(st_imager_state, &cdev->writer.imager_state,
-            sizeof(gs_imager_state));
+        RELOC_USING(st_gs_gstate, &cdev->writer.gs_gstate,
+            sizeof(gs_gstate));
     } else {
         /* 041207
          * clist is reader.
          * See note above in ENUM_PTRS_WITH section.
          */
-        RELOC_VAR(cdev->reader.band_complexity_array);
         RELOC_VAR(cdev->reader.offset_map);
         RELOC_VAR(cdev->reader.icc_table);
-        RELOC_VAR(cdev->reader.icc_cache_cl);
+        RELOC_VAR(cdev->reader.color_usage_array);
     }
 } RELOC_PTRS_END
 public_st_device_clist();
@@ -123,7 +120,7 @@ private_st_clist_icctable();
 
 /* Forward declarations of driver procedures */
 dev_proc_open_device(clist_open);
-static dev_proc_output_page(clist_output_page);
+dev_proc_output_page(clist_output_page);
 static dev_proc_close_device(clist_close);
 static dev_proc_get_band(clist_get_band);
 /* Driver procedures defined in other files are declared in gxcldev.h. */
@@ -205,6 +202,7 @@ const gx_device_procs gs_clist_device_procs = {
     clist_strip_copy_rop2,
     clist_strip_tile_rect_devn,
     clist_copy_alpha_hl_color,
+    clist_process_page,
 };
 
 /*------------------- Choose the implementation -----------------------
@@ -233,11 +231,6 @@ clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory)
 
 /* ------ Define the command set and syntax ------ */
 
-/* Initialization for imager state. */
-/* The initial scale is arbitrary. */
-const gs_imager_state clist_imager_state_initial =
-{gs_imager_state_initial(300.0 / 72.0, false)};
-
 /*
  * The buffer area (data, data_size) holds a bitmap cache when both writing
  * and reading.  The rest of the space is used for the command buffer and
@@ -311,8 +304,8 @@ clist_init_tile_cache(gx_device * dev, byte * init_data, ulong data_size)
     cdev->tile_table = (tile_hash *) data;
     data += hsize;
     bits_size -= hsize;
-    gx_bits_cache_chunk_init(&cdev->chunk, data, bits_size);
-    gx_bits_cache_init(&cdev->bits, &cdev->chunk);
+    gx_bits_cache_chunk_init(cdev->cache_chunk, data, bits_size);
+    gx_bits_cache_init(&cdev->bits, cdev->cache_chunk);
     return 0;
 }
 
@@ -416,7 +409,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     cdev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
 
     /* Call create_buf_device to get the memory planarity set up. */
-    code = cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
+    code = cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, NULL);
     if (code < 0)
         return code;
     /* HACK - if the buffer device can't do copy_alpha, disallow */
@@ -425,34 +418,50 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
         cdev->disable_mask |= clist_disable_copy_alpha;
     if (cdev->procs.open_device == pattern_clist_open_device) {
         bits_size = data_size / 2;
-    } else if (band_height) {
-        /*
-         * The band height is fixed, so the band buffer requirement
-         * is completely determined.
-         */
-        ulong band_data_size;
-
-        if (gdev_mem_data_size(&bdev, band_width, band_height, &band_data_size) < 0 ||
-            band_data_size >= band_space) {
-            if (pbdev->finalize)
-                pbdev->finalize(pbdev);
-            return_error(gs_error_rangecheck);
-        }
-        bits_size = min(band_space - band_data_size, data_size >> 1);
+        cdev->page_line_ptrs_offset = 0;
     } else {
-        /*
-         * Choose the largest band height that will fit in the
-         * rendering-time buffer.
-         */
-        bits_size = clist_tile_cache_size(target, band_space);
-        bits_size = min(bits_size, data_size >> 1);
-        band_height = gdev_mem_max_height(&bdev, band_width,
-                          band_space - bits_size, page_uses_transparency);
-        if (band_height == 0) {
-            if (pbdev->finalize)
-                pbdev->finalize(pbdev);
-            return_error(gs_error_rangecheck);
+        if (band_height) {
+            /*
+             * The band height is fixed, so the band buffer requirement
+             * is completely determined.
+             */
+            ulong band_data_size;
+            int adjusted;
+
+            adjusted = (dev_proc(dev, dev_spec_op)(dev, gxdso_adjust_bandheight, NULL, band_height));
+            if (adjusted > 0)
+                band_height = adjusted;
+
+            if (gdev_mem_data_size(&bdev, band_width, band_height, &band_data_size) < 0 ||
+                band_data_size >= band_space) {
+                if (pbdev->finalize)
+                    pbdev->finalize(pbdev);
+                return_error(gs_error_rangecheck);
+            }
+            bits_size = min(band_space - band_data_size, data_size >> 1);
+        } else {
+            int adjusted;
+            /*
+             * Choose the largest band height that will fit in the
+             * rendering-time buffer.
+             */
+            bits_size = clist_tile_cache_size(target, band_space);
+            bits_size = min(bits_size, data_size >> 1);
+            band_height = gdev_mem_max_height(&bdev, band_width,
+                              band_space - bits_size, page_uses_transparency);
+            if (band_height == 0) {
+                if (pbdev->finalize)
+                    pbdev->finalize(pbdev);
+                return_error(gs_error_rangecheck);
+            }
+            adjusted = (dev_proc(dev, dev_spec_op)(dev, gxdso_adjust_bandheight, NULL, band_height));
+            if (adjusted > 0)
+                band_height = adjusted;
         }
+        /* The above calculated bits_size's include space for line ptrs. What is
+         * the offset for the line_ptrs within the buffer? */
+        if (gdev_mem_bits_size(&bdev, band_width, band_height, &cdev->page_line_ptrs_offset) < 0)
+            return_error(gs_error_VMerror);
     }
     cdev->ins_count = 0;
     code = clist_init_tile_cache(dev, data, bits_size);
@@ -506,8 +515,7 @@ clist_reset(gx_device * dev)
         gx_clist_state *states = cdev->states;
 
         for (band = 0; band < nbands; band++, states++) {
-            static const gx_clist_state cls_initial =
-            {cls_initial_values};
+            static const gx_clist_state cls_initial = { cls_initial_values };
 
             *states = cls_initial;
         }
@@ -527,7 +535,7 @@ clist_reset(gx_device * dev)
     cdev->tile_depth = 0;
     cdev->tile_known_min = nbands;
     cdev->tile_known_max = -1;
-    cdev->imager_state = clist_imager_state_initial;
+    GS_STATE_INIT_VALUES_CLIST((&cdev->gs_gstate));
     cdev->clip_path = NULL;
     cdev->clip_path_id = gs_no_id;
     cdev->color_space.byte1 = 0;
@@ -566,8 +574,6 @@ clist_init(gx_device * dev)
 
     if (code >= 0) {
         cdev->image_enum_id = gs_no_id;
-        cdev->error_is_retryable = 0;
-        cdev->driver_call_nesting = 0;
         cdev->ignore_lo_mem_warnings = 0;
     }
     return code;
@@ -591,14 +597,6 @@ clist_reinit_output_file(gx_device *dev)
     int c_block =
         cdev->cend - cdev->cbuf + 2 + cdev->nbands * 2 + (cdev->nbands + 1);
 
-    /* All this is for partial page rendering's benefit, do only */
-    /* if partial page rendering is available */
-    if ( clist_test_VMerror_recoverable(cdev) )
-        { if (cdev->page_bfile != 0)
-            code = cdev->page_info.io_procs->set_memory_warning(cdev->page_bfile, b_block);
-        if (code >= 0 && cdev->page_cfile != 0)
-            code = cdev->page_info.io_procs->set_memory_warning(cdev->page_cfile, c_block);
-        }
     return code;
 }
 
@@ -612,13 +610,8 @@ clist_emit_page_header(gx_device *dev)
     int code = 0;
 
     if ((cdev->disable_mask & clist_disable_pass_thru_params)) {
-        do
-            if ((code = clist_put_current_params(cdev)) >= 0)
-                break;
-        while ((code = clist_VMerror_recover(cdev, code)) >= 0);
+        code = clist_put_current_params(cdev);
         cdev->permanent_error = (code < 0 ? code : 0);
-        if (cdev->permanent_error < 0)
-            cdev->error_is_retryable = 0;
     }
     return code;
 }
@@ -628,10 +621,6 @@ static void
 clist_reset_page(gx_device_clist_writer *cwdev)
 {
     cwdev->page_bfile_end_pos = 0;
-    /* Indicate that the colors_used information hasn't been computed. */
-    cwdev->page_info.scan_lines_per_colors_used = 0;
-    memset(cwdev->page_info.band_color_usage, 0,
-           sizeof(cwdev->page_info.band_color_usage));
 }
 
 /* Open the device's bandfiles */
@@ -665,7 +654,6 @@ clist_open_output_file(gx_device *dev)
         ) {
         clist_close_output_file(dev);
         cdev->permanent_error = code;
-        cdev->error_is_retryable = 0;
     }
     return code;
 }
@@ -678,10 +666,12 @@ clist_close_page_info(gx_band_page_info_t *ppi)
     if (ppi->cfile != NULL) {
         ppi->io_procs->fclose(ppi->cfile, ppi->cfname, true);
         ppi->cfile = NULL;
+        ppi->cfname[0] = 0;     /* prevent re-use in case this is a fake path */
     }
     if (ppi->bfile != NULL) {
         ppi->io_procs->fclose(ppi->bfile, ppi->bfname, true);
         ppi->bfile = NULL;
+        ppi->bfname[0] = 0;     /* prevent re-use in case this is a fake path */
     }
     return 0;
 }
@@ -697,20 +687,6 @@ clist_close_output_file(gx_device *dev)
     return clist_close_page_info(&cdev->page_info);
 }
 
-static void
-clist_set_planar(gx_device *dev)
-{
-    gx_device_clist_common * cdev = &((gx_device_clist *)dev)->common;
-    int ret;
-
-    ret = dev_proc(dev, dev_spec_op)(dev, gxdso_is_native_planar, NULL, 0);
-    if (ret > 0) {
-        cdev->is_planar = ret;
-    } else {
-        cdev->is_planar = 0;
-    }
-}
-
 /* Open the device by initializing the device state and opening the */
 /* scratch files. */
 int
@@ -723,24 +699,45 @@ clist_open(gx_device *dev)
 
     cdev->permanent_error = 0;
     cdev->is_open = false;
-    clist_set_planar(dev);
+
+    cdev->cache_chunk = (gx_bits_cache_chunk *)gs_alloc_bytes(cdev->memory->non_gc_memory, sizeof(gx_bits_cache_chunk), "alloc tile cache for clist");
+    if (!cdev->cache_chunk)
+        return_error(gs_error_VMerror);
+    memset(cdev->cache_chunk, 0x00, sizeof(gx_bits_cache_chunk));
+
     code = clist_init(dev);
     if (code < 0)
         return code;
+    cdev->icc_cache_list_len = 0;
+    cdev->icc_cache_list = NULL;
     code = clist_open_output_file(dev);
     if ( code >= 0)
         code = clist_emit_page_header(dev);
     if (code >= 0)
        dev->is_open = save_is_open;
-     return code;
+
+    return code;
 }
 
 static int
 clist_close(gx_device *dev)
 {
+    int i;
     gx_device_clist_writer * const cdev =
         &((gx_device_clist *)dev)->writer;
 
+    /* I'd like to free the cache chunk in here, but we can't, because the pattern clist
+     * device gets closed, but not discarded, later it gets run. So we have to free the memory
+     * in *2* places, once in gdev_prn_tear_down() for regular clists, and once in
+     * gx_pattern_cache_free_entry() for pattern clists....
+     */
+    for(i = 0; i < cdev->icc_cache_list_len; i++) {
+        rc_decrement(cdev->icc_cache_list[i], "clist_close");
+    }
+    cdev->icc_cache_list_len = 0;
+    gs_free_object(cdev->memory->thread_safe_memory, cdev->icc_cache_list, "clist_close");
+    cdev->icc_cache_list = NULL;
+
     if (cdev->do_not_open_or_close_bandfiles)
         return 0;
     if (cdev->procs.open_device == pattern_clist_open_device) {
@@ -751,7 +748,7 @@ clist_close(gx_device *dev)
 }
 
 /* The output_page procedure should never be called! */
-static int
+int
 clist_output_page(gx_device * dev, int num_copies, int flush)
 {
     return_error(gs_error_Fatal);
@@ -761,34 +758,37 @@ clist_output_page(gx_device * dev, int num_copies, int flush)
 int
 clist_finish_page(gx_device *dev, bool flush)
 {
-    gx_device_clist_writer * const cdev =       &((gx_device_clist *)dev)->writer;
+    gx_device_clist_writer *const cdev = &((gx_device_clist *)dev)->writer;
     int code;
 
     /* If this is a reader clist, which is about to be reset to a writer,
-     * free any band_complexity_array memory used by same.
+     * free any color_usage array used by same.
      * since we have been rendering, shut down threads
+     * Also free the icc_table at this time and the icc_cache
      */
     if (!CLIST_IS_WRITER((gx_device_clist *)dev)) {
-        gx_clist_reader_free_band_complexity_array( (gx_device_clist *)dev );
+        gx_device_clist_reader * const crdev =  &((gx_device_clist *)dev)->reader;
+
+        gs_free_object(cdev->memory, crdev->color_usage_array, "clist_color_usage_array");
+        crdev->color_usage_array = NULL;
         clist_teardown_render_threads(dev);
-    }
 
-    /* Also free the icc_table at this time and the icc_cache */
-    if (!CLIST_IS_WRITER((gx_device_clist *)dev)) {
        /* Free the icc table associated with this device.
            The threads that may have pointed to this were destroyed in
            the above call to clist_teardown_render_threads.  Since they
            all maintained a copy of the cache and the table there should not
            be any issues. */
-        gx_device_clist_reader * const crdev =  &((gx_device_clist *)dev)->reader;
-        clist_icc_freetable(crdev->icc_table, crdev->memory);
+        clist_free_icc_table(crdev->icc_table, crdev->memory);
+        crdev->icc_table = NULL;
         rc_decrement(crdev->icc_cache_cl,"clist_finish_page");
+        crdev->icc_cache_cl = NULL;
     }
     if (flush) {
         if (cdev->page_cfile != 0)
             cdev->page_info.io_procs->rewind(cdev->page_cfile, true, cdev->page_cfname);
         if (cdev->page_bfile != 0)
             cdev->page_info.io_procs->rewind(cdev->page_bfile, true, cdev->page_bfname);
+        cdev->page_info.bfile_end_pos = 0;
         clist_reset_page(cdev);
     } else {
         if (cdev->page_cfile != 0)
@@ -822,9 +822,11 @@ clist_end_page(gx_device_clist_writer * cldev)
         /* Save the table */
         code = clist_icc_writetable(cldev);
         /* Free the table */
-        clist_icc_freetable(cldev->icc_table, cldev->memory);
+        clist_free_icc_table(cldev->icc_table, cldev->memory);
         cldev->icc_table = NULL;
     }
+    if (code >= 0)
+        code = clist_write_color_usage_array(cldev);
     if (code >= 0) {
         /*
          * Write the terminating entry in the block file.
@@ -838,7 +840,6 @@ clist_end_page(gx_device_clist_writer * cldev)
             code = 0;
     }
     if (code >= 0) {
-        clist_compute_color_usage(cldev);
         ecode |= code;
         cldev->page_bfile_end_pos = cldev->page_info.io_procs->ftell(cldev->page_bfile);
     }
@@ -862,6 +863,19 @@ clist_end_page(gx_device_clist_writer * cldev)
                 (unsigned long)cldev->page_bfile_end_pos);
     }
 #endif
+    if (cldev->page_uses_transparency && gs_debug[':']) {
+        /* count how many bands were skipped */
+        int skip_count = 0;
+        int band;
+
+        for (band=0; band < cldev->nbands - 1; band++) {
+            if (cldev->states[band].color_usage.trans_bbox.p.y >
+                cldev->states[band].color_usage.trans_bbox.q.y)
+                skip_count++;
+        }
+        dprintf2("%d bands skipped out of %d\n", skip_count, cldev->nbands);
+    }
+
     return 0;
 }
 
@@ -869,7 +883,7 @@ gx_color_usage_bits
 gx_color_index2usage(gx_device *dev, gx_color_index color)
 {
     gx_color_usage_bits bits = 0;
-    int i;
+    uchar i;
 
     for (i = 0; i < dev->color_info.num_components; i++) {
         if (color & dev->color_info.comp_mask[i])
@@ -879,109 +893,6 @@ gx_color_index2usage(gx_device *dev, gx_color_index color)
     return bits;
 }
 
-/* Compute the set of used colors in the page_info structure.
- *
- * NB: Area for improvement, move states[band] and page_info to clist
- * rather than writer device, or remove completely as this is used by the old planar devices
- * to operate on a plane at a time.
- */
-
-void
-clist_compute_color_usage(gx_device_clist_writer *cldev)
-{
-    int nbands = cldev->nbands;
-    int bands_per_colors_used =
-        (nbands + PAGE_INFO_NUM_COLORS_USED - 1) /
-        PAGE_INFO_NUM_COLORS_USED;
-    int band;
-
-    cldev->page_info.scan_lines_per_colors_used =
-        cldev->page_band_height * bands_per_colors_used;
-    memset(cldev->page_info.band_color_usage, 0,
-           sizeof(cldev->page_info.band_color_usage));
-    for (band = 0; band < nbands; ++band) {
-        int entry = band / bands_per_colors_used;
-
-        cldev->page_info.band_color_usage[entry].or |=
-            cldev->states[band].color_usage.or;
-        cldev->page_info.band_color_usage[entry].slow_rop |=
-            cldev->states[band].color_usage.slow_rop;
-
-    }
-}
-
-/* Recover recoverable VM error if possible without flushing */
-int     /* ret -ve err, >= 0 if recovered w/# = cnt pages left in page queue */
-clist_VMerror_recover(gx_device_clist_writer *cldev,
-                      int old_error_code)
-{
-    int code = old_error_code;
-    int pages_remain;
-
-    if (!clist_test_VMerror_recoverable(cldev) ||
-        !cldev->error_is_retryable ||
-        old_error_code != gs_error_VMerror
-        )
-        return old_error_code;
-
-    /* Do some rendering, return if enough memory is now free */
-    do {
-        pages_remain =
-            (*cldev->free_up_bandlist_memory)( (gx_device *)cldev, false );
-        if (pages_remain < 0) {
-            code = pages_remain;        /* abort, error or interrupt req */
-            break;
-        }
-        if (clist_reinit_output_file( (gx_device *)cldev ) == 0) {
-            code = pages_remain;        /* got enough memory to continue */
-            break;
-        }
-    } while (pages_remain);
-
-    if_debug1m('L', cldev->memory, "[L]soft flush of command list, status: %d\n", code);
-    return code;
-}
-
-/* If recoverable VM error, flush & try to recover it */
-int     /* ret 0 ok, else -ve error */
-clist_VMerror_recover_flush(gx_device_clist_writer *cldev,
-                            int old_error_code)
-{
-    int free_code = 0;
-    int reset_code = 0;
-    int code;
-
-    /* If the device has the ability to render partial pages, flush
-     * out the bandlist, and reset the writing state. Then, get the
-     * device to render this band. When done, see if there's now enough
-     * memory to satisfy the minimum low-memory guarantees. If not,
-     * get the device to render some more. If there's nothing left to
-     * render & still insufficient memory, declare an error condition.
-     */
-    if (!clist_test_VMerror_recoverable(cldev) ||
-        old_error_code != gs_error_VMerror
-        )
-        return old_error_code;  /* sorry, don't have any means to recover this error */
-    free_code = (*cldev->free_up_bandlist_memory)( (gx_device *)cldev, true );
-
-    /* Reset the state of bands to "don't know anything" */
-    reset_code = clist_reset( (gx_device *)cldev );
-    if (reset_code >= 0)
-        reset_code = clist_open_output_file( (gx_device *)cldev );
-    if ( reset_code >= 0 &&
-         (cldev->disable_mask & clist_disable_pass_thru_params)
-         )
-        reset_code = clist_put_current_params(cldev);
-    if (reset_code < 0) {
-        cldev->permanent_error = reset_code;
-        cldev->error_is_retryable = 0;
-    }
-
-    code = (reset_code < 0 ? reset_code : free_code < 0 ? old_error_code : 0);
-    if_debug1m('L', cldev->memory, "[L]hard flush of command list, status: %d\n", code);
-    return code;
-}
-
 /* Write the target device's current parameter list */
 static int      /* ret 0 all ok, -ve error */
 clist_put_current_params(gx_device_clist_writer *cldev)
@@ -1028,27 +939,6 @@ clist_get_band(gx_device * dev, int y, int *band_start)
     return min(dev->height - start, band_height);
 }
 
-/* copy constructor if from != NULL
- * default constructor if from == NULL
- */
-void
-clist_copy_band_complexity(gx_band_complexity_t *self, const gx_band_complexity_t *from)
-{
-    if (from) {
-        memcpy(self, from, sizeof(gx_band_complexity_t));
-    } else {
-        /* default */
-        self->uses_color = false;
-        self->nontrivial_rops = false;
-#if 0
-        /* todo: halftone phase */
-
-        self->x0 = 0;
-        self->y0 = 0;
-#endif
-    }
-}
-
 /* ICC table operations.  See gxclist.h for details */
 /* This checks the table for a hash code entry */
 bool
@@ -1069,24 +959,39 @@ clist_icc_searchtable(gx_device_clist_writer *cdev, int64_t hashcode)
      return(false);  /* No entry */
 }
 
-/* Free the table */
-int
-clist_icc_freetable(clist_icctable_t *icc_table, gs_memory_t *memory)
+static void
+clist_free_icc_table_contents(clist_icctable_t *icc_table)
 {
     int number_entries;
     clist_icctable_entry_t *curr_entry, *next_entry;
     int k;
 
-    if (icc_table == NULL)
-        return(0);
     number_entries = icc_table->tablesize;
     curr_entry = icc_table->head;
     for (k = 0; k < number_entries; k++) {
         next_entry = curr_entry->next;
-        gs_free_object(icc_table->memory, curr_entry, "clist_icc_freetable");
+        rc_decrement(curr_entry->icc_profile, "clist_free_icc_table");
+        gs_free_object(icc_table->memory, curr_entry, "clist_free_icc_table");
         curr_entry = next_entry;
     }
-    gs_free_object(icc_table->memory, icc_table, "clist_icc_freetable");
+}
+
+void
+clist_icc_table_finalize(const gs_memory_t *memory, void * vptr)
+{
+    clist_icctable_t *icc_table = (clist_icctable_t *)vptr;
+
+    clist_free_icc_table_contents(icc_table);
+}
+
+/* Free the table */
+int
+clist_free_icc_table(clist_icctable_t *icc_table, gs_memory_t *memory)
+{
+    if (icc_table == NULL)
+        return(0);
+
+    gs_free_object(icc_table->memory, icc_table, "clist_free_icc_table");
     return(0);
 }
 
@@ -1132,7 +1037,7 @@ clist_icc_writetable(gx_device_clist_writer *cldev)
         curr_entry = curr_entry->next;
     }
     /* Now go ahead and save the table data */
-    cmd_write_icctable(cldev, buf, size_data);
+    cmd_write_pseudo_band(cldev, buf, size_data, ICC_TABLE_OFFSET);
     gs_free_object(cldev->memory, buf, "clist_icc_writetable");
     return(0);
 }
@@ -1189,7 +1094,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
     }
     if ( icc_table == NULL ) {
         entry = (clist_icctable_entry_t *) gs_alloc_struct(stable_mem,
-                               clist_icctable_entry_t, &st_clist_icctable_entry, 
+                               clist_icctable_entry_t, &st_clist_icctable_entry,
                                "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
@@ -1200,7 +1105,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->icc_profile = icc_profile;
         entry->render_is_valid = icc_profile->rend_is_valid;
         rc_increment(icc_profile);
-        icc_table = gs_alloc_struct(stable_mem, clist_icctable_t, 
+        icc_table = gs_alloc_struct(stable_mem, clist_icctable_t,
                                     &st_clist_icctable, "clist_icc_addentry");
         if (icc_table == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate icc table");
@@ -1221,10 +1126,10 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
             curr_entry = curr_entry->next;
         }
          /* Add a new ICC profile */
-        entry = 
-            (clist_icctable_entry_t *) gs_alloc_struct(icc_table->memory, 
+        entry =
+            (clist_icctable_entry_t *) gs_alloc_struct(icc_table->memory,
                                                        clist_icctable_entry_t,
-                                                       &st_clist_icctable_entry, 
+                                                       &st_clist_icctable_entry,
                                                        "clist_icc_addentry");
         if (entry == NULL)
             return gs_rethrow(-1, "insufficient memory to allocate entry in icc table");
@@ -1242,6 +1147,51 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
     return(0);
 }
 
+/* This writes out the color_usage_array for maxband+1 */
+int
+clist_write_color_usage_array(gx_device_clist_writer *cldev)
+{
+   gx_color_usage_t *color_usage_array;
+   int i, size_data = cldev->nbands * sizeof(gx_color_usage_t);
+
+    /* Now serialize the table data */
+    color_usage_array = (gx_color_usage_t *)gs_alloc_bytes(cldev->memory, size_data,
+                                       "clist_write_color_usage_array");
+    if (color_usage_array == NULL)
+        return gs_rethrow(-1, "insufficient memory for color_usage_array");
+    for (i = 0; i < cldev->nbands; i++) {
+        memcpy(&(color_usage_array[i]), &(cldev->states[i].color_usage), sizeof(gx_color_usage_t));
+    }
+    /* Now go ahead and save the table data */
+    cmd_write_pseudo_band(cldev, (unsigned char *)color_usage_array,
+                          size_data, COLOR_USAGE_OFFSET);
+    gs_free_object(cldev->memory, color_usage_array, "clist_write_color_usage_array");
+    return(0);
+}
+
+/* Compute color_usage over a Y range while writing clist */
+/* Sets color_usage fields and range_start.               */
+/* Returns range end (max dev->height)                    */
+/* NOT expected to be used. */
+int
+clist_writer_color_usage(gx_device_clist_writer *cldev, int y, int height,
+                     gx_color_usage_t *color_usage, int *range_start)
+{
+        gx_color_usage_bits or = 0;
+        bool slow_rop = false;
+        int i, band_height = cldev->page_band_height;
+        int start = y / band_height, end = (y + height) / band_height;
+
+        for (i = start; i < end; ++i) {
+            or |= cldev->states[i].color_usage.or;
+            slow_rop |= cldev->states[i].color_usage.slow_rop;
+        }
+        color_usage->or = or;
+        color_usage->slow_rop = slow_rop;
+        *range_start = start * band_height;
+        return min(end * band_height, cldev->height) - *range_start;
+}
+
 int
 clist_writer_push_no_cropping(gx_device_clist_writer *cdev)
 {
@@ -1356,3 +1306,85 @@ clist_put_data(const gx_device_clist *cdev, int select, int64_t offset, const by
        when the buffer is not fully written, except with an error. */
     return pinfo->io_procs->fwrite_chars(buf, length, pfile);
 }
+
+gx_device_clist *
+clist_make_accum_device(gx_device *target, const char *dname, void *base, int space,
+                        gx_device_buf_procs_t *buf_procs, gx_band_params_t *band_params,
+                        bool use_memory_clist, bool uses_transparency,
+                        gs_pattern1_instance_t *pinst)
+{
+        gs_memory_t *mem = target->memory;
+        gx_device_clist *cdev = gs_alloc_struct(mem, gx_device_clist,
+                        &st_device_clist, "clist_make_accum_device");
+        gx_device_clist_writer *cwdev = (gx_device_clist_writer *)cdev;
+
+        if (cdev == 0)
+            return 0;
+        memset(cdev, 0, sizeof(*cdev));
+        cwdev->params_size = sizeof(gx_device_clist);
+        cwdev->static_procs = NULL;
+        cwdev->dname = dname;
+        cwdev->memory = mem;
+        cwdev->stype = &st_device_clist;
+        cwdev->stype_is_dynamic = false;
+        rc_init(cwdev, mem, 1);
+        cwdev->retained = true;
+        cwdev->is_open = false;
+        cwdev->color_info = target->color_info;
+        cwdev->pinst = pinst;
+        cwdev->cached_colors = target->cached_colors;
+        if (pinst != NULL) {
+            cwdev->width = pinst->size.x;
+            cwdev->height = pinst->size.y;
+            cwdev->band_params.BandHeight = pinst->size.y;
+        } else {
+            cwdev->width = target->width;
+            cwdev->height = target->height;
+        }
+        cwdev->LeadingEdge = target->LeadingEdge;
+        cwdev->is_planar = target->is_planar;
+        cwdev->HWResolution[0] = target->HWResolution[0];
+        cwdev->HWResolution[1] = target->HWResolution[1];
+        cwdev->icc_cache_cl = NULL;
+        cwdev->icc_table = NULL;
+        cwdev->UseCIEColor = target->UseCIEColor;
+        cwdev->LockSafetyParams = true;
+        cwdev->procs = gs_clist_device_procs;
+        gx_device_copy_color_params((gx_device *)cwdev, target);
+        rc_assign(cwdev->target, target, "clist_make_accum_device");
+        clist_init_io_procs(cdev, use_memory_clist);
+        cwdev->data = base;
+        cwdev->data_size = space;
+        memcpy (&(cwdev->buf_procs), buf_procs, sizeof(gx_device_buf_procs_t));
+        cwdev->page_uses_transparency = uses_transparency;
+        cwdev->band_params.BandWidth = cwdev->width;
+        cwdev->band_params.BandBufferSpace = 0;
+        cwdev->do_not_open_or_close_bandfiles = false;
+        cwdev->bandlist_memory = mem->non_gc_memory;
+        set_dev_proc(cwdev, get_clipping_box, gx_default_get_clipping_box);
+        set_dev_proc(cwdev, get_profile, gx_forward_get_profile);
+        set_dev_proc(cwdev, set_graphics_type_tag, gx_forward_set_graphics_type_tag);
+        cwdev->graphics_type_tag = target->graphics_type_tag;	/* initialize to same as target */
+
+        /* to be set by caller: cwdev->finalize = finalize; */
+
+        /* Fields left zeroed :
+            int   max_fill_band;
+            int   is_printer;
+            float MediaSize[2];
+            float ImagingBBox[4];
+            bool  ImagingBBox_set;
+            float MarginsHWResolution[2];
+            float Margins[2];
+            float HWMargins[4];
+            long  PageCount;
+            long  ShowpageCount;
+            int   NumCopies;
+            bool  NumCopies_set;
+            bool  IgnoreNumCopies;
+            int   disable_mask;
+            gx_page_device_procs page_procs;
+
+        */
+        return cdev;
+}
