@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Implementation of clipping paths, other than actual clipping */
 #include "gx.h"
 #include "gserrors.h"
@@ -22,7 +24,7 @@
 #include "gxfixed.h"
 #include "gxpaint.h"
 #include "gscoord.h"		/* needs gsmatrix.h */
-#include "gxistate.h"
+#include "gxgstate.h"
 #include "gzpath.h"
 #include "gzcpath.h"
 #include "gzacpath.h"
@@ -36,7 +38,6 @@ public_st_clip_list();
 public_st_clip_path();
 private_st_clip_rect_list();
 public_st_device_clip();
-private_st_cpath_enum();
 private_st_cpath_path_list();
 
 /* GC procedures for gx_clip_path */
@@ -102,7 +103,8 @@ static const gx_clip_list clip_list_empty = {
     0, /* insert */
     0, /* xmin */
     0, /* xmax */
-    0  /* count */
+    0, /* count */
+    0  /* transpose = false */
 };
 
 /* ------ Clipping path memory management ------ */
@@ -421,6 +423,10 @@ gx_cpath_to_path_synthesize(const gx_clip_path * pcpath, gx_path * ppath)
                 code = gx_path_add_line_notes(ppath, pts[0].x, pts[0].y,
                                            gx_cpath_enum_notes(&cenum));
                 break;
+            case gs_pe_gapto:
+                code = gx_path_add_gap_notes(ppath, pts[0].x, pts[0].y,
+                                           gx_cpath_enum_notes(&cenum));
+                break;
             case gs_pe_curveto:
                 code = gx_path_add_curve_notes(ppath, pts[0].x, pts[0].y,
                                                pts[1].x, pts[1].y,
@@ -558,33 +564,38 @@ gx_cpath_reset(gx_clip_path * pcpath)
     return gx_cpath_from_rectangle(pcpath, &null_rect);
 }
 
-/* If a clipping path is a rectangle, return the rectangle. */
-const gs_fixed_rect *
-cpath_is_rectangle(const gx_clip_path * pcpath)
+/* If a clipping path is a rectangle, return the rectangle.
+ * If its a rectangular path, also return the rectangle.
+ */
+gx_path_rectangular_type cpath_is_rectangle(const gx_clip_path * pcpath, gs_fixed_rect *rect)
 {
-    if (pcpath->path_valid)
-        return NULL;
+    if (pcpath->path_valid) {
+        return gx_path_is_rectangle((const gx_path *)&pcpath->path, rect);
+    }
     if (pcpath->inner_box.p.x != pcpath->path.bbox.p.x ||
         pcpath->inner_box.p.y != pcpath->path.bbox.p.y ||
         pcpath->inner_box.q.x != pcpath->path.bbox.q.x ||
         pcpath->inner_box.q.y != pcpath->path.bbox.q.y)
-        return NULL;
-    return &pcpath->inner_box;
+        return prt_none;
+    rect->p.x = pcpath->inner_box.p.x;
+    rect->p.y = pcpath->inner_box.p.y;
+    rect->q.x = pcpath->inner_box.q.x;
+    rect->q.y = pcpath->inner_box.q.y;
+    return prt_closed;
 }
 
 /* Intersect a new clipping path with an old one. */
 /* Flatten the new path first (in a copy) if necessary. */
 int
-gx_cpath_clip(gs_state *pgs, gx_clip_path *pcpath,
+gx_cpath_clip(gs_gstate *pgs, gx_clip_path *pcpath,
               /*const*/ gx_path *ppath_orig, int rule)
 {
-    return gx_cpath_intersect(pcpath, ppath_orig, rule,
-                              (gs_imager_state *)pgs);
+    return gx_cpath_intersect(pcpath, ppath_orig, rule, pgs);
 }
 
 int
 gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
-                   int rule, gs_imager_state *pis, const gx_fill_params * params)
+                   int rule, gs_gstate *pgs, const gx_fill_params * params)
 {
     gx_path fpath;
     /*const*/ gx_path *ppath = ppath_orig;
@@ -593,10 +604,10 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
 
     /* Flatten the path if necessary. */
     if (gx_path_has_curves_inline(ppath)) {
-        gx_path_init_local(&fpath, pis->memory);
+        gx_path_init_local(&fpath, pgs->memory);
         code = gx_path_add_flattened_accurate(ppath, &fpath,
-                                              gs_currentflat_inline(pis),
-                                              pis->accurate_curves);
+                                              gs_currentflat_inline(pgs),
+                                              pgs->accurate_curves);
         if (code < 0)
             return code;
         ppath = &fpath;
@@ -612,17 +623,14 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
             /* The new path is void. */
             if (gx_path_current_point(ppath, &new_box.p) < 0) {
                 /* Use the user space origin (arbitrarily). */
-                new_box.p.x = float2fixed(pis->ctm.tx);
-                new_box.p.y = float2fixed(pis->ctm.ty);
+                new_box.p.x = float2fixed(pgs->ctm.tx);
+                new_box.p.y = float2fixed(pgs->ctm.ty);
             }
             new_box.q = new_box.p;
             changed = 1;
         } else {
-            if (params != NULL) {
-                /* Called from gx_default_fill_path for converting
-                   a filling path into a clipping path.
-                   Apply same adjustment as for filling the path. */
-                gs_fixed_point adjust = params->adjust;
+            {   /* Apply same adjustment as for filling the path. */
+                gs_fixed_point adjust = params != NULL ? params->adjust : pgs->fill_adjust;
                 fixed adjust_xl, adjust_xu, adjust_yl, adjust_yu;
 
                 if (adjust.x == -1)
@@ -683,7 +691,7 @@ gx_cpath_intersect_with_params(gx_clip_path *pcpath, /*const*/ gx_path *ppath_or
                 goto ex;
         }
         code = gx_cpath_intersect_path_slow(pcpath, (params != NULL ? ppath_orig : ppath),
-                            rule, pis, params);
+                            rule, pgs, params);
         if (code < 0)
             goto ex;
         if (path_valid) {
@@ -702,10 +710,10 @@ ex:
 }
 int
 gx_cpath_intersect(gx_clip_path *pcpath, /*const*/ gx_path *ppath_orig,
-                   int rule, gs_imager_state *pis)
+                   int rule, gs_gstate *pgs)
 {
     return gx_cpath_intersect_with_params(pcpath, ppath_orig,
-                   rule, pis, NULL);
+                   rule, pgs, NULL);
 }
 
 /* Scale a clipping path by a power of 2. */
@@ -1096,12 +1104,12 @@ gx_cpath_copy(const gx_clip_path * from, gx_clip_path * pcpath)
 
 /* Print a clipping list. */
 static void
-gx_clip_list_print(const gx_clip_list *list)
+gx_clip_list_print(const gs_memory_t *mem, const gx_clip_list *list)
 {
     const gx_clip_rect *pr;
 
-    dlprintf3("   list count=%d xmin=%d xmax=%d\n",
-             list->count, list->xmin, list->xmax);
+    dmlprintf3(mem, "   list count=%d xmin=%d xmax=%d\n",
+               list->count, list->xmin, list->xmax);
     switch (list->count) {
         case 0:
             pr = 0;
@@ -1113,31 +1121,31 @@ gx_clip_list_print(const gx_clip_list *list)
             pr = list->head;
     }
     for (; pr != 0; pr = pr->next)
-        dlprintf4("   rect: (%d,%d),(%d,%d)\n",
-                  pr->xmin, pr->ymin, pr->xmax, pr->ymax);
+        dmlprintf4(mem, "   rect: (%d,%d),(%d,%d)\n",
+                   pr->xmin, pr->ymin, pr->xmax, pr->ymax);
 }
 
 /* Print a clipping path */
 void
-gx_cpath_print(const gx_clip_path * pcpath)
+gx_cpath_print(const gs_memory_t *mem, const gx_clip_path * pcpath)
 {
     if (pcpath->path_valid)
         gx_path_print(&pcpath->path);
     else
-        dlputs("   (path not valid)\n");
-    dlprintf4("   inner_box=(%g,%g),(%g,%g)\n",
-              fixed2float(pcpath->inner_box.p.x),
-              fixed2float(pcpath->inner_box.p.y),
-              fixed2float(pcpath->inner_box.q.x),
-              fixed2float(pcpath->inner_box.q.y));
-    dlprintf4("     outer_box=(%g,%g),(%g,%g)",
-              fixed2float(pcpath->outer_box.p.x),
-              fixed2float(pcpath->outer_box.p.y),
-              fixed2float(pcpath->outer_box.q.x),
-              fixed2float(pcpath->outer_box.q.y));
-    dprintf2("     rule=%d list.refct=%ld\n",
-             pcpath->rule, pcpath->rect_list->rc.ref_count);
-    gx_clip_list_print(gx_cpath_list(pcpath));
+        dmlputs(mem, "   (path not valid)\n");
+    dmlprintf4(mem, "   inner_box=(%g,%g),(%g,%g)\n",
+               fixed2float(pcpath->inner_box.p.x),
+               fixed2float(pcpath->inner_box.p.y),
+               fixed2float(pcpath->inner_box.q.x),
+               fixed2float(pcpath->inner_box.q.y));
+    dmlprintf4(mem, "     outer_box=(%g,%g),(%g,%g)",
+               fixed2float(pcpath->outer_box.p.x),
+               fixed2float(pcpath->outer_box.p.y),
+               fixed2float(pcpath->outer_box.q.x),
+               fixed2float(pcpath->outer_box.q.y));
+    dmprintf2(mem, "     rule=%d list.refct=%ld\n",
+              pcpath->rule, pcpath->rect_list->rc.ref_count);
+    gx_clip_list_print(mem, gx_cpath_list(pcpath));
 }
 
 #endif /* DEBUG */
