@@ -1,23 +1,25 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdevice.h 9288 2008-12-13 20:05:37Z leonardo $ */
+
 /* Definitions for device implementors */
 
 #ifndef gxdevice_INCLUDED
 #  define gxdevice_INCLUDED
 
-#include "stdio_.h"		/* for FILE */
+#include "stdio_.h"             /* for FILE */
 #include "gxdevcli.h"
 #include "gsfname.h"
 #include "gsparam.h"
@@ -61,6 +63,22 @@
 #  define DEFAULT_HEIGHT_10THS DEFAULT_HEIGHT_10THS_US_LETTER
 #endif
 
+/* Define parameters for machines with little dinky RAMs.... */
+#if ARCH_SMALL_MEMORY
+#   define MAX_BITMAP 32000
+#   define BUFFER_SPACE 25000
+#   define MIN_MEMORY_LEFT 32000
+#else
+#   define MAX_BITMAP 10000000L /* reasonable on most modern hosts */
+#   define BUFFER_SPACE 4000000L
+#   define MIN_MEMORY_LEFT 500000L
+#endif
+#define MIN_BUFFER_SPACE 10000	/* give up if less than this */
+
+#ifndef MaxPatternBitmap_DEFAULT
+#  define MaxPatternBitmap_DEFAULT MAX_BITMAP
+#endif
+
 /* ---------------- Device structure ---------------- */
 
 /*
@@ -80,30 +98,41 @@
  * Note also that the macro does not initialize procs, which is
  * the next element of the structure.
  */
-#define std_device_part1_(devtype, ptr_procs, dev_name, stype, open_init)\
-	sizeof(devtype), ptr_procs, dev_name,\
-	0 /*memory*/, stype, 0 /*stype_is_dynamic*/, 0 /*finalize*/,\
-	{ 0 } /*rc*/, 0 /*retained*/, open_init() /*is_open, max_fill_band*/
-	/* color_info goes here */
+    #define std_device_part1_(devtype, ptr_procs, dev_name, stype, open_init)\
+        sizeof(devtype), ptr_procs, dev_name,\
+        0 /*memory*/, stype, 0 /*stype_is_dynamic*/, 0 /*finalize*/,\
+        { 0 } /*rc*/, 0 /*retained*/, 0 /* parent */, 0 /* child */, 0 /* subclass_data */, 0, /* PageList */\
+        open_init() /*is_open, max_fill_band*/
+        /* color_info goes here */
 /*
  * The MetroWerks compiler has some bizarre bug that produces a spurious
  * error message if the width and/or height are defined as 0 below,
  * unless we use the +/- workaround in the next macro.
  */
 #define std_device_part2_(width, height, x_dpi, y_dpi)\
-	{ gx_no_color_index, gx_no_color_index },\
-          width, height, 0/*TrayOrientation*/,\
-	{ (float)((((width) * 72.0 + 0.5) - 0.5) / (x_dpi))/*MediaSize[0]*/,\
-	  (float)((((height) * 72.0 + 0.5) - 0.5) / (y_dpi))/*MediaSize[1]*/},\
-	{ 0, 0, 0, 0 }/*ImagingBBox*/, 0/*ImagingBBox_set*/,\
-	{ x_dpi, y_dpi }/*HWResolution*/, { x_dpi, y_dpi }/*MarginsHWResolution*/
+        { gx_no_color_index, gx_no_color_index },\
+          width, height, 0/*Pad*/, 0/*align*/, 0/*Num planes*/, 0/*TrayOrientation*/,\
+        { (float)((((width) * 72.0 + 0.5) - 0.5) / (x_dpi))/*MediaSize[0]*/,\
+          (float)((((height) * 72.0 + 0.5) - 0.5) / (y_dpi))/*MediaSize[1]*/},\
+        { 0, 0, 0, 0 }/*ImagingBBox*/, 0/*ImagingBBox_set*/,\
+        { x_dpi, y_dpi }/*HWResolution*/, { x_dpi, y_dpi }/*MarginsHWResolution*/
 
 /* offsets and margins go here */
 #define std_device_part3_()\
-	0/*PageCount*/, 0/*ShowpageCount*/, 1/*NumCopies*/, 0/*NumCopies_set*/,\
-	0/*IgnoreNumCopies*/, 0/*UseCIEColor*/, 0/*LockSafetyParams*/,\
-	0/*band_offset_x*/, 0/*band_offset_y*/, {false}/* sgr */,\
-	{ gx_default_install, gx_default_begin_page, gx_default_end_page }
+        0/*FirstPage*/, 0/*LastPage*/, 0/*PageHandlerPushed*/, 0/*DisablePageHandler*/, 0/* Object Filter*/, 0/*ObjectHandlerPushed*/,\
+        0/*PageCount*/, 0/*ShowpageCount*/, 1/*NumCopies*/, 0/*NumCopies_set*/,\
+        0/*IgnoreNumCopies*/, 0/*UseCIEColor*/, 0/*LockSafetyParams*/,\
+        0/*band_offset_x*/, 0/*band_offset_y*/, false /*BLS_force_memory*/, \
+        {false}/* sgr */,\
+        0/* MaxPatternBitmap */, 0/*page_uses_transparency*/,\
+        { MAX_BITMAP, BUFFER_SPACE,\
+             { BAND_PARAMS_INITIAL_VALUES },\
+           0/*false*/, /* params_are_read_only */\
+           BandingAuto /* banding_type */\
+        }, /*space_params*/\
+        0/*Profile Array*/,\
+        0/* graphics_type_tag default GS_UNKNOWN_TAG */,\
+        { gx_default_install, gx_default_begin_page, gx_default_end_page }
 /*
  * We need a number of different variants of the std_device_ macro simply
  * because we can't pass the color_info or offsets/margins
@@ -117,94 +146,94 @@
  *        6 for margins/offset).
  *      ...std_color_full_body, which takes 7 values (1 for dci_std_color,
  *        6 for margins/offset).
- *      
+ *
  */
 #define std_device_body_with_macros_(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, open_init, dci_macro, margins_macro)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init),\
-	dci_macro(),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	margins_macro(),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, stype, open_init),\
+        dci_macro(),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        margins_macro(),\
+        std_device_part3_()
 
 #define std_device_std_body_type(dtype, pprocs, dname, stype, w, h, xdpi, ydpi)\
-	std_device_body_with_macros_(dtype, pprocs, dname, stype,\
-	  w, h, xdpi, ydpi,\
-	  open_init_closed, dci_black_and_white_, no_margins_)
+        std_device_body_with_macros_(dtype, pprocs, dname, stype,\
+          w, h, xdpi, ydpi,\
+          open_init_closed, dci_black_and_white_, no_margins_)
 
 #define std_device_std_body(dtype, pprocs, dname, w, h, xdpi, ydpi)\
-	std_device_std_body_type(dtype, pprocs, dname, 0, w, h, xdpi, ydpi)
+        std_device_std_body_type(dtype, pprocs, dname, 0, w, h, xdpi, ydpi)
 
 #define std_device_std_body_type_open(dtype, pprocs, dname, stype, w, h, xdpi, ydpi)\
-	std_device_body_with_macros_(dtype, pprocs, dname, stype,\
-	  w, h, xdpi, ydpi,\
-	  open_init_open, dci_black_and_white_, no_margins_)
+        std_device_body_with_macros_(dtype, pprocs, dname, stype,\
+          w, h, xdpi, ydpi,\
+          open_init_open, dci_black_and_white_, no_margins_)
 
 #define std_device_std_body_open(dtype, pprocs, dname, w, h, xdpi, ydpi)\
-	std_device_std_body_type_open(dtype, pprocs, dname, 0, w, h, xdpi, ydpi)
+        std_device_std_body_type_open(dtype, pprocs, dname, 0, w, h, xdpi, ydpi)
 
 #define std_device_full_body_type(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, xoff, yoff, lm, bm, rm, tm)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
-	dci_values(ncomp, depth, mg, mc, dg, dc),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+        dci_values(ncomp, depth, mg, mc, dg, dc),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
+        std_device_part3_()
 
 #define std_device_full_body_type_extended(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, mcomp, ncomp, pol, depth, gi, mg, mc, dg, dc, ef, cn, xoff, yoff, lm, bm, rm, tm)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+        std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
         dci_extended_alpha_values(mcomp, ncomp, pol, depth, gi, mg, mc, dg, dc, 1, 1, ef, cn), \
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
-	std_device_part3_()
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
+        std_device_part3_()
 
 #define std_device_full_body(dtype, pprocs, dname, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, xoff, yoff, lm, bm, rm, tm)\
-	std_device_full_body_type(dtype, pprocs, dname, 0, w, h, xdpi, ydpi,\
-	    ncomp, depth, mg, mc, dg, dc, xoff, yoff, lm, bm, rm, tm)
+        std_device_full_body_type(dtype, pprocs, dname, 0, w, h, xdpi, ydpi,\
+            ncomp, depth, mg, mc, dg, dc, xoff, yoff, lm, bm, rm, tm)
 
 #define std_device_dci_alpha_type_body(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, ta, ga)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
-	dci_alpha_values(ncomp, depth, mg, mc, dg, dc, ta, ga),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(0, 0, 0, 0, 0, 0),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+        dci_alpha_values(ncomp, depth, mg, mc, dg, dc, ta, ga),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(0, 0, 0, 0, 0, 0),\
+        std_device_part3_()
 
 #define std_device_dci_type_body(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)\
-	std_device_dci_alpha_type_body(dtype, pprocs, dname, stype, w, h,\
-	  xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, 1, 1)
+        std_device_dci_alpha_type_body(dtype, pprocs, dname, stype, w, h,\
+          xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, 1, 1)
 
 #define std_device_dci_body(dtype, pprocs, dname, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)\
-	std_device_dci_type_body(dtype, pprocs, dname, 0,\
-	  w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)
+        std_device_dci_type_body(dtype, pprocs, dname, 0,\
+          w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)
 
 #define std_device_color_full_body(dtype, pprocs, dname, w, h, xdpi, ydpi, depth, max_value, dither, xoff, yoff, lm, bm, rm, tm)\
-	std_device_part1_(dtype, pprocs, dname, 0, open_init_closed),\
-	dci_color(depth, max_value, dither),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, 0, open_init_closed),\
+        dci_color(depth, max_value, dither),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
+        std_device_part3_()
 
 #define std_device_color_body(dtype, pprocs, dname, w, h, xdpi, ydpi, depth, max_value, dither)\
-	std_device_color_full_body(dtype, pprocs, dname,\
-	  w, h, xdpi, ydpi,\
-	  depth, max_value, dither,\
-	  0, 0, 0, 0, 0, 0)
+        std_device_color_full_body(dtype, pprocs, dname,\
+          w, h, xdpi, ydpi,\
+          depth, max_value, dither,\
+          0, 0, 0, 0, 0, 0)
 
 #define std_device_color_stype_body(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, depth, max_value, dither)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
-	dci_color(depth, max_value, dither),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(0, 0, 0, 0, 0, 0),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+        dci_color(depth, max_value, dither),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(0, 0, 0, 0, 0, 0),\
+        std_device_part3_()
 
 #define std_device_std_color_full_body_type(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, depth, xoff, yoff, lm, bm, rm, tm)\
-	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
-	dci_std_color(depth),\
-	std_device_part2_(w, h, xdpi, ydpi),\
-	offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
-	std_device_part3_()
+        std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+        dci_std_color(depth),\
+        std_device_part2_(w, h, xdpi, ydpi),\
+        offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
+        std_device_part3_()
 
 #define std_device_std_color_full_body(dtype, pprocs, dname, w, h, xdpi, ydpi, depth, xoff, yoff, lm, bm, rm, tm)\
-	std_device_std_color_full_body_type(dtype, pprocs, dname, 0,\
-	    w, h, xdpi, ydpi, depth, xoff, yoff, lm, bm, rm, tm)
+        std_device_std_color_full_body_type(dtype, pprocs, dname, 0,\
+            w, h, xdpi, ydpi, depth, xoff, yoff, lm, bm, rm, tm)
 
 /* ---------------- Default implementations ---------------- */
 
@@ -224,7 +253,7 @@ dev_proc_tile_rectangle(gx_default_tile_rectangle);
 dev_proc_copy_mono(gx_default_copy_mono);
 dev_proc_copy_color(gx_default_copy_color);
 dev_proc_draw_line(gx_default_draw_line);
-dev_proc_get_bits(gx_no_get_bits);	/* gives error */
+dev_proc_get_bits(gx_no_get_bits);      /* gives error */
 dev_proc_get_bits(gx_default_get_bits);
 dev_proc_get_params(gx_default_get_params);
 dev_proc_put_params(gx_default_put_params);
@@ -232,13 +261,13 @@ dev_proc_map_cmyk_color(gx_default_map_cmyk_color);
 dev_proc_get_xfont_procs(gx_default_get_xfont_procs);
 dev_proc_get_xfont_device(gx_default_get_xfont_device);
 dev_proc_map_rgb_alpha_color(gx_default_map_rgb_alpha_color);
-dev_proc_get_page_device(gx_default_get_page_device);	/* returns NULL */
-dev_proc_get_page_device(gx_page_device_get_page_device);	/* returns dev */
+dev_proc_get_page_device(gx_default_get_page_device);   /* returns NULL */
+dev_proc_get_page_device(gx_page_device_get_page_device);       /* returns dev */
 dev_proc_get_alpha_bits(gx_default_get_alpha_bits);
-dev_proc_copy_alpha(gx_no_copy_alpha);	/* gives error */
+dev_proc_copy_alpha(gx_no_copy_alpha);  /* gives error */
 dev_proc_copy_alpha(gx_default_copy_alpha);
 dev_proc_get_band(gx_default_get_band);
-dev_proc_copy_rop(gx_no_copy_rop);	/* gives error */
+dev_proc_copy_rop(gx_no_copy_rop);      /* gives error */
 dev_proc_copy_rop(gx_default_copy_rop);
 dev_proc_fill_path(gx_default_fill_path);
 dev_proc_stroke_path(gx_default_stroke_path);
@@ -251,12 +280,12 @@ dev_proc_begin_image(gx_default_begin_image);
 dev_proc_image_data(gx_default_image_data);
 dev_proc_end_image(gx_default_end_image);
 dev_proc_strip_tile_rectangle(gx_default_strip_tile_rectangle);
-dev_proc_strip_copy_rop(gx_no_strip_copy_rop);	/* gives error */
+dev_proc_strip_copy_rop(gx_no_strip_copy_rop);  /* gives error */
 dev_proc_strip_copy_rop(gx_default_strip_copy_rop);
 dev_proc_get_clipping_box(gx_default_get_clipping_box);
 dev_proc_get_clipping_box(gx_get_largest_clipping_box);
 dev_proc_begin_typed_image(gx_default_begin_typed_image);
-dev_proc_get_bits_rectangle(gx_no_get_bits_rectangle);	/* gives error */
+dev_proc_get_bits_rectangle(gx_no_get_bits_rectangle);  /* gives error */
 dev_proc_get_bits_rectangle(gx_default_get_bits_rectangle);
 dev_proc_map_color_rgb_alpha(gx_default_map_color_rgb_alpha);
 dev_proc_create_compositor(gx_no_create_compositor);
@@ -267,15 +296,22 @@ dev_proc_create_compositor(gx_null_create_compositor);
 dev_proc_get_hardware_params(gx_default_get_hardware_params);
 dev_proc_text_begin(gx_default_text_begin);
 dev_proc_finish_copydevice(gx_default_finish_copydevice);
-dev_proc_pattern_manage(gx_default_pattern_manage);
+dev_proc_dev_spec_op(gx_default_dev_spec_op);
 dev_proc_fill_rectangle_hl_color(gx_default_fill_rectangle_hl_color);
 dev_proc_include_color_space(gx_default_include_color_space);
 dev_proc_fill_linear_color_scanline(gx_default_fill_linear_color_scanline);
+dev_proc_fill_linear_color_scanline(gx_hl_fill_linear_color_scanline);
 dev_proc_fill_linear_color_trapezoid(gx_default_fill_linear_color_trapezoid);
 dev_proc_fill_linear_color_triangle(gx_default_fill_linear_color_triangle);
 dev_proc_update_spot_equivalent_colors(gx_default_update_spot_equivalent_colors);
 dev_proc_ret_devn_params(gx_default_ret_devn_params);
 dev_proc_fillpage(gx_default_fillpage);
+dev_proc_get_profile(gx_default_get_profile);
+dev_proc_set_graphics_type_tag(gx_default_set_graphics_type_tag);
+dev_proc_strip_copy_rop2(gx_default_strip_copy_rop2);
+dev_proc_strip_tile_rect_devn(gx_default_strip_tile_rect_devn);
+dev_proc_copy_alpha_hl_color(gx_default_copy_alpha_hl_color);
+dev_proc_process_page(gx_default_process_page);
 /* BACKWARD COMPATIBILITY */
 #define gx_non_imaging_create_compositor gx_null_create_compositor
 
@@ -299,6 +335,8 @@ dev_proc_decode_color(cmyk_1bit_map_color_cmyk);
 dev_proc_map_cmyk_color(cmyk_8bit_map_cmyk_color);
 dev_proc_map_color_rgb(cmyk_8bit_map_color_rgb);
 dev_proc_decode_color(cmyk_8bit_map_color_cmyk);
+dev_proc_map_cmyk_color(cmyk_16bit_map_cmyk_color);
+dev_proc_decode_color(cmyk_16bit_map_color_cmyk);
 dev_proc_encode_color(gx_default_8bit_map_gray_color);
 dev_proc_decode_color(gx_default_8bit_map_color_gray);
 
@@ -348,7 +386,7 @@ dev_proc_get_color_mapping_procs(gx_forward_get_color_mapping_procs);
 dev_proc_get_color_comp_index(gx_forward_get_color_comp_index);
 dev_proc_encode_color(gx_forward_encode_color);
 dev_proc_decode_color(gx_forward_decode_color);
-dev_proc_pattern_manage(gx_forward_pattern_manage);
+dev_proc_dev_spec_op(gx_forward_dev_spec_op);
 dev_proc_fill_rectangle_hl_color(gx_forward_fill_rectangle_hl_color);
 dev_proc_include_color_space(gx_forward_include_color_space);
 dev_proc_fill_linear_color_scanline(gx_forward_fill_linear_color_scanline);
@@ -357,8 +395,16 @@ dev_proc_fill_linear_color_triangle(gx_forward_fill_linear_color_triangle);
 dev_proc_update_spot_equivalent_colors(gx_forward_update_spot_equivalent_colors);
 dev_proc_ret_devn_params(gx_forward_ret_devn_params);
 dev_proc_fillpage(gx_forward_fillpage);
+dev_proc_copy_planes(gx_forward_copy_planes);
+dev_proc_create_compositor(gx_forward_create_compositor);
+dev_proc_get_profile(gx_forward_get_profile);
+dev_proc_set_graphics_type_tag(gx_forward_set_graphics_type_tag);
+dev_proc_strip_copy_rop2(gx_forward_strip_copy_rop2);
+dev_proc_strip_tile_rect_devn(gx_forward_strip_tile_rect_devn);
+dev_proc_copy_alpha_hl_color(gx_forward_copy_alpha_hl_color);
 
 /* ---------------- Implementation utilities ---------------- */
+int gx_default_get_param(gx_device *dev, char *Param, void *list);
 
 /* Convert the device procedures to the proper form (see above). */
 void gx_device_set_procs(gx_device *);
@@ -420,20 +466,28 @@ void gx_device_copy_params(gx_device *dev, const gx_device *target);
  * is currently allowed.
  */
 int gx_parse_output_file_name(gs_parsed_file_name_t *pfn,
-			      const char **pfmt, const char *fname,
-			      uint len);
+                              const char **pfmt, const char *fname,
+                              uint len, gs_memory_t *memory);
+
+/*
+ * Returns true if the outputfile requests separate pages (contains %d)
+ */
+bool gx_outputfile_is_separate_pages(const char *fname, gs_memory_t *memory);
 
 /*
  * Open the output file for a device.  Note that if the file name is empty,
  * it may be replaced with the name of a scratch file.
  */
 int gx_device_open_output_file(const gx_device * dev, char *fname,
-			       bool binary, bool positionable,
-			       FILE ** pfile);
+                               bool binary, bool positionable,
+                               FILE ** pfile);
 
 /* Close the output file for a device. */
 int gx_device_close_output_file(const gx_device * dev, const char *fname,
-				FILE *file);
+                                FILE *file);
+
+/* Delete the current output file for a device (file must be closed first) */
+int gx_device_delete_output_file(const gx_device * dev, const char *fname);
 
 /*
  * Define the number of levels for a colorant above which we do not halftone.
@@ -442,7 +496,7 @@ int gx_device_close_output_file(const gx_device * dev, const char *fname,
 
 /*
  * Determine whether a given device needs to halftone.  Eventually this
- * should take an imager state as an additional argument.
+ * should take a gs_gstate as an additional argument.
  */
 #define gx_device_must_halftone(dev)\
   ((gx_device_has_color(dev) ? (dev)->color_info.max_color :\
@@ -468,42 +522,42 @@ dev_proc_output_page(gx_finish_output_page);
  */
 #define fit_fill_xy(dev, x, y, w, h)\
   BEGIN\
-	if ( (x | y) < 0 ) {\
-	  if ( x < 0 )\
-	    w += x, x = 0;\
-	  if ( y < 0 )\
-	    h += y, y = 0;\
-	}\
+        if ( (x | y) < 0 ) {\
+          if ( x < 0 )\
+            w += x, x = 0;\
+          if ( y < 0 )\
+            h += y, y = 0;\
+        }\
   END
 #define fit_fill_y(dev, y, h)\
   BEGIN\
-	if ( y < 0 )\
-	  h += y, y = 0;\
+        if ( y < 0 )\
+          h += y, y = 0;\
   END
 #define fit_fill_w(dev, x, w)\
   BEGIN\
-	if ( w > (dev)->width - x )\
-	  w = (dev)->width - x;\
+        if ( w > (dev)->width - x )\
+          w = (dev)->width - x;\
   END
 #define fit_fill_h(dev, y, h)\
   BEGIN\
-	if ( h > (dev)->height - y )\
-	  h = (dev)->height - y;\
+        if ( h > (dev)->height - y )\
+          h = (dev)->height - y;\
   END
 #define fit_fill_xywh(dev, x, y, w, h)\
   BEGIN\
-	fit_fill_xy(dev, x, y, w, h);\
-	fit_fill_w(dev, x, w);\
-	fit_fill_h(dev, y, h);\
+        fit_fill_xy(dev, x, y, w, h);\
+        fit_fill_w(dev, x, w);\
+        fit_fill_h(dev, y, h);\
   END
 /*
  * Clip all edges, and return from the procedure if the result is empty.
  */
 #define fit_fill(dev, x, y, w, h)\
   BEGIN\
-	fit_fill_xywh(dev, x, y, w, h);\
-	if ( w <= 0 || h <= 0 )\
-	  return 0;\
+        fit_fill_xywh(dev, x, y, w, h);\
+        if ( w <= 0 || h <= 0 )\
+          return 0;\
   END
 
 /*
@@ -517,25 +571,25 @@ dev_proc_output_page(gx_finish_output_page);
  */
 #define fit_copy_xyw(dev, data, data_x, raster, id, x, y, w, h)\
   BEGIN\
-	if ( (x | y) < 0 ) {\
-	  if ( x < 0 )\
-	    w += x, data_x -= x, x = 0;\
-	  if ( y < 0 )\
-	    h += y, data -= y * raster, id = gx_no_bitmap_id, y = 0;\
-	}\
-	if ( w > (dev)->width - x )\
-	  w = (dev)->width - x;\
+        if ( (x | y) < 0 ) {\
+          if ( x < 0 )\
+            w += x, data_x -= x, x = 0;\
+          if ( y < 0 )\
+            h += y, data -= (int)(y * raster), id = gx_no_bitmap_id, y = 0; \
+        }\
+        if ( w > (dev)->width - x )\
+          w = (dev)->width - x;\
   END
 /*
  * Clip all edges, and return from the procedure if the result is empty.
  */
 #define fit_copy(dev, data, data_x, raster, id, x, y, w, h)\
   BEGIN\
-	fit_copy_xyw(dev, data, data_x, raster, id, x, y, w, h);\
-	if ( h > (dev)->height - y )\
-	  h = (dev)->height - y;\
-	if ( w <= 0 || h <= 0 )\
-	  return 0;\
+        fit_copy_xyw(dev, data, data_x, raster, id, x, y, w, h);\
+        if ( h > (dev)->height - y )\
+          h = (dev)->height - y;\
+        if ( w <= 0 || h <= 0 )\
+          return 0;\
   END
 
 /* ---------------- Media parameters ---------------- */
@@ -544,7 +598,7 @@ dev_proc_output_page(gx_finish_output_page);
 /* The device get_params procedure would call these. */
 
 typedef struct gdev_input_media_s {
-    float PageSize[4];		/* nota bene */
+    float PageSize[4];          /* nota bene */
     const char *MediaColor;
     float MediaWeight;
     const char *MediaType;
@@ -556,13 +610,13 @@ extern const gdev_input_media_t gdev_input_media_default;
 void gdev_input_media_init(gdev_input_media_t * pim);
 
 int gdev_begin_input_media(gs_param_list * mlist, gs_param_dict * pdict,
-			   int count);
+                           int count);
 
 int gdev_write_input_page_size(int index, gs_param_dict * pdict,
-			       floatp width_points, floatp height_points);
+                               double width_points, double height_points);
 
 int gdev_write_input_media(int index, gs_param_dict * pdict,
-			   const gdev_input_media_t * pim);
+                           const gdev_input_media_t * pim);
 
 int gdev_end_input_media(gs_param_list * mlist, gs_param_dict * pdict);
 
@@ -574,13 +628,37 @@ typedef struct gdev_output_media_s {
 extern const gdev_output_media_t gdev_output_media_default;
 
 int gdev_begin_output_media(gs_param_list * mlist, gs_param_dict * pdict,
-			    int count);
+                            int count);
 
 int gdev_write_output_media(int index, gs_param_dict * pdict,
-			    const gdev_output_media_t * pom);
+                            const gdev_output_media_t * pom);
+
+/* Can be called from set user params */
+void gx_default_put_icc_dir(gs_param_string *icc_pro, gx_device * dev);
 
 int gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict);
 
 void gx_device_request_leadingedge(gx_device *dev, int le_req);
 
+/* ---------------- Device subclassing procedures ---------------- */
+int gs_is_pdf14trans_compositor(const gs_composite_t * pct);
+
+#define subclass_common\
+    t_dev_proc_create_compositor *saved_compositor_method
+
+typedef int (t_dev_proc_create_compositor) (gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte, gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev);
+
+typedef struct {
+    t_dev_proc_create_compositor *saved_compositor_method;
+} generic_subclass_data;
+
+
+int gx_copy_device_procs(gx_device_procs *dest_procs, gx_device_procs *src_procs, gx_device_procs *prototype_procs);
+int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, unsigned int private_data_size);
+int gx_device_unsubclass(gx_device *dev);
+int gx_update_from_subclass(gx_device *dev);
+int gx_subclass_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte,
+    gs_gstate *pgs, gs_memory_t *memory, gx_device *cdev);
+
+
 #endif /* gxdevice_INCLUDED */
