@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Common code for ImageType 1 and 4 initialization */
 #include "gx.h"
 #include "math_.h"
@@ -39,6 +41,7 @@
 #include "gscindex.h"
 #include "gsicc_cache.h"
 #include "gsicc_cms.h"
+#include "gxdevsop.h"
 
 /* Structure descriptors */
 private_st_gx_image_enum();
@@ -126,11 +129,10 @@ RELOC_PTRS_END
 /* Forward declarations */
 static int color_draws_b_w(gx_device * dev,
                             const gx_drawing_color * pdcolor);
-static void image_init_map(byte * map, int map_size, const float *decode);
-static void image_init_colors(gx_image_enum * penum, int bps, int spp,
+static int image_init_colors(gx_image_enum * penum, int bps, int spp,
                                gs_image_format_t format,
                                const float *decode,
-                               const gs_imager_state * pis, gx_device * dev,
+                               const gs_gstate * pgs, gx_device * dev,
                                const gs_color_space * pcs, bool * pdcb);
 
 /* Procedures for unpacking the input data into bytes or fracs. */
@@ -185,11 +187,15 @@ gx_image_enum_alloc(const gs_image_common_t * pic,
         penum->rect.x = 0, penum->rect.y = 0;
         penum->rect.w = width, penum->rect.h = height;
     }
+    penum->rrect.x = penum->rect.x;
+    penum->rrect.y = penum->rect.y;
+    penum->rrect.w = penum->rect.w;
+    penum->rrect.h = penum->rect.h;
 #ifdef DEBUG
     if (gs_debug_c('b')) {
-        dlprintf2("[b]Image: w=%d h=%d", width, height);
+        dmlprintf2(mem, "[b]Image: w=%d h=%d", width, height);
         if (prect)
-            dprintf4(" ((%d,%d),(%d,%d))",
+            dmprintf4(mem, " ((%d,%d),(%d,%d))",
                      prect->p.x, prect->p.y, prect->q.x, prect->q.y);
     }
 #endif
@@ -197,6 +203,44 @@ gx_image_enum_alloc(const gs_image_common_t * pic,
     return 0;
 }
 
+/* Convert and restrict to a valid range. */
+static inline fixed float2fixed_rounded_boxed(double src) {
+    float v = floor(src*fixed_scale + 0.5);
+
+    if (v <= min_fixed)
+        return min_fixed;
+    else if (v >= max_fixed)
+        return max_fixed;
+    else
+        return 	(fixed)v;
+}
+
+/* Compute the image matrix combining the ImageMatrix with either the pmat or the pgs ctm */
+int
+gx_image_compute_mat(const gs_gstate *pgs, const gs_matrix *pmat, const gs_matrix *ImageMatrix,
+                     gs_matrix_double *rmat)
+{
+    int code = 0;
+
+    if (pmat == 0)
+        pmat = &ctm_only(pgs);
+    if (ImageMatrix->xx == pmat->xx && ImageMatrix->xy == pmat->xy &&
+        ImageMatrix->yx == pmat->yx && ImageMatrix->yy == pmat->yy) {
+        /* Process common special case separately to accept singular matrix. */
+        rmat->xx = rmat->yy = 1.;
+        rmat->xy = rmat->yx = 0.;
+        rmat->tx = pmat->tx - ImageMatrix->tx;
+        rmat->ty = pmat->ty - ImageMatrix->ty;
+    } else {
+        if ((code = gs_matrix_invert_to_double(ImageMatrix, rmat)) < 0 ||
+            (code = gs_matrix_multiply_double(rmat, pmat, rmat)) < 0
+            ) {
+            return code;
+        }
+    }
+    return code;
+}
+
 /*
  * Finish initialization for processing an ImageType 1 or 4 image.
  * Assumes the following members of *penum are set in addition to those
@@ -205,7 +249,7 @@ gx_image_enum_alloc(const gs_image_common_t * pic,
  *      masked, adjust
  */
 int
-gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
+gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
                     const gs_matrix *pmat, const gs_image_common_t * pic,
                 const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
                 gs_memory_t * mem, gx_image_enum *penum)
@@ -220,7 +264,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     gs_matrix_double mat;
     int index_bps;
     const gs_color_space *pcs = pim->ColorSpace;
-    gs_logical_operation_t lop = (pis ? pis->log_op : lop_default);
+    gs_logical_operation_t lop = (pgs ? pgs->log_op : lop_default);
     int code;
     int log2_xbytes = (bps <= 8 ? 0 : arch_log2_sizeof_frac);
     int spp, nplanes, spread;
@@ -230,6 +274,10 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     gs_fixed_point row_extent, col_extent, x_extent, y_extent;
     bool device_color = true;
     gs_fixed_rect obox, cbox;
+    bool gridfitimages = 0;
+    bool in_pattern_accumulator = 0;
+    int orthogonal;
+    int force_interpolation = 0;
 
     penum->clues = NULL;
     penum->icc_setup.has_transfer = false;
@@ -238,24 +286,11 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     penum->icc_setup.need_decode = false;
     penum->Width = width;
     penum->Height = height;
-    if (pmat == 0)
-        pmat = &ctm_only(pis);
-    if (pim->ImageMatrix.xx == pmat->xx && pim->ImageMatrix.xy == pmat->xy &&
-        pim->ImageMatrix.yx == pmat->yx && pim->ImageMatrix.yy == pmat->yy) {
-        /* Process common special case separately to accept singular matrix. */
-        mat.xx = mat.yy = 1.;
-        mat.xy = mat.yx = 0.;
-        mat.tx = pmat->tx - pim->ImageMatrix.tx;
-        mat.ty = pmat->ty - pim->ImageMatrix.ty;
-    } else {
-        if ((code = gs_matrix_invert_to_double(&pim->ImageMatrix, &mat)) < 0 ||
-            (code = gs_matrix_multiply_double(&mat, pmat, &mat)) < 0
-            ) {
-            gs_free_object(mem, penum, "gx_default_begin_image");
-            return code;
-        }
-    }
 
+    if ((code = gx_image_compute_mat(pgs, pmat, &(pim->ImageMatrix), &mat)) < 0) {
+        gs_free_object(mem, penum, "gx_default_begin_image");
+        return code;
+    }
     /* Grid fit: A common construction in postscript/PDF files is for images
      * to be constructed as a series of 'stacked' 1 pixel high images.
      * Furthermore, many of these are implemented as an imagemask plotted on
@@ -265,7 +300,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * covered. Bug 692666 is such a problem.
      *
      * As a workaround for this problem, the code below was introduced. The
-     * concept is that orthogonal images can be 'grid fitted' (or 'stretched')
+     * concept is that orthogonal images can be 'grid fitted' (or 'stretch')
      * to entirely cover pixels that they touch. Initially I had this working
      * for all images regardless of type, but as testing has proceeded, this
      * showed more and more regressions, so I've cut the cases back in which
@@ -273,15 +308,40 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * either 1 pixel high, or wide, and then not if we are rendering a
      * glyph (such as from a type3 font).
      */
-    if (pis != NULL && pis->is_gstate && ((gs_state *)pis)->show_gstate != NULL) {
+
+    /* Ask the device if we are in a pattern accumulator */
+    in_pattern_accumulator = (dev_proc(dev, dev_spec_op)(dev, gxdso_in_pattern_accumulator, NULL, 0));
+    if (in_pattern_accumulator < 0)
+        in_pattern_accumulator = 0;
+
+    /* Figure out if we are orthogonal */
+    if (mat.xy == 0 && mat.yx == 0)
+        orthogonal = 1;
+    else if (mat.xx == 0 && mat.yy == 0)
+        orthogonal = 2;
+    else
+        orthogonal = 0;
+
+    /* If we are in a pattern accumulator, we choose to always grid fit
+     * orthogonal images. We do this by asking the device whether we
+     * should grid fit. This allows us to avoid nasty blank lines around
+     * the edges of cells.
+     */
+    gridfitimages = in_pattern_accumulator && orthogonal;
+
+    if (pgs != NULL && pgs->show_gstate != NULL) {
         /* If we're a graphics state, and we're in a text object, then we
          * must be in a type3 font. Don't fiddle with it. */
-    } else if (!penum->masked || penum->image_parent_type != 0) {
-        /* We only grid fit for ImageMasks, currently */
-    } else if (pis != NULL && pis->fill_adjust.x == 0 && pis->fill_adjust.y == 0) {
+    } else if (!gridfitimages &&
+               (!penum->masked || penum->image_parent_type != 0)) {
+        /* Other than for images we are specifically looking to grid fit (such as
+         * ones in a pattern device), we only grid fit imagemasks */
+    } else if (gridfitimages && (penum->masked && penum->image_parent_type == 0)) {
+        /* We don't gridfit imagemasks in a pattern accumulator */
+    } else if (pgs != NULL && pgs->fill_adjust.x == 0 && pgs->fill_adjust.y == 0) {
         /* If fill adjust is disabled, so is grid fitting */
-    } else if (mat.xy == 0 && mat.yx == 0) {
-        if (width == 1) {
+    } else if (orthogonal == 1) {
+        if (width == 1 || gridfitimages) {
             if (mat.xx > 0) {
                 fixed ix0 = int2fixed(fixed2int(float2fixed(mat.tx)));
                 double x1 = mat.tx + mat.xx * width;
@@ -296,7 +356,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.xx = (double)(fixed2float(ix1 - ix0)/width);
             }
         }
-        if (height == 1) {
+        if (height == 1 || gridfitimages) {
             if (mat.yy > 0) {
                 fixed iy0 = int2fixed(fixed2int(float2fixed(mat.ty)));
                 double y1 = mat.ty + mat.yy * height;
@@ -311,8 +371,8 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.yy = ((double)fixed2float(iy1 - iy0)/height);
             }
         }
-    } else if (mat.xx == 0 && mat.yy == 0) {
-        if (height == 1) {
+    } else if (orthogonal == 2) {
+        if (height == 1 || gridfitimages) {
             if (mat.yx > 0) {
                 fixed ix0 = int2fixed(fixed2int(float2fixed(mat.tx)));
                 double x1 = mat.tx + mat.yx * height;
@@ -327,7 +387,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mat.yx = (double)(fixed2float(ix1 - ix0)/height);
             }
         }
-        if (width == 1) {
+        if (width == 1 || gridfitimages) {
             if (mat.xy > 0) {
                 fixed iy0 = int2fixed(fixed2int(float2fixed(mat.ty)));
                 double y1 = mat.ty + mat.xy * width;
@@ -344,6 +404,111 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         }
     }
 
+    /* When rendering to a pattern accumulator, if we are downscaling
+     * then enable interpolation, as otherwise dropouts can cause
+     * serious problems. */
+    if (in_pattern_accumulator) {
+        double ome = ((double)(fixed_1 - fixed_epsilon)) / (double)fixed_1; /* One Minus Epsilon */
+
+        if (orthogonal == 1) {
+            if ((mat.xx > -ome && mat.xx < ome) || (mat.yy > -ome && mat.yy < ome)) {
+                force_interpolation = true;
+            }
+        } else if (orthogonal == 2) {
+            if ((mat.xy > -ome && mat.xy < ome) || (mat.yx > -ome && mat.yx < ome)) {
+                force_interpolation = true;
+            }
+        }
+    }
+
+    /* Can we restrict the amount of image we need? */
+    while (pcpath) /* So we can break out of it */
+    {
+        gs_rect rect, rect_out;
+        gs_matrix mi;
+        gs_fixed_rect obox;
+        gs_int_rect irect;
+        if ((code = gs_matrix_invert(&ctm_only(pgs), &mi)) < 0 ||
+            (code = gs_matrix_multiply(&mi, &pic->ImageMatrix, &mi)) < 0) {
+            /* Give up trying to shrink the render box, but continue processing */
+            break;
+        }
+        gx_cpath_outer_box(pcpath, &obox);
+        rect.p.x = fixed2float(obox.p.x);
+        rect.p.y = fixed2float(obox.p.y);
+        rect.q.x = fixed2float(obox.q.x);
+        rect.q.y = fixed2float(obox.q.y);
+        code = gs_bbox_transform(&rect, &mi, &rect_out);
+        if (code < 0) {
+            /* Give up trying to shrink the render box, but continue processing */
+            break;
+        }
+        irect.p.x = (int)(rect_out.p.x-1.0);
+        irect.p.y = (int)(rect_out.p.y-1.0);
+        irect.q.x = (int)(rect_out.q.x+1.0);
+        irect.q.y = (int)(rect_out.q.y+1.0);
+        /* Need to expand the region to allow for the fact that the mitchell
+         * scaler reads multiple pixels in. This calculation can probably be
+         * improved. */
+        /* If mi.{xx,yy} > 1 then we are downscaling. During downscaling,
+         * the support increases to ensure that we don't lose pixels contributions
+         * entirely. */
+        /* I do not understand the need for the +/- 1 fudge factors,
+         * but they seem to be required. Increasing the render rectangle can
+         * never be bad at least... RJW */
+        {
+            float support = any_abs(mi.xx);
+            int isupport;
+            if (any_abs(mi.yy) > support)
+                support = any_abs(mi.yy);
+            if (any_abs(mi.xy) > support)
+                support = any_abs(mi.xy);
+            if (any_abs(mi.yx) > support)
+                support = any_abs(mi.yx);
+            isupport = (int)(MAX_ISCALE_SUPPORT * (support+1)) + 1;
+            irect.p.x -= isupport;
+            irect.p.y -= isupport;
+            irect.q.x += isupport;
+            irect.q.y += isupport;
+        }
+        if (penum->rrect.x < irect.p.x) {
+            penum->rrect.w -= irect.p.x - penum->rrect.x;
+            if (penum->rrect.w < 0)
+               penum->rrect.w = 0;
+            penum->rrect.x = irect.p.x;
+        }
+        if (penum->rrect.x + penum->rrect.w > irect.q.x) {
+            penum->rrect.w = irect.q.x - penum->rrect.x;
+            if (penum->rrect.w < 0)
+                penum->rrect.w = 0;
+        }
+        if (penum->rrect.y < irect.p.y) {
+            penum->rrect.h -= irect.p.y - penum->rrect.y;
+            if (penum->rrect.h < 0)
+                penum->rrect.h = 0;
+            penum->rrect.y = irect.p.y;
+        }
+        if (penum->rrect.y + penum->rrect.h > irect.q.y) {
+            penum->rrect.h = irect.q.y - penum->rrect.y;
+            if (penum->rrect.h < 0)
+                penum->rrect.h = 0;
+        }
+        break; /* Out of the while */
+    }
+    /* Check for the intersection being null */
+    if (penum->rrect.x + penum->rrect.w <= penum->rect.x  ||
+        penum->rect.x  + penum->rect.w  <= penum->rrect.x ||
+        penum->rrect.y + penum->rrect.h <= penum->rect.y  ||
+        penum->rect.y  + penum->rect.h  <= penum->rrect.y)
+    {
+          /* Something may have gone wrong with the floating point above.
+           * set the region to something sane. */
+        penum->rrect.x = penum->rect.x;
+        penum->rrect.y = penum->rect.y;
+        penum->rrect.w = 0;
+        penum->rrect.h = 0;
+    }
+
     /*penum->matrix = mat;*/
     penum->matrix.xx = mat.xx;
     penum->matrix.xy = mat.xy;
@@ -351,7 +516,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     penum->matrix.yy = mat.yy;
     penum->matrix.tx = mat.tx;
     penum->matrix.ty = mat.ty;
-    if_debug6('b', " [%g %g %g %g %g %g]\n",
+    if_debug6m('b', mem, " [%g %g %g %g %g %g]\n",
               mat.xx, mat.xy, mat.yx, mat.yy, mat.tx, mat.ty);
     /* following works for 1, 2, 4, 8, 12, 16 */
     index_bps = (bps < 8 ? bps >> 1 : (bps >> 2) + 1);
@@ -373,14 +538,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         mty = float2fixed(mat.ty + f) - int2fixed(f);
     }
 
-    row_extent.x = float2fixed_rounded(width * mat.xx);
+    row_extent.x = float2fixed_rounded_boxed(width * mat.xx);
     row_extent.y =
         (is_fzero(mat.xy) ? fixed_0 :
-         float2fixed_rounded(width * mat.xy));
+         float2fixed_rounded_boxed(width * mat.xy));
     col_extent.x =
         (is_fzero(mat.yx) ? fixed_0 :
-         float2fixed_rounded(height * mat.yx));
-    col_extent.y = float2fixed_rounded(height * mat.yy);
+         float2fixed_rounded_boxed(height * mat.yx));
+    col_extent.y = float2fixed_rounded_boxed(height * mat.yy);
     gx_image_enum_common_init((gx_image_enum_common_t *)penum,
                               (const gs_data_image_t *)pim,
                               &image1_enum_procs, dev,
@@ -392,14 +557,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     } else {
         int rw = penum->rect.w, rh = penum->rect.h;
 
-        x_extent.x = float2fixed_rounded(rw * mat.xx);
+        x_extent.x = float2fixed_rounded_boxed(rw * mat.xx);
         x_extent.y =
             (is_fzero(mat.xy) ? fixed_0 :
-             float2fixed_rounded(rw * mat.xy));
+             float2fixed_rounded_boxed(rw * mat.xy));
         y_extent.x =
             (is_fzero(mat.yx) ? fixed_0 :
-             float2fixed_rounded(rh * mat.yx));
-        y_extent.y = float2fixed_rounded(rh * mat.yy);
+             float2fixed_rounded_boxed(rh * mat.yx));
+        y_extent.y = float2fixed_rounded_boxed(rh * mat.yy);
     }
     /* Set icolor0 and icolor1 to point to image clues locations if we have
        1spp or an imagemask, otherwise image clues is not used and
@@ -459,24 +624,26 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         if (pcs->cmm_icc_profile_data != NULL) {
             device_color = false;
         } else {
-            device_color = (*pcst->concrete_space) (pcs, pis) == pcs;
+            device_color = (*pcst->concrete_space) (pcs, pgs) == pcs;
         }
 
-        image_init_colors(penum, bps, spp, format, decode, pis, dev,
+        code = image_init_colors(penum, bps, spp, format, decode, pgs, dev,
                           pcs, &device_color);
+        if (code < 0) 
+            return gs_throw(code, "Image colors initialization failed");
         /* If we have a CIE based color space and the icc equivalent profile
            is not yet set, go ahead and handle that now.  It may already
            be done due to the above init_colors which may go through remap. */
         if (gs_color_space_is_PSCIE(pcs) && pcs->icc_equivalent == NULL) {
-            gs_colorspace_set_icc_equivalent(pcs, &(penum->icc_setup.is_lab),
-                                                pis->memory);
+            gs_colorspace_set_icc_equivalent((gs_color_space *)pcs, &(penum->icc_setup.is_lab),
+                                                pgs->memory);
             if (penum->icc_setup.is_lab) {
                 /* Free what ever profile was created and use the icc manager's
                    cielab profile */
-                gs_color_space *curr_pcs = pcs;
+                gs_color_space *curr_pcs = (gs_color_space *)pcs;
                 rc_decrement(curr_pcs->icc_equivalent,"gx_image_enum_begin");
                 rc_decrement(curr_pcs->cmm_icc_profile_data,"gx_image_enum_begin");
-                curr_pcs->cmm_icc_profile_data = pis->icc_manager->lab_profile;
+                curr_pcs->cmm_icc_profile_data = pgs->icc_manager->lab_profile;
                 rc_increment(curr_pcs->cmm_icc_profile_data);
             }
         }
@@ -576,14 +743,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
      * the given sub-image, or else is constructing output out of
      * overlapping pieces.
      */
-    penum->interpolate = pim->Interpolate;
+    penum->interpolate = force_interpolation ? interp_force : pim->Interpolate ? interp_on : interp_off;
     penum->x_extent = x_extent;
     penum->y_extent = y_extent;
     penum->posture =
         ((x_extent.y | y_extent.x) == 0 ? image_portrait :
          (x_extent.x | y_extent.y) == 0 ? image_landscape :
          image_skewed);
-    penum->pis = pis;
+    penum->pgs = pgs;
     penum->pcs = pcs;
     penum->memory = mem;
     penum->buffer = buffer;
@@ -593,14 +760,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     penum->color_cache = NULL;
     penum->ht_buffer = NULL;
     penum->thresh_buffer = NULL;
-    penum->cie_range = NULL;
+    penum->use_cie_range = false;
     penum->line_size = 0;
     penum->use_rop = lop != (masked ? rop3_T : rop3_S);
 #ifdef DEBUG
     if (gs_debug_c('*')) {
         if (penum->use_rop)
-            dprintf1("[%03x]", lop);
-        dprintf5("%c%d%c%dx%d ",
+            dmprintf1(mem, "[%03x]", lop);
+        dmprintf5(mem, "%c%d%c%dx%d ",
                  (masked ? (color_is_pure(pdcolor) ? 'm' : 'h') : 'i'),
                  bps,
                  (penum->posture == image_portrait ? ' ' :
@@ -672,20 +839,28 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                 mty = (((mty + diff) | fixed_half) & -fixed_half) - diff;
             }
         }
-        if_debug5('b', "[b]Image: %sspp=%d, bps=%d, mt=(%g,%g)\n",
-                  (masked? "masked, " : ""), spp, bps,
-                  fixed2float(mtx), fixed2float(mty));
-        if_debug9('b',
-                  "[b]   cbox=(%g,%g),(%g,%g), obox=(%g,%g),(%g,%g), clip_image=0x%x\n",
-                  fixed2float(cbox.p.x), fixed2float(cbox.p.y),
-                  fixed2float(cbox.q.x), fixed2float(cbox.q.y),
-                  fixed2float(obox.p.x), fixed2float(obox.p.y),
-                  fixed2float(obox.q.x), fixed2float(obox.q.y),
-                  penum->clip_image);
+        if_debug5m('b', mem, "[b]Image: %sspp=%d, bps=%d, mt=(%g,%g)\n",
+                   (masked? "masked, " : ""), spp, bps,
+                   fixed2float(mtx), fixed2float(mty));
+        if_debug9m('b', mem,
+                   "[b]   cbox=(%g,%g),(%g,%g), obox=(%g,%g),(%g,%g), clip_image=0x%x\n",
+                   fixed2float(cbox.p.x), fixed2float(cbox.p.y),
+                   fixed2float(cbox.q.x), fixed2float(cbox.q.y),
+                   fixed2float(obox.p.x), fixed2float(obox.p.y),
+                   fixed2float(obox.q.x), fixed2float(obox.q.y),
+                   penum->clip_image);
+        /* These DDAs enumerate the starting position of each source pixel
+         * row in device space. */
         dda_init(penum->dda.row.x, mtx, col_extent.x, height);
         dda_init(penum->dda.row.y, mty, col_extent.y, height);
-        penum->dst_width = row_extent.x;
-        penum->dst_height = col_extent.y;
+        if (penum->posture == image_portrait) {
+            penum->dst_width = row_extent.x;
+            penum->dst_height = col_extent.y;
+        } else {
+            penum->dst_width = col_extent.x;
+            penum->dst_height = row_extent.y;
+        }
+        /* For gs_image_class_0_interpolate. */
         penum->yi0 = fixed2int_pixround_perfect(dda_current(penum->dda.row.y)); /* For gs_image_class_0_interpolate. */
         if (penum->rect.y) {
             int y = penum->rect.y;
@@ -697,12 +872,16 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         }
         penum->cur.x = penum->prev.x = dda_current(penum->dda.row.x);
         penum->cur.y = penum->prev.y = dda_current(penum->dda.row.y);
+        /* These DDAs enumerate the starting positions of each row of our
+         * source pixel data, in the subrectangle ('strip') that we are
+         * actually rendering. */
         dda_init(penum->dda.strip.x, penum->cur.x, row_extent.x, width);
         dda_init(penum->dda.strip.y, penum->cur.y, row_extent.y, width);
         if (penum->rect.x) {
             dda_advance(penum->dda.strip.x, penum->rect.x);
             dda_advance(penum->dda.strip.y, penum->rect.x);
-        } {
+        }
+        {
             fixed ox = dda_current(penum->dda.strip.x);
             fixed oy = dda_current(penum->dda.strip.y);
 
@@ -725,18 +904,16 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
         static sample_unpack_proc_t procs[2][6] = {
         {   sample_unpack_1, sample_unpack_2,
             sample_unpack_4, sample_unpack_8,
-            0, 0
+            sample_unpack_12, sample_unpack_16
         },
         {   sample_unpack_1_interleaved, sample_unpack_2_interleaved,
             sample_unpack_4_interleaved, sample_unpack_8_interleaved,
-            0, 0
+            sample_unpack_12, sample_unpack_16
         }};
         int num_planes = penum->num_planes;
         bool interleaved = (num_planes == 1 && penum->plane_depths[0] != penum->bps);
         int i;
 
-        procs[0][4] = procs[1][4] = sample_unpack_12_proc;
-        procs[0][5] = procs[1][5] = sample_unpack_16_proc;
         if (interleaved) {
             int num_components = penum->plane_depths[0] / penum->bps;
 
@@ -748,17 +925,9 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
             if (i == num_components)
                 interleaved = false; /* Use single table. */
         }
-        if (index_bps >= 4) {
-            if ((penum->unpack = procs[interleaved][index_bps]) == 0) {         /* bps case not supported. */
-                gx_default_end_image(dev,
-                                     (gx_image_enum_common_t *) penum,
-                                     false);
-                return_error(gs_error_rangecheck);
-            }
-        } else {
-            penum->unpack = procs[interleaved][index_bps];
-        }
-        if_debug1('b', "[b]unpack=%d\n", bps);
+        penum->unpack = procs[interleaved][index_bps];
+
+        if_debug1m('b', mem, "[b]unpack=%d\n", bps);
         /* Set up pixel0 for image class procedures. */
         penum->dda.pixel0 = penum->dda.strip;
         for (i = 0; i < gx_image_class_table_count; ++i)
@@ -795,6 +964,10 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
                                  false);
             return code;
         }
+        /* The 'target' must not be NULL for gx_make_rop_texture_device */
+        if (!penum->clip_dev && !dev)
+            return_error(gs_error_undefined);
+
         gx_make_rop_texture_device(rtdev,
                                    (penum->clip_dev != 0 ?
                                     (gx_device *) penum->clip_dev :
@@ -917,7 +1090,6 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
                    num_des_comp * num_entries * sizeof(byte), "image_init_color_cache");
     penum->color_cache->is_transparent = (bool*) gs_alloc_bytes(penum->memory,
              num_entries * sizeof(bool), "image_init_color_cache");
-    penum->color_cache->free_contone = true;
     /* Initialize */
     memset(penum->color_cache->is_transparent,0,num_entries * sizeof(bool));
     /* Depending upon if we need decode and ICC CM, fill the cache a couple
@@ -955,7 +1127,7 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
                     for (kk = 0; kk < num_des_comp; kk++) {
                         conc[kk] = gx_color_value_from_byte(psrc[kk]);
                     }
-                    cmap_transfer(&(conc[0]), penum->pis, penum->dev);
+                    cmap_transfer(&(conc[0]), penum->pgs, penum->dev);
                     for (kk = 0; kk < num_des_comp; kk++) {
                         psrc[kk] = gx_color_value_to_byte(conc[kk]);
                     }
@@ -965,21 +1137,10 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
             }
         } else {
             /* Indexing only.  No CM, decode or transfer functions. */
-            if (penum->pcs->params.indexed.use_proc) {
-                /* Have to do the slow way */
-                for (k = 0; k < num_entries; k++) {
-                    gs_cspace_indexed_lookup_bytes(penum->pcs, (float)k, psrc);
-                    memcpy(&(penum->color_cache->device_contone[k * num_des_comp]),
-                               psrc, num_des_comp);
-                }
-            } else {
-                /* Possible GC issue? In this case we just use the pointer
-                   of the index table data directly for our cache */
-                gs_free_object(penum->memory, penum->color_cache->device_contone,
-                                "image_init_color_cache(device_contone)");
-                penum->color_cache->device_contone =
-                    (byte*) penum->pcs->params.indexed.lookup.table.data;
-                penum->color_cache->free_contone = false;
+            for (k = 0; k < num_entries; k++) {
+                gs_cspace_indexed_lookup_bytes(penum->pcs, (float)k, psrc);
+                memcpy(&(penum->color_cache->device_contone[k * num_des_comp]),
+                           psrc, num_des_comp);
             }
         }
     } else {
@@ -992,7 +1153,7 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
                                              "image_init_color_cache");
         if (need_decode) {
             if (is_indexed) {
-                /* Decode and un-indexed */
+                /* Decode and lookup in index */
                 for (k = 0; k < num_entries; k++) {
                     image_cache_decode(penum, k, &value, decode_scale);
                     gs_cspace_indexed_lookup_bytes(penum->pcs, value, psrc);
@@ -1007,18 +1168,23 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
         } else {
             /* No Decode */
             if (is_indexed) {
-                /* If index uses a table then just use its pointer */
-                if (penum->pcs->params.indexed.use_proc) {
+                /* If index uses a num_entries sized table then just use its pointer */
+                if (penum->pcs->params.indexed.use_proc ||
+                    penum->pcs->params.indexed.hival < (num_entries - 1)) {
                     /* Have to do the slow way */
-                    for (k = 0; k < num_entries; k++) {
+                    for (k = 0; k <= penum->pcs->params.indexed.hival; k++) {
                         gs_cspace_indexed_lookup_bytes(penum->pcs, (float)k, psrc);
                         memcpy(&(temp_buffer[k * num_src_comp]), psrc, num_src_comp);
                     }
+                    /* just use psrc results from converting 'hival' to fill the remaining slots */
+                    for (; k < num_entries; k++) {
+                        memcpy(&(temp_buffer[k * num_src_comp]), psrc, num_src_comp);
+                    }
                 } else {
                     /* Use the index table directly. */
                     gs_free_object(penum->memory, temp_buffer, "image_init_color_cache");
                     free_temp_buffer = false;
-                    temp_buffer = penum->pcs->params.indexed.lookup.table.data;
+                    temp_buffer = (byte *)(penum->pcs->params.indexed.lookup.table.data);
                 }
             } else {
                 /* CM only */
@@ -1045,7 +1211,7 @@ image_init_color_cache(gx_image_enum * penum, int bps, int spp)
                 for (kk = 0; kk < num_des_comp; kk++) {
                     conc[kk] = gx_color_value_from_byte(byte_ptr[kk]);
                 }
-                cmap_transfer(&(conc[0]), penum->pis, penum->dev);
+                cmap_transfer(&(conc[0]), penum->pgs, penum->dev);
                 for (kk = 0; kk < num_des_comp; kk++) {
                     byte_ptr[kk] = gx_color_value_to_byte(conc[kk]);
                 }
@@ -1099,20 +1265,20 @@ image_init_clues(gx_image_enum * penum, int bps, int spp)
 }
 
 /* Initialize the color mapping tables for a non-mask image. */
-static void
+static int
 image_init_colors(gx_image_enum * penum, int bps, int spp,
                   gs_image_format_t format, const float *decode /*[spp*2] */ ,
-                  const gs_imager_state * pis, gx_device * dev,
+                  const gs_gstate * pgs, gx_device * dev,
                   const gs_color_space * pcs, bool * pdcb)
 {
-    int ci, decode_type;
+    int ci, decode_type, code;
     static const float default_decode[] = {
         0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0
     };
 
     /* Clues are only used with image_mono_render */
     if (spp == 1) {
-    image_init_clues(penum, bps, spp);
+        image_init_clues(penum, bps, spp);
     }
     decode_type = 3; /* 0=custom, 1=identity, 2=inverted, 3=impossible */
     for (ci = 0; ci < spp; ci +=2 ) {
@@ -1195,26 +1361,33 @@ image_init_colors(gx_image_enum * penum, int bps, int spp,
             for (i = 15 - step; i > 0; i -= step)
                 pmap->decode_lookup[i] = pmap->decode_base +
                     i * (255.0 / 15) * pmap->decode_factor;
-        } else
+            pmap->inverted = 0;
+        } else {
             pmap->decoding = sd_compute;
+            pmap->inverted = 0;
+        }
         if (spp == 1) {         /* and ci == 0 *//* Pre-map entries 0 and 255. */
             gs_client_color cc;
 
             /* Image clues are used in this case */
             cc.paint.values[0] = real_decode[0];
-            (*pcs->type->remap_color) (&cc, pcs, penum->icolor0,
-                                       pis, dev, gs_color_select_source);
+            code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor0,
+                                       pgs, dev, gs_color_select_source);
+            if (code < 0) 
+                return code;
             cc.paint.values[0] = real_decode[1];
-            (*pcs->type->remap_color) (&cc, pcs, penum->icolor1,
-                                       pis, dev, gs_color_select_source);
+            code = (*pcs->type->remap_color) (&cc, pcs, penum->icolor1,
+                                       pgs, dev, gs_color_select_source);
+            if (code < 0) 
+                return code;
         }
     }
-
+    return 0;
 }
 /* Construct a mapping table for sample values. */
 /* map_size is 2, 4, 16, or 256.  Note that 255 % (map_size - 1) == 0, */
 /* so the division 0xffffL / (map_size - 1) is always exact. */
-static void
+void
 image_init_map(byte * map, int map_size, const float *decode)
 {
     float min_v = decode[0];
@@ -1261,12 +1434,12 @@ gx_image_scale_mask_colors(gx_image_enum *penum, int component_index)
 
 /* Used to indicate for ICC procesing if we have decoding to do */
 bool
-gx_has_transfer(const gs_imager_state *pis, int num_comps)
+gx_has_transfer(const gs_gstate *pgs, int num_comps)
 {
     int k;
 
     for (k = 0; k < num_comps; k++) {
-        if (pis->effective_transfer[k]->proc != gs_identity_transfer) {
+        if (pgs->effective_transfer[k]->proc != gs_identity_transfer) {
             return(true);
         }
     }
