@@ -131,7 +131,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
     gs_color_space *pcs = NULL;
     cos_value_t cs_value;
     cos_value_t *pcsvalue;
-    byte palette[arch_sizeof_color_index * 2];
+    byte palette[ARCH_SIZEOF_COLOR_INDEX * 2];
     gs_image_t image;
     pdf_image_writer writer;
     pdf_stream_position_t ipos;
@@ -167,8 +167,8 @@ pdf_copy_mono(gx_device_pdf *pdev,
             if (pres == 0) {        /* Define the character in an embedded font. */
                 gs_image_t_init_mask(&image, false);
                 invert = 0xff;
-                x_offset = x - (int)show_enum->pis->current_point.x;
-                y_offset = y - (int)show_enum->pis->current_point.y;
+                x_offset = x - (int)show_enum->pgs->current_point.x;
+                y_offset = y - (int)show_enum->pgs->current_point.y;
                 x -= x_offset;
                 y -= y_offset;
                 y -= h;
