@@ -74,6 +74,7 @@ static const gs_param_item_t pdf_param_items[] = {
     pi("ReEncodeCharacters", gs_param_type_bool, ReEncodeCharacters),
     pi("FirstObjectNumber", gs_param_type_long, FirstObjectNumber),
     pi("CompressFonts", gs_param_type_bool, CompressFonts),
+    pi("CompressStreams", gs_param_type_bool, CompressStreams),
     pi("PrintStatistics", gs_param_type_bool, PrintStatistics),
     pi("MaxInlineImageSize", gs_param_type_long, MaxInlineImageSize),
     pi("DSCEncodingToUnicode", gs_param_type_int_array, DSCEncodingToUnicode),
@@ -124,11 +125,9 @@ static const gs_param_item_t pdf_param_items[] = {
     pi("PreserveSMask", gs_param_type_bool, PreserveSMask),
     pi("PreserveTrMode", gs_param_type_bool, PreserveTrMode),
     pi("NoT3CCITT", gs_param_type_bool, NoT3CCITT),
-    pi("PDFUseOldCMS", gs_param_type_bool, UseOldColor),
     pi("FastWebView", gs_param_type_bool, Linearise),
     pi("NoOutputFonts", gs_param_type_bool, FlattenFonts),
-    pi("FirstPage", gs_param_type_int, FirstPage),
-    pi("LastPage", gs_param_type_int, LastPage),
+    pi("WantsPageLabels", gs_param_type_bool, WantsPageLabels),
 #undef pi
     gs_param_item_end
 };
@@ -289,7 +288,6 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     float cl = (float)pdev->CompatibilityLevel;
     bool locked = pdev->params.LockDistillerParams, ForOPDFRead;
     gs_param_name param_name;
-    enum psdf_color_conversion_strategy save_ccs = pdev->params.ColorConversionStrategy;
 
     pdev->pdf_memory = gs_memory_stable(pdev->memory);
     /*
@@ -396,6 +394,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
         default:
             ecode = code;
             param_signal_error(plist, param_name, ecode);
+            break;
         case 0:
             if (!(locked && pdev->params.LockDistillerParams)) {
                 /*
@@ -477,9 +476,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
         ecode = param_put_enum(plist, "ProcessColorModel", &pcm,
                                pcm_names, ecode);
         if (pcm >= 0) {
-            pdf_set_process_color_model(pdev, pcm);
-            pdf_set_initial_color(pdev, &pdev->saved_fill_color, &pdev->saved_stroke_color,
-                            &pdev->fill_used_process_color, &pdev->stroke_used_process_color);
+            pdev->color_info.cm_name = pcm_names[pcm];
         }
     }
     if (ecode < 0)
@@ -496,7 +493,7 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
      * or less impossible to alter the setting in the (potentially saved) page
      * device dictionary, so we use this rather clunky method.
      */
-    if (pdev->PDFA < 0 || pdev->PDFA > 2){
+    if (pdev->PDFA < 0 || pdev->PDFA > 3){
         ecode = gs_note_error(gs_error_rangecheck);
         param_signal_error(plist, "PDFA", ecode);
         goto fail;
@@ -525,9 +522,6 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
          pdev->PreserveSMask = false;
     }
 
-    if (pdev->PDFA == 1) {
-        pdev->UseOldColor = true;
-    }
     /*
      * We have to set version to the new value, because the set of
      * legal parameter values for psdf_put_params varies according to
@@ -560,114 +554,63 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     ecode = gdev_psdf_put_params(dev, plist);
     if (ecode < 0)
         goto fail;
-    if (!pdev->UseOldColor) {
-        if (pdev->params.ConvertCMYKImagesToRGB) {
-            if (pdev->params.ColorConversionStrategy == ccs_CMYK) {
-                emprintf(pdev->memory, "ConvertCMYKImagesToRGB is not compatible with ColorConversionStrategy of CMYK\n");
+    if (pdev->params.ConvertCMYKImagesToRGB) {
+        if (pdev->params.ColorConversionStrategy == ccs_CMYK) {
+            emprintf(pdev->memory, "ConvertCMYKImagesToRGB is not compatible with ColorConversionStrategy of CMYK\n");
+        } else {
+            if (pdev->params.ColorConversionStrategy == ccs_Gray) {
+                emprintf(pdev->memory, "ConvertCMYKImagesToRGB is not compatible with ColorConversionStrategy of Gray\n");
             } else {
-                if (pdev->params.ColorConversionStrategy == ccs_Gray) {
-                    emprintf(pdev->memory, "ConvertCMYKImagesToRGB is not compatible with ColorConversionStrategy of Gray\n");
-                } else {
-                    if (pdev->icc_struct)
-                        rc_decrement(pdev->icc_struct,
-                                     "reset default profile\n");
-                    pdf_set_process_color_model(pdev,1);
-                    ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
-                    if (ecode < 0)
-                        goto fail;
-                }
-            }
-        }
-        switch (pdev->params.ColorConversionStrategy) {
-            case ccs_LeaveColorUnchanged:
-            case ccs_UseDeviceDependentColor:
-            case ccs_UseDeviceIndependentColor:
-            case ccs_UseDeviceIndependentColorForImages:
-            case ccs_ByObjectType:
-                break;
-            case ccs_CMYK:
                 if (pdev->icc_struct)
                     rc_decrement(pdev->icc_struct,
                                  "reset default profile\n");
-                pdf_set_process_color_model(pdev, 2);
+                pdf_set_process_color_model(pdev,1);
                 ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
                 if (ecode < 0)
                     goto fail;
-                break;
-            case ccs_Gray:
+            }
+        }
+    }
+    switch (pdev->params.ColorConversionStrategy) {
+        case ccs_LeaveColorUnchanged:
+        case ccs_UseDeviceDependentColor:
+        case ccs_UseDeviceIndependentColor:
+        case ccs_UseDeviceIndependentColorForImages:
+        case ccs_ByObjectType:
+            break;
+        case ccs_CMYK:
+            if (pdev->icc_struct)
+                rc_decrement(pdev->icc_struct,
+                             "reset default profile\n");
+            pdf_set_process_color_model(pdev, 2);
+            ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
+            if (ecode < 0)
+                goto fail;
+            break;
+        case ccs_Gray:
+            if (pdev->icc_struct)
+                rc_decrement(pdev->icc_struct,
+                             "reset default profile\n");
+            pdf_set_process_color_model(pdev,0);
+            ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
+            if (ecode < 0)
+                goto fail;
+            break;
+        case ccs_sRGB:
+        case ccs_RGB:
+            /* Only bother to do this if we didn't handle it above */
+            if (!pdev->params.ConvertCMYKImagesToRGB) {
                 if (pdev->icc_struct)
                     rc_decrement(pdev->icc_struct,
                                  "reset default profile\n");
-                pdf_set_process_color_model(pdev,0);
+                pdf_set_process_color_model(pdev,1);
                 ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
                 if (ecode < 0)
                     goto fail;
-                break;
-            case ccs_sRGB:
-            case ccs_RGB:
-                /* Only bother to do this if we didn't handle it above */
-                if (!pdev->params.ConvertCMYKImagesToRGB) {
-                    if (pdev->icc_struct)
-                        rc_decrement(pdev->icc_struct,
-                                     "reset default profile\n");
-                    pdf_set_process_color_model(pdev,1);
-                    ecode = gsicc_init_device_profile_struct((gx_device *)pdev, NULL, 0);
-                    if (ecode < 0)
-                        goto fail;
-                }
-                break;
-            default:
-                break;
-        }
-    } else {
-        if ((pdev->params.ColorConversionStrategy == ccs_CMYK &&
-             strcmp(pdev->color_info.cm_name, "DeviceCMYK")) ||
-            ((pdev->params.ColorConversionStrategy == ccs_sRGB ||
-             pdev->params.ColorConversionStrategy == ccs_sRGB) &&
-              strcmp(pdev->color_info.cm_name, "DeviceRGB")) ||
-            (pdev->params.ColorConversionStrategy == ccs_Gray &&
-              strcmp(pdev->color_info.cm_name, "DeviceGray"))) {
-            emprintf(pdev->memory,
-                     "ColorConversionStrategy is incompatible to ProcessColorModel.\n");
-            ecode = gs_note_error(gs_error_rangecheck);
-            pdev->params.ColorConversionStrategy = save_ccs;
-        }
-        if (pdev->params.ColorConversionStrategy == ccs_UseDeviceIndependentColor) {
-            if (!pdev->UseCIEColor) {
-                emprintf(pdev->memory,
-                         "Set UseCIEColor for UseDeviceIndependentColor to work properly.\n");
-                ecode = gs_note_error(gs_error_rangecheck);
-                pdev->UseCIEColor = true;
             }
-        }
-        if (pdev->params.ColorConversionStrategy == ccs_UseDeviceIndependentColorForImages) {
-            if (!pdev->UseCIEColor) {
-                emprintf(pdev->memory,
-                         "UseDeviceDependentColorForImages is not supported. Use UseDeviceIndependentColor.\n");
-                pdev->params.ColorConversionStrategy = ccs_UseDeviceIndependentColor;
-                if (!pdev->UseCIEColor) {
-                    emprintf(pdev->memory,
-                             "Set UseCIEColor for UseDeviceIndependentColor to work properly.\n");
-                    ecode = gs_note_error(gs_error_rangecheck);
-                    pdev->UseCIEColor = true;
-                }
-            }
-        }
-        if (pdev->params.ColorConversionStrategy == ccs_UseDeviceDependentColor) {
-            if (!strcmp(pdev->color_info.cm_name, "DeviceCMYK")) {
-                emprintf(pdev->memory,
-                         "Replacing the deprecated device parameter value UseDeviceDependentColor with CMYK.\n");
-                pdev->params.ColorConversionStrategy = ccs_CMYK;
-            } else if (!strcmp(pdev->color_info.cm_name, "DeviceRGB")) {
-                emprintf(pdev->memory,
-                         "Replacing the deprecated device parameter value UseDeviceDependentColor with sRGB.\n");
-                pdev->params.ColorConversionStrategy = ccs_sRGB;
-            } else {
-                emprintf(pdev->memory,
-                         "Replacing the deprecated device parameter value UseDeviceDependentColor with Gray.\n");
-                pdev->params.ColorConversionStrategy = ccs_Gray;
-            }
-        }
+            break;
+        default:
+            break;
     }
     if (cl < 1.5f && pdev->params.ColorImage.Filter != NULL &&
             !strcmp(pdev->params.ColorImage.Filter, "JPXEncode")) {
@@ -692,28 +635,13 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
     }
     if (ecode < 0)
         goto fail;
-    /*
-     * Acrobat Reader doesn't handle user-space coordinates larger than
-     * MAX_USER_COORD.  To compensate for this, reduce the resolution so
-     * that the page size in device space (which we equate to user space) is
-     * significantly less than MAX_USER_COORD.  Note that this still does
-     * not protect us against input files that use coordinates far outside
-     * the page boundaries.
-     */
-#define MAX_EXTENT ((int)(MAX_USER_COORD * 0.9))
-    /* Changing resolution or page size requires closing the device, */
-    if (dev->height > MAX_EXTENT || dev->width > MAX_EXTENT) {
-        double factor =
-            max(dev->height / (double)MAX_EXTENT,
-                dev->width / (double)MAX_EXTENT);
 
-        gx_device_set_resolution(dev, dev->HWResolution[0] / factor,
-                                 dev->HWResolution[1] / factor);
-    }
-#undef MAX_EXTENT
     if (pdev->FirstObjectNumber != save_dev->FirstObjectNumber) {
         if (pdev->xref.file != 0) {
-            gp_fseek_64(pdev->xref.file, 0L, SEEK_SET);
+            if (gp_fseek_64(pdev->xref.file, 0L, SEEK_SET) != 0) {
+                ecode = gs_error_ioerror;
+                goto fail;
+            }
             pdf_initialize_ids(pdev);
         }
     }
@@ -738,6 +666,11 @@ gdev_pdf_put_params_impl(gx_device * dev, const gx_device_pdf * save_dev, gs_par
         pdev->Linearise = false;
     }
 
+    if (pdev->Linearise && pdev->OwnerPassword.size != 0) {
+        emprintf(pdev->memory, "Can't linearise encrypted PDF, ignoring\n");
+        pdev->Linearise = false;
+    }
+
     if (pdev->FlattenFonts)
         pdev->PreserveTrMode = false;
     return 0;
@@ -779,6 +712,42 @@ gdev_pdf_put_params(gx_device * dev, gs_param_list * plist)
 
 /* ---------------- Process DSC comments ---------------- */
 
+/* Bug #695850 DSC comments are not actually encoded, nor are they PostScript strings
+ * they are simply a sequence of bytes. SO it would seem we should just preserve that
+ * byte sequence. Bizarrely, Distiller treats the comment as 'almost' a PostScript
+ * string. In particular it converts octal codes into an equivalent binary byte. It
+ * also converts (eg) '\n' and '\r' into 'n' and 'r' and invalid octal escapes
+ * (eg \11) simply have the '\' turned into a '?'.
+ * We think this is nuts and have no intention of trying to mimic it precisely. This
+ * routine will find octal escapes and convert them into binary. The way this works is
+ * a little obscure. The DSC parser does convert the comment into a PostScript string
+ * and so has to escape any unusual characters. This means our octal escaped values in
+ * the original DSC comment have had the escape character ('\') escaped to become '\\'.
+ * All we need to do is remove the escape of the escape and we will end up with a
+ * properly escaped PostScript string.
+ */
+static int unescape_octals(gx_device_pdf * pdev, char *src, int size)
+{
+    char *start, *dest;
+
+    start = src;
+    dest = src;
+
+    while(size) {
+        if (size > 4 && src[0] == '\\' && src[1] == '\\' &&
+                src[2] > 0x29 && src[2] < 0x35 &&
+                src[3] > 0x29 &&src[3] < 0x38 &&
+                src[4] > 0x29 && src[4] < 0x38) {
+            src++;
+            size--;
+        } else {
+            *dest++ = *src++;
+            size--;
+        }
+    }
+    return (dest - start);
+}
+
 static int
 pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
 {
@@ -799,8 +768,9 @@ pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
 
     for (i = 0; i + 1 < pma->size && code >= 0; i += 2) {
         const gs_param_string *pkey = &pma->data[i];
-        const gs_param_string *pvalue = &pma->data[i + 1];
+        gs_param_string *pvalue = (gs_param_string *)&pma->data[i + 1];
         const char *key;
+        int newsize;
 
         /*
          * %%For, %%Creator, and %%Title are recognized only if either
@@ -812,13 +782,25 @@ pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
          * but we do the same -- we ignore %%CreationDate here.
          */
 
-        if (pdf_key_eq(pkey, "Creator"))
+        if (pdf_key_eq(pkey, "Creator")) {
             key = "/Creator";
-        else if (pdf_key_eq(pkey, "Title"))
+            newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
+            code = cos_dict_put_c_key_string(pdev->Info, key,
+                                             pvalue->data, newsize);
+            continue;
+        } else if (pdf_key_eq(pkey, "Title")) {
             key = "/Title";
-        else if (pdf_key_eq(pkey, "For"))
+            newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
+            code = cos_dict_put_c_key_string(pdev->Info, key,
+                                             pvalue->data, newsize);
+            continue;
+        } else if (pdf_key_eq(pkey, "For")) {
             key = "/Author";
-        else {
+            newsize = unescape_octals(pdev, (char *)pvalue->data, pvalue->size);
+            code = cos_dict_put_c_key_string(pdev->Info, key,
+                                             pvalue->data, newsize);
+            continue;
+        } else {
             pdf_page_dsc_info_t *ppdi;
             char scan_buf[200]; /* arbitrary */
 
@@ -886,10 +868,6 @@ pdf_dsc_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
             }
             continue;
         }
-
-        if (pdev->ParseDSCCommentsForDocInfo || pdev->PreserveEPSInfo)
-            code = cos_dict_put_c_key_string(pdev->Info, key,
-                                             pvalue->data, pvalue->size);
     }
     return code;
 }
