@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2014 Artifex Software, Inc.
+/* Copyright (C) 2001-2015 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -16,13 +16,13 @@
 /* XPS output device */
 #include "string_.h"
 #include "stdio_.h"
+#include "zlib.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gdevvec.h"
 #include "gxpath.h"
 #include "gzcpath.h"
 #include "stream.h"
-#include "zlib.h"
 #include "stdint_.h"
 #include "gdevtifs.h"
 #include "gsicc_create.h"
@@ -30,9 +30,6 @@
 #include "gximdecode.h" /* Need so that we can unpack and decode */
 
 #define MAXPRINTERNAME 64
-#if defined(__WIN32__) && XPSPRINT==1
-int XPSPrint(char *FileName, char *PrinterName, int *reason);
-#endif
 
 #define MAXNAME 64
 #define PROFILEPATH "Documents/1/Resources/Profiles/"
@@ -151,14 +148,14 @@ typedef struct xps_image_enum_s {
     byte *devc_buffer; /* Needed for case where we are mapping to device colors */
     gs_color_space *pcs;     /* Needed for Sep, DeviceN, Indexed */
     gsicc_link_t *icc_link;  /* Needed for CIELAB */
-    const gs_imager_state *pis;    /* Needed for color conversions of DeviceN etc */
+    const gs_gstate *pgs;    /* Needed for color conversions of DeviceN etc */
     FILE *fid;
 } xps_image_enum_t;
 
 gs_private_st_suffix_add3(st_xps_image_enum, xps_image_enum_t,
     "xps_image_enum_t", xps_image_enum_enum_ptrs,
     xps_image_enum_reloc_ptrs, st_vector_image_enum,
-    buffer, devc_buffer, pis);
+    buffer, devc_buffer, pgs);
 
 typedef struct gx_device_xps_s {
     /* superclass state */
@@ -278,13 +275,13 @@ xps_setlogop(gx_device_vector *vdev, gs_logical_operation_t lop,
              gs_logical_operation_t diff);
 
 static int
-xps_can_handle_hl_color(gx_device_vector *vdev, const gs_imager_state *pis,
+xps_can_handle_hl_color(gx_device_vector *vdev, const gs_gstate *pgs,
                         const gx_drawing_color * pdc);
 static int
-xps_setfillcolor(gx_device_vector *vdev, const gs_imager_state *pis,
+xps_setfillcolor(gx_device_vector *vdev, const gs_gstate *pgs,
                  const gx_drawing_color *pdc);
 static int
-xps_setstrokecolor(gx_device_vector *vdev, const gs_imager_state *pis,
+xps_setstrokecolor(gx_device_vector *vdev, const gs_gstate *pgs,
                    const gx_drawing_color *pdc);
 
 static int
@@ -404,7 +401,9 @@ zip_new_info_node(gx_device_xps *xps_dev, const char *filename)
 
     if (gs_debug_c('_')) {
         gx_device_xps_f2i_t *f2i = xps_dev->f2i;
+#ifdef DEBUG
         int node = 1;
+#endif
         gx_device_xps_f2i_t *prev_f2i;
         
         while (f2i != NULL) {
@@ -701,7 +700,7 @@ static int
 zip_close_archive_file(gx_device_xps *xps_dev, const char *filename)
 {
     gx_device_xps_zinfo_t *info = zip_look_up_file_info(xps_dev, filename);
-    gx_device_xps_zdata_t data = info->data;
+    gx_device_xps_zdata_t data;
     byte buf[4];
     unsigned long crc = 0;
     int count = 0;
@@ -716,7 +715,8 @@ zip_close_archive_file(gx_device_xps *xps_dev, const char *filename)
     if (info->saved)
         return 0;
 
-    if (data.count >= 0) {
+    data = info->data;
+    if ((int)data.count >= 0) {
         FILE *fp = data.fp;
         uint nread;
 
@@ -881,8 +881,8 @@ xps_setfillbrush(gx_device_xps *xps, xps_brush_t type)
 static int
 xps_open_device(gx_device *dev)
 {
-    gx_device_vector *const vdev = (gx_device_vector*)dev;
-    gx_device_xps *const xps = (gx_device_xps*)dev;
+    gx_device_vector * vdev = (gx_device_vector*)dev;
+    gx_device_xps * xps = (gx_device_xps*)dev;
     int code = 0;
 
     vdev->v_memory = dev->memory;
@@ -893,6 +893,14 @@ xps_open_device(gx_device *dev)
     if (code < 0)
       return gs_rethrow_code(code);
 
+    /* In case ths device has been subclassed, descend to the terminal
+     * of the chain. (Setting the variables in a subclassing device
+     * doesn't do anythign useful.....)
+     */
+    while(vdev->child)
+        vdev = (gx_device_vector *)vdev->child;
+    xps = (gx_device_xps*)vdev;
+
     /* xps-specific initialization goes here */
     xps->page_count = 0;
     xps->relationship_count = 0;
@@ -1061,7 +1069,7 @@ xps_release_icc_info(gx_device *dev)
 static int
 xps_close_device(gx_device *dev)
 {
-    const gx_device_xps *xps = (gx_device_xps*)dev;
+    gx_device_xps *xps = (gx_device_xps*)dev;
     int code;
 
     /* closing for the FixedDocument */
@@ -1079,14 +1087,13 @@ xps_close_device(gx_device *dev)
     /* Release the icc info */
     xps_release_icc_info(dev);
 
-#if defined(__WIN32__) && XPSPRINT==1
     code = gdev_vector_close_file((gx_device_vector*)dev);
     if (code < 0)
         return gs_rethrow_code(code);
 
     if (strlen((const char *)xps->PrinterName)) {
         int reason;
-        code = XPSPrint(xps->fname, (char *)xps->PrinterName, &reason);
+        code = gp_xpsprint(xps->fname, (char *)xps->PrinterName, &reason);
         if (code < 0) {
             switch(code) {
                 case -1:
@@ -1137,11 +1144,8 @@ xps_close_device(gx_device *dev)
             }
             return(gs_throw_code(gs_error_invalidaccess));
         }
-        return(0);
     }
-#else
-    return gdev_vector_close_file((gx_device_vector*)dev);
-#endif
+    return(0);
 }
 
 /* Respond to a device parameter query from the client */
@@ -1213,7 +1217,7 @@ xps_put_params(gx_device *dev, gs_param_list *plist)
 
 static int xps_finish_copydevice(gx_device *dev, const gx_device *from_dev)
 {
-    const gx_device_xps *xps = (gx_device_xps*)dev;
+    gx_device_xps *xps = (gx_device_xps*)dev;
 
     memset(xps->PrinterName, 0x00, MAXPRINTERNAME);
     return 0;
@@ -1238,7 +1242,7 @@ set_state_color(gx_device_vector *vdev, const gx_drawing_color *pdc, gx_color_in
 }
 
 static int
-xps_setfillcolor(gx_device_vector *vdev, const gs_imager_state *pis, const gx_drawing_color *pdc)
+xps_setfillcolor(gx_device_vector *vdev, const gs_gstate *pgs, const gx_drawing_color *pdc)
 {
     gx_device_xps *xps = (gx_device_xps *)vdev;
 
@@ -1248,7 +1252,7 @@ xps_setfillcolor(gx_device_vector *vdev, const gs_imager_state *pis, const gx_dr
 }
 
 static int
-xps_setstrokecolor(gx_device_vector *vdev, const gs_imager_state *pis, const gx_drawing_color *pdc)
+xps_setstrokecolor(gx_device_vector *vdev, const gs_gstate *pgs, const gx_drawing_color *pdc)
 {
     gx_device_xps *xps = (gx_device_xps *)vdev;
     
@@ -1329,10 +1333,13 @@ static int
 xps_setlinecap(gx_device_vector *vdev, gs_line_cap cap)
 {
     gx_device_xps *xps = (gx_device_xps *)vdev;
+#ifdef DEBUG
+    /* Only used for debug print, so guiard to prevent warnings on non-debug builds */
     const char *linecap_names[] = {"butt", "round", "square",
         "triangle", "unknown"};
+#endif
 
-    if (cap < 0 || cap > gs_cap_unknown)
+    if ((int)cap < 0 || (int)cap > gs_cap_unknown)
         return gs_throw_code(gs_error_rangecheck);
     if_debug1m('_', xps->memory, "xps_setlinecap(%s)\n", linecap_names[cap]);
 
@@ -1344,10 +1351,13 @@ static int
 xps_setlinejoin(gx_device_vector *vdev, gs_line_join join)
 {
     gx_device_xps *xps = (gx_device_xps *)vdev;
+#ifdef DEBUG
+    /* Only used for debug print, so guiard to prevent warnings on non-debug builds */
     const char *linejoin_names[] = {"miter", "round", "bevel",
         "none", "triangle", "unknown"};
+#endif
 
-    if (join < 0 || join > gs_join_unknown)
+    if ((int)join < 0 || (int)join > gs_join_unknown)
         return gs_throw_code(gs_error_rangecheck);
     if_debug1m('_', xps->memory, "xps_setlinejoin(%s)\n", linejoin_names[join]);
 
@@ -1383,7 +1393,7 @@ xps_setlogop(gx_device_vector *vdev, gs_logical_operation_t lop,
         /* Other state */
 
 static int
-xps_can_handle_hl_color(gx_device_vector *vdev, const gs_imager_state *pis,
+xps_can_handle_hl_color(gx_device_vector *vdev, const gs_gstate *pgs,
                           const gx_drawing_color *pdc)
 {
     if_debug0m('_', vdev->memory, "xps_can_handle_hl_color\n");
@@ -1512,25 +1522,25 @@ xps_dorect(gx_device_vector *vdev, fixed x0, fixed y0,
 }
 
 static int
-gdev_xps_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
+gdev_xps_fill_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                    const gx_fill_params * params,
                    const gx_drawing_color * pdcolor, const gx_clip_path * pcpath)
 {
     if (gx_path_is_void(ppath)) {
         return 0;
     }
-    return gdev_vector_fill_path(dev, pis, ppath, params, pdcolor, pcpath);
+    return gdev_vector_fill_path(dev, pgs, ppath, params, pdcolor, pcpath);
 }
 
 static int
-gdev_xps_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
+gdev_xps_stroke_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                      const gx_stroke_params * params,
                      const gx_drawing_color * pdcolor, const gx_clip_path * pcpath)
 {
     if (gx_path_is_void(ppath)) {
         return 0;
     }
-    return gdev_vector_stroke_path(dev, pis, ppath, params, pdcolor, pcpath);
+    return gdev_vector_stroke_path(dev, pgs, ppath, params, pdcolor, pcpath);
 }
            
 static int
@@ -1763,20 +1773,20 @@ xps_add_image_relationship(xps_image_enum_t *pie)
 }
 
 static int
-xps_write_profile(const gs_imager_state *pis, char *name, cmm_profile_t *profile, gx_device_xps *xps_dev)
+xps_write_profile(const gs_gstate *pgs, char *name, cmm_profile_t *profile, gx_device_xps *xps_dev)
 {
     byte *profile_buffer;
     int size;
 
     /* Need V2 ICC Profile */
-    profile_buffer = gsicc_create_getv2buffer(pis, profile, &size);
+    profile_buffer = gsicc_create_getv2buffer(pgs, profile, &size);
 
     /* Now go ahead and add to the zip archive */
     return add_data_to_zip_file(xps_dev, name, profile_buffer, size);
 }
 
 static int
-xps_begin_image(gx_device *dev, const gs_imager_state *pis, 
+xps_begin_image(gx_device *dev, const gs_gstate *pgs, 
                 const gs_image_t *pim, gs_image_format_t format, 
                 const gs_int_rect *prect, const gx_drawing_color *pdcolor,
                 const gx_clip_path *pcpath, gs_memory_t *mem,
@@ -1809,14 +1819,15 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
     if (csindex == gs_color_space_index_Indexed && pim->BitsPerComponent != 8)
         goto use_default;
 
-    /* Also need imager state for these color spaces */
-    if (pis == NULL && (csindex == gs_color_space_index_Indexed ||
+    /* Also need  gs_gstate for these color spaces */
+    if (pgs == NULL && (csindex == gs_color_space_index_Indexed ||
         csindex == gs_color_space_index_Separation ||
         csindex == gs_color_space_index_DeviceN))
         goto use_default;
 
     gs_matrix_invert(&pim->ImageMatrix, &mat);
-    gs_matrix_multiply(&mat, &ctm_only(pis), &mat);
+    if (pgs)
+        gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
 
     pie = gs_alloc_struct(mem, xps_image_enum_t, &st_xps_image_enum,
                           "xps_begin_image");
@@ -1824,7 +1835,7 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
         return_error(gs_error_VMerror);
     pie->buffer = NULL;
     pie->devc_buffer = NULL;
-    pie->pis = NULL;
+    pie->pgs = NULL;
 
     /* Set the brush types to image */
     xps_setstrokebrush(xdev, xps_imagebrush);
@@ -1855,7 +1866,9 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
         if (gs_color_space_is_PSCIE(pcs)) {
             if (pcs->icc_equivalent == NULL) {
                 bool is_lab;
-                gs_colorspace_set_icc_equivalent(pcs, &is_lab, pis->memory);
+                if (pgs == NULL)
+                    return(gs_error_invalidaccess);
+                gs_colorspace_set_icc_equivalent(pcs, &is_lab, pgs->memory);
             }
             icc_profile = pcs->icc_equivalent->cmm_icc_profile_data;
         } else {
@@ -1873,9 +1886,11 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
         rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
         rendering_params.rendering_intent = gsPERCEPTUAL;
         rendering_params.cmm = gsCMM_DEFAULT;
-        pie->icc_link = gsicc_get_link_profile(pis, dev, icc_profile,
-            pis->icc_manager->default_rgb, &rendering_params, pis->memory, false);
-        icc_profile = pis->icc_manager->default_rgb;
+        if (pgs == NULL)
+            return(gs_error_invalidaccess);
+        pie->icc_link = gsicc_get_link_profile(pgs, dev, icc_profile,
+            pgs->icc_manager->default_rgb, &rendering_params, pgs->memory, false);
+        icc_profile = pgs->icc_manager->default_rgb;
     } else {
         pie->icc_link = NULL;
     }
@@ -1885,8 +1900,10 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
     if (xps_find_icc(xdev, icc_profile) == NULL) {
         icc_data = (xps_icc_data_t*)gs_alloc_bytes(dev->memory->non_gc_memory,
             sizeof(xps_icc_data_t), "xps_begin_image");
-        if (icc_data == NULL)
+        if (icc_data == NULL) {
             gs_throw(gs_error_VMerror, "Allocation of icc_data failed");
+            return(gs_error_VMerror);
+        }
 
         icc_data->hash = gsicc_get_hash(icc_profile);
         if (xdev->icc_data == NULL) {
@@ -1907,7 +1924,9 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
 
         /* Add profile to the package. Here like images we are going to write
            the data now.  Rather than later. */
-        code = xps_write_profile(pis, &(pie->icc_name[0]), icc_profile, xdev);
+        if (pgs == NULL)
+            return(gs_error_invalidaccess);
+        code = xps_write_profile(pgs, &(pie->icc_name[0]), icc_profile, xdev);
         if (code < 0)
             return gs_rethrow_code(code);
 
@@ -1930,6 +1949,8 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
         (*dev_proc(dev, get_clipping_box)) (dev, &bbox);
         gx_cpath_init_local(&cpath, dev->memory);
         code = gx_cpath_from_rectangle(&cpath, &bbox);
+        if (code < 0)
+            return gs_rethrow_code(code);
         pcpath = &cpath;
     } else {
         /* Force vector device to do new path as the clip path is the image
@@ -1939,7 +1960,9 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
         ((gx_device_vector*) vdev)->clip_path_id = vdev->no_clip_path_id;
     }
 
-    code = gdev_vector_begin_image(vdev, pis, pim, format, prect,
+    if (pgs == NULL)
+        return(gs_error_invalidaccess);
+    code = gdev_vector_begin_image(vdev, pgs, pim, format, prect,
         pdcolor, pcpath, mem, &xps_image_enum_procs,
         (gdev_vector_image_enum_t *)pie);
     if (code < 0)
@@ -2000,21 +2023,15 @@ xps_begin_image(gx_device *dev, const gs_imager_state *pis,
             *pinfo = NULL;
             return_error(gs_error_VMerror);
         }
-        /* Also, the color remaps need the imager state */
-        pie->pis = pis;
+        /* Also, the color remaps need the gs_gstate */
+        pie->pgs = pgs;
     }
 
     *pinfo = (gx_image_enum_common_t *)pie;
     return 0;
-use_default:
-    if (pie != NULL && pie->buffer != NULL)
-        gs_free_object(mem, pie->buffer, "xps_begin_image");
-    if (pie != NULL && pie->devc_buffer != NULL)
-        gs_free_object(mem, pie->devc_buffer, "xps_begin_image");
-    if (pie != NULL)
-        gs_free_object(mem, pie, "xps_begin_image");
 
-    return gx_default_begin_image(dev, pis, pim, format, prect,
+use_default:
+    return gx_default_begin_image(dev, pgs, pim, format, prect,
         pdcolor, pcpath, mem, pinfo);
 }
 
@@ -2029,7 +2046,7 @@ static int
 set_device_colors(xps_image_enum_t *pie)
 {
     gx_device *pdev = pie->dev;
-    const gs_imager_state *pis = pie->pis;
+    const gs_gstate *pgs = pie->pgs;
     gs_color_space *pcs = pie->pcs;
     byte *src = pie->buffer;
     byte *des = pie->devc_buffer;
@@ -2051,7 +2068,7 @@ set_device_colors(xps_image_enum_t *pie)
             for (j = 0; j < num_src; j++, pos_src++) {
                 cc.paint.values[j] = (float)(src_ptr[pos_src]) / 65535.0;
             }
-            code = remap_color(&cc, pcs, &devc, pis, pdev, gs_color_select_source);
+            code = remap_color(&cc, pcs, &devc, pgs, pdev, gs_color_select_source);
             dev_proc(pdev, decode_color)(pdev, devc.colors.pure, cm_values);
             for (j = 0; j < num_des; j++, pos_des++) {
                 des[pos_des] = (cm_values[j] >> 8);
@@ -2066,7 +2083,7 @@ set_device_colors(xps_image_enum_t *pie)
             for (j = 0; j < num_src; j++, pos_src++) {
                 cc.paint.values[j] = (float)(src[pos_src]) / scale;
             }
-            code = remap_color(&cc, pcs, &devc, pis, pdev, gs_color_select_source);
+            code = remap_color(&cc, pcs, &devc, pgs, pdev, gs_color_select_source);
             dev_proc(pdev, decode_color)(pdev, devc.colors.pure, cm_values);
             for (j = 0; j < num_des; j++, pos_des++) {
                 des[pos_des] = (cm_values[j] >> 8);
