@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* Ghostscript DLL loader for Windows */
 
 #include "windows_.h"
@@ -22,7 +24,6 @@
 #define GSREVISION gs_revision
 #include "ierrors.h"
 #include "iapi.h"
-#include "vdtrace.h"
 
 #include "dwres.h"
 #include "dwdll.h"
@@ -64,7 +65,7 @@ static int poll(void)
     }
     /* If text window closing then abort Ghostscript */
     if (tw->quitnow)
-        return e_Fatal;
+        return gs_error_Fatal;
     return 0;
 }
 
@@ -285,11 +286,6 @@ int new_main(int argc, char *argv[])
         return 1;
     }
 
-#ifdef DEBUG
-    visual_tracer_init();
-    gsdll.set_visual_tracer(&visual_tracer);
-#endif
-
     gsdll.set_stdio(instance, gsdll_stdin, gsdll_stdout, gsdll_stderr);
     gsdll.set_poll(instance, gsdll_poll);
     gsdll.set_display_callback(instance, &display);
@@ -333,21 +329,17 @@ int new_main(int argc, char *argv[])
     if (code == 0)
         code = gsdll.run_string(instance, start_string, 0, &exit_code);
     code1 = gsdll.exit(instance);
-    if (code == 0 || (code == e_Quit && code1 != 0))
+    if (code == 0 || (code == gs_error_Quit && code1 != 0))
         code = code1;
 #if defined(_MSC_VER) || defined(__BORLANDC__)
     } __except(exception_code() == EXCEPTION_STACK_OVERFLOW) {
-        code = e_Fatal;
+        code = gs_error_Fatal;
         text_puts(tw, "*** C stack overflow. Quiting...\n");
     }
 #endif
 
     gsdll.delete_instance(instance);
 
-#ifdef DEBUG
-    visual_tracer_close();
-#endif
-
     unload_dll(&gsdll);
 
     free(nargv);
@@ -355,12 +347,12 @@ int new_main(int argc, char *argv[])
     exit_status = 0;
     switch (code) {
         case 0:
-        case e_Quit:
+        case gs_error_Quit:
             break;
-        case e_Fatal:
+        case gs_error_Fatal:
             exit_status = 1;
             break;
-        case e_Info:
+        case gs_error_Info:
         default:
             exit_status = 255;
     }
@@ -388,6 +380,22 @@ set_font(void)
     WritePrivateProfileString(szIniSection, "FontSize", buf, szIniName);
 }
 
+typedef BOOL (SetProcessDPIAwareFn)(void);
+
+static void
+avoid_windows_scale(void)
+{
+    /* Fetch the function address and only call it if it is there; this keeps
+     * compatability with Windows < 8.1 */
+    HMODULE hUser32 = LoadLibrary(TEXT("user32.dll"));
+    SetProcessDPIAwareFn *ptr;
+
+    ptr = (SetProcessDPIAwareFn *)GetProcAddress(hUser32, "SetProcessDPIAware");
+    if (ptr != NULL)
+        ptr();
+    FreeLibrary(hUser32);
+}
+
 int PASCAL
 WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmdShow)
 {
@@ -399,7 +407,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
     int argc;
     LPSTR argv[MAXCMDTOKENS];
     LPSTR p;
-#ifndef WINDOWS_NO_UNICODE
+#ifndef GS_NO_UTF8
     LPSTR pstart;
 #endif
     char command[256];
@@ -409,6 +417,9 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
     int len = sizeof(winposbuf);
     int x, y, cx, cy;
 
+    /* Mark us as being 'system dpi aware' to avoid horrid scaling */
+    avoid_windows_scale();
+
     /* copy the hInstance into a variable so it can be used */
     ghInstance = hInstance;
 
@@ -429,7 +440,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
      *          if called with CreateProcess(command, args, ...)
      * Consequently we must use GetCommandLine()
      */
-#ifdef WINDOWS_NO_UNICODE
+#ifdef GS_NO_UTF8
     p = GetCommandLine();
 #else
     {
@@ -477,7 +488,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int cmd
     }
     argv[argc] = NULL;
 
-#ifndef WINDOWS_NO_UNICODE
+#ifndef GS_NO_UTF8
     free(pstart);
 #endif
 
