@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -65,23 +67,25 @@ struct gc_mark_stack_s {
 /* Forward references */
 
 static void gc_init_mark_stack(gc_mark_stack *, uint);
-static void gc_objects_clear_marks(const gs_memory_t *mem, chunk_t *);
+static void gc_objects_clear_marks(const gs_memory_t *mem, clump_t *);
 static void gc_unmark_names(name_table *, op_array_table *, op_array_table *);
 static int gc_trace(gs_gc_root_t *, gc_state_t *, gc_mark_stack *);
-static int gc_rescan_chunk(chunk_t *, gc_state_t *, gc_mark_stack *);
-static int gc_trace_chunk(const gs_memory_t *mem, chunk_t *, gc_state_t *, gc_mark_stack *);
+static int gc_rescan_clump(clump_t *, gc_state_t *, gc_mark_stack *);
+static int gc_trace_clump(const gs_memory_t *mem, clump_t *, gc_state_t *, gc_mark_stack *);
 static bool gc_trace_finish(gc_state_t *);
-static void gc_clear_reloc(chunk_t *);
-static void gc_objects_set_reloc(gc_state_t * gcst, chunk_t *);
-static void gc_do_reloc(chunk_t *, gs_ref_memory_t *, gc_state_t *);
-static void gc_objects_compact(chunk_t *, gc_state_t *);
-static void gc_free_empty_chunks(gs_ref_memory_t *);
+static void gc_clear_reloc(clump_t *);
+static void gc_objects_set_reloc(gc_state_t * gcst, clump_t *);
+static void gc_do_reloc(clump_t *, gs_ref_memory_t *, gc_state_t *);
+static void gc_objects_compact(clump_t *, gc_state_t *);
+static void gc_free_empty_clumps(gs_ref_memory_t *);
 
 /* Forward references for pointer types */
 static ptr_proc_unmark(ptr_struct_unmark);
 static ptr_proc_mark(ptr_struct_mark);
 static ptr_proc_unmark(ptr_string_unmark);
 static ptr_proc_mark(ptr_string_mark);
+static ptr_proc_unmark(ptr_name_index_unmark);
+static ptr_proc_mark(ptr_name_index_mark);
 /*ptr_proc_unmark(ptr_ref_unmark); *//* in igc.h */
 /*ptr_proc_mark(ptr_ref_mark); *//* in igc.h */
 static ptr_proc_reloc(igc_reloc_struct_ptr, void);
@@ -98,7 +102,7 @@ static const gc_procs_with_refs_t igc_procs = {
 /* Pointer type descriptors. */
 /* Note that the trace/mark routine has special knowledge of ptr_ref_type */
 /* and ptr_struct_type -- it assumes that no other types have embedded */
-/* pointers.  Note also that the reloc procedures for string and ref */
+/* pointers.  Note also that the reloc procedures for string, ref and name */
 /* pointers are never called. */
 typedef ptr_proc_reloc((*ptr_proc_reloc_t), void);
 const gs_ptr_procs_t ptr_struct_procs =
@@ -109,18 +113,20 @@ const gs_ptr_procs_t ptr_const_string_procs =
 {ptr_string_unmark, ptr_string_mark, NULL};
 const gs_ptr_procs_t ptr_ref_procs =
 {ptr_ref_unmark, ptr_ref_mark, NULL};
+const gs_ptr_procs_t ptr_name_index_procs =
+{ptr_name_index_unmark, ptr_name_index_mark, NULL};
 
 /* ------ Main program ------ */
 
 /* Top level of garbage collector. */
 #ifdef DEBUG
 static void
-end_phase(const char *str)
+end_phase(const gs_memory_t *mem, const char *str)
 {
     if (gs_debug_c('6')) {
-        dlprintf1("[6]---------------- end %s ----------------\n",
-                  (const char *)str);
-        dflush();
+        dmlprintf1(mem, "[6]---------------- end %s ----------------\n",
+                   (const char *)str);
+        dmflush(mem);
     }
 }
 static const char *const depth_dots_string = "..........";
@@ -145,7 +151,7 @@ gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
             ialloc_validate_memory(mem, gcst);
 }
 #else  /* !DEBUG */
-#  define end_phase(str) DO_NOTHING
+#  define end_phase(mem,str) DO_NOTHING
 #endif /* DEBUG */
 
 void
@@ -161,7 +167,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     int min_collect_vm_space;	/* min VM space to collect */
     int ispace;
     gs_ref_memory_t *mem;
-    chunk_t *cp;
+    clump_t *cp;
     gs_gc_root_t *rp;
     gc_state_t state;
     struct _msd {
@@ -170,6 +176,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     } ms_default;
     gc_mark_stack *mark_stack = &ms_default.stack;
     const gs_memory_t *cmem;
+    clump_splay_walker sw;
 
     /* Optionally force global GC for debugging. */
 
@@ -204,17 +211,17 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
   for (i = min_collect; i <= max_trace; ++i)
 #define for_space_mems(i, mem)\
   for (mem = space_memories[i]; mem != 0; mem = &mem->saved->state)
-#define for_mem_chunks(mem, cp)\
-  for (cp = (mem)->cfirst; cp != 0; cp = cp->cnext)
-#define for_space_chunks(i, mem, cp)\
-  for_space_mems(i, mem) for_mem_chunks(mem, cp)
-#define for_chunks(n, mem, cp)\
-  for_spaces(ispace, n) for_space_chunks(ispace, mem, cp)
-#define for_collected_chunks(mem, cp)\
-  for_collected_spaces(ispace) for_space_chunks(ispace, mem, cp)
-#define for_roots(n, mem, rp)\
-  for_spaces(ispace, n)\
-    for (mem = space_memories[ispace], rp = mem->roots; rp != 0; rp = rp->next)
+#define for_mem_clumps(mem, cp, sw)\
+  for (cp = clump_splay_walk_init(sw, mem); cp != 0; cp = clump_splay_walk_fwd(sw))
+#define for_space_clumps(i, mem, cp, sw)\
+  for_space_mems(i, mem) for_mem_clumps(mem, cp, sw)
+#define for_clumps(i, n, mem, cp, sw)\
+  for_spaces(i, n) for_space_clumps(i, mem, cp, sw)
+#define for_collected_clumps(i, mem, cp, sw)\
+  for_collected_spaces(i) for_space_clumps(i, mem, cp, sw)
+#define for_roots(i, n, mem, rp)\
+  for_spaces(i, n)\
+    for (mem = space_memories[i], rp = mem->roots; rp != 0; rp = rp->next)
 
     /* Initialize the state. */
 
@@ -237,7 +244,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
                                 (void **)&space_memories[ispace],
                                 "gc_top_level");
 
-    end_phase("register space roots");
+    end_phase(state.heap,"register space roots");
 
 #ifdef DEBUG
 
@@ -245,7 +252,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
     gc_validate_spaces(space_memories, max_trace, &state);
 
-    end_phase("pre-validate pointers");
+    end_phase(state.heap,"pre-validate pointers");
 
 #endif
 
@@ -256,25 +263,25 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     /* Clear marks in spaces to be collected. */
 
     for_collected_spaces(ispace)
-        for_space_chunks(ispace, mem, cp) {
-        gc_objects_clear_marks((const gs_memory_t *)mem, cp);
-        gc_strings_set_marks(cp, false);
-    }
+        for_space_clumps(ispace, mem, cp, &sw) {
+            gc_objects_clear_marks((const gs_memory_t *)mem, cp);
+            gc_strings_set_marks(cp, false);
+        }
 
-    end_phase("clear chunk marks");
+    end_phase(state.heap,"clear clump marks");
 
     /* Clear the marks of roots.  We must do this explicitly, */
-    /* since some roots are not in any chunk. */
+    /* since some roots are not in any clump. */
 
-    for_roots(max_trace, mem, rp) {
+    for_roots(ispace, max_trace, mem, rp) {
         enum_ptr_t eptr;
 
         eptr.ptr = *rp->p;
-        if_debug_root('6', "[6]unmarking root", rp);
+        if_debug_root('6', (const gs_memory_t *)mem, "[6]unmarking root", rp);
         (*rp->ptype->unmark)(&eptr, &state);
     }
 
-    end_phase("clear root marks");
+    end_phase(state.heap,"clear root marks");
 
     if (global) {
         op_array_table *global_ops = get_global_op_array(cmem);
@@ -294,7 +301,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     {
         gc_mark_stack *end = mark_stack;
 
-        for_chunks(max_trace, mem, cp) {
+        for_clumps(ispace, max_trace, mem, cp, &sw) {
             uint avail = cp->ctop - cp->cbot;
 
             if (avail >= sizeof(gc_mark_stack) + sizeof(ms_entry) *
@@ -308,8 +315,9 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
                 end->next = pms;
                 pms->prev = end;
                 pms->on_heap = false;
-                if_debug2('6', "[6]adding free 0x%lx(%u) to mark stack\n",
-                          (ulong) pms, pms->count);
+                if_debug2m('6', (const gs_memory_t *)mem,
+                           "[6]adding free 0x%lx(%u) to mark stack\n",
+                           (ulong) pms, pms->count);
             }
             cp->rescan_bot = cp->cend;
             cp->rescan_top = cp->cbase;
@@ -323,28 +331,28 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
         /* Mark from roots. */
 
-        for_roots(max_trace, mem, rp) {
-            if_debug_root('6', "[6]marking root", rp);
+        for_roots(ispace, max_trace, mem, rp) {
+            if_debug_root('6', (const gs_memory_t *)mem, "[6]marking root", rp);
             more |= gc_trace(rp, &state, mark_stack);
         }
 
-        end_phase("mark");
+        end_phase(state.heap,"mark");
 
-        /* If this is a local GC, mark from non-local chunks. */
+        /* If this is a local GC, mark from non-local clumps. */
 
         if (!global)
-            for_chunks(min_collect - 1, mem, cp)
-                more |= gc_trace_chunk((const gs_memory_t *)mem, cp, &state, mark_stack);
+            for_clumps(ispace, min_collect - 1, mem, cp, &sw)
+                more |= gc_trace_clump((const gs_memory_t *)mem, cp, &state, mark_stack);
 
         /* Handle mark stack overflow. */
 
         while (more < 0) {	/* stack overflowed */
             more = 0;
-            for_chunks(max_trace, mem, cp)
-                more |= gc_rescan_chunk(cp, &state, mark_stack);
+            for_clumps(ispace, max_trace, mem, cp, &sw)
+                more |= gc_rescan_clump(cp, &state, mark_stack);
         }
 
-        end_phase("mark overflow");
+        end_phase(state.heap,"mark overflow");
     }
 
     /* Free the mark stack. */
@@ -366,13 +374,13 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
         }
     }
 
-    end_phase("free mark stack");
+    end_phase(state.heap,"free mark stack");
 
     if (global) {
         gc_trace_finish(&state);
         names_trace_finish(state.ntable, &state);
 
-        end_phase("finish trace");
+        end_phase(state.heap,"finish trace");
     }
 
     /* Filter save change lists with removing elements,
@@ -390,17 +398,17 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     /* We have to clear the marks first, because we want the */
     /* relocation to wind up as o_untraced, not o_unmarked. */
 
-    for_chunks(min_collect - 1, mem, cp)
+    for_clumps(ispace, min_collect - 1, mem, cp, &sw)
         gc_objects_clear_marks((const gs_memory_t *)mem, cp);
 
-    end_phase("post-clear marks");
+    end_phase(state.heap,"post-clear marks");
 
-    for_chunks(min_collect - 1, mem, cp)
+    for_clumps(ispace, min_collect - 1, mem, cp, &sw)
         gc_clear_reloc(cp);
 
-    end_phase("clear reloc");
+    end_phase(state.heap,"clear reloc");
 
-    /* Set the relocation of roots outside any chunk to o_untraced, */
+    /* Set the relocation of roots outside any clump to o_untraced, */
     /* so we won't try to relocate pointers to them. */
     /* (Currently, there aren't any.) */
 
@@ -417,7 +425,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
     /* we are going to compact.  Also finalize freed objects. */
     state.cur_mem = (gs_memory_t *)mem;
 
-    for_collected_chunks(mem, cp) {
+    for_collected_clumps(ispace, mem, cp, &sw) {
         gc_objects_set_reloc(&state, cp);
         gc_strings_set_reloc(cp);
     }
@@ -430,22 +438,23 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
             gs_enable_free((gs_memory_t *)space_memories[i], true);
     }
 
-    end_phase("set reloc");
+    end_phase(state.heap,"set reloc");
 
     /* Relocate pointers. */
 
     state.relocating_untraced = true;
-    for_chunks(min_collect - 1, mem, cp)
+    for_clumps(ispace, min_collect - 1, mem, cp, &sw)
         gc_do_reloc(cp, mem, &state);
     state.relocating_untraced = false;
-    for_collected_chunks(mem, cp)
+    for_collected_clumps(ispace, mem, cp, &sw)
         gc_do_reloc(cp, mem, &state);
 
-    end_phase("relocate chunks");
+    end_phase(state.heap,"relocate clumps");
 
-    for_roots(max_trace, mem, rp) {
-        if_debug3('6', "[6]relocating root 0x%lx: 0x%lx -> 0x%lx\n",
-                  (ulong) rp, (ulong) rp->p, (ulong) * rp->p);
+    for_roots(ispace, max_trace, mem, rp) {
+        if_debug3m('6', (const gs_memory_t *)mem,
+                   "[6]relocating root 0x%lx: 0x%lx -> 0x%lx\n",
+                   (ulong) rp, (ulong) rp->p, (ulong) * rp->p);
         if (rp->ptype == ptr_ref_type) {
             ref *pref = (ref *) * rp->p;
 
@@ -454,21 +463,22 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
                            &state);
         } else
             *rp->p = (*rp->ptype->reloc) (*rp->p, &state);
-        if_debug3('6', "[6]relocated root 0x%lx: 0x%lx -> 0x%lx\n",
-                  (ulong) rp, (ulong) rp->p, (ulong) * rp->p);
+        if_debug3m('6', (const gs_memory_t *)mem,
+                   "[6]relocated root 0x%lx: 0x%lx -> 0x%lx\n",
+                   (ulong) rp, (ulong) rp->p, (ulong) * rp->p);
     }
 
-    end_phase("relocate roots");
+    end_phase(state.heap,"relocate roots");
 
     /* Compact data.  We only do this for spaces we are collecting. */
 
     for_collected_spaces(ispace) {
         for_space_mems(ispace, mem) {
-            for_mem_chunks(mem, cp) {
-                if_debug_chunk('6', "[6]compacting chunk", cp);
+            for_mem_clumps(mem, cp, &sw) {
+                if_debug_clump('6', (const gs_memory_t *)mem, "[6]compacting clump", cp);
                 gc_objects_compact(cp, &state);
-                gc_strings_compact(cp);
-                if_debug_chunk('6', "[6]after compaction:", cp);
+                gc_strings_compact(cp, cmem);
+                if_debug_clump('6', (const gs_memory_t *)mem, "[6]after compaction:", cp);
                 if (mem->pcc == cp)
                     mem->cc = *cp;
             }
@@ -477,20 +487,20 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
         }
     }
 
-    end_phase("compact");
+    end_phase(state.heap,"compact");
 
-    /* Free empty chunks. */
+    /* Free empty clumps. */
 
     for_collected_spaces(ispace) {
         for_space_mems(ispace, mem) {
-            gc_free_empty_chunks(mem);
+            gc_free_empty_clumps(mem);
         }
     }
 
-    end_phase("free empty chunks");
+    end_phase(state.heap,"free empty clumps");
 
     /*
-     * Update previous_status to reflect any freed chunks,
+     * Update previous_status to reflect any freed clumps,
      * and set inherited to the negative of allocated,
      * so it has no effect.  We must update previous_status by
      * working back-to-front along the save chain, using pointer reversal.
@@ -519,20 +529,21 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
             next = mem->saved;
             mem->saved = prev;
             mem->previous_status = total;
-            if_debug3('6',
-                      "[6]0x%lx previous allocated=%lu, used=%lu\n",
-                      (ulong) mem, total.allocated, total.used);
+            if_debug3m('6', (const gs_memory_t *)mem,
+                       "[6]0x%lx previous allocated=%lu, used=%lu\n",
+                       (ulong) mem, total.allocated, total.used);
             gs_memory_status((gs_memory_t *) mem, &total);
             mem->gc_allocated = mem->allocated + total.allocated;
         }
         mem = space_memories[ispace];
         mem->previous_status = total;
         mem->gc_allocated = mem->allocated + total.allocated;
-        if_debug3('6', "[6]0x%lx previous allocated=%lu, used=%lu\n",
-                  (ulong) mem, total.allocated, total.used);
+        if_debug3m('6', (const gs_memory_t *)mem,
+                   "[6]0x%lx previous allocated=%lu, used=%lu\n",
+                   (ulong) mem, total.allocated, total.used);
     }
 
-    end_phase("update stats");
+    end_phase(state.heap,"update stats");
 
   no_collect:
 
@@ -542,7 +553,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
         gs_unregister_root((gs_memory_t *)space_memories[ispace],
                            &space_roots[ispace], "gc_top_level");
 
-    end_phase("unregister space roots");
+    end_phase(state.heap,"unregister space roots");
 
 #ifdef DEBUG
 
@@ -550,7 +561,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
     gc_validate_spaces(space_memories, max_trace, &state);
 
-    end_phase("validate pointers");
+    end_phase(state.heap,"validate pointers");
 
 #endif
 }
@@ -584,12 +595,19 @@ ptr_string_unmark(enum_ptr_t *pep, gc_state_t * gcst)
     discard(gc_string_mark(pep->ptr, pep->size, false, gcst));
 }
 
-/* Unmark the objects in a chunk. */
+/* Unmark a single name. */
 static void
-gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
+ptr_name_index_unmark(enum_ptr_t *pep, gc_state_t * gcst)
 {
-    if_debug_chunk('6', "[6]unmarking chunk", cp);
-    SCAN_CHUNK_OBJECTS(cp)
+    /* Do nothing */
+}
+
+/* Unmark the objects in a clump. */
+static void
+gc_objects_clear_marks(const gs_memory_t *mem, clump_t * cp)
+{
+    if_debug_clump('6', mem, "[6]unmarking clump", cp);
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
         struct_proc_clear_marks((*proc)) =
         pre->o_type->clear_marks;
@@ -597,9 +615,9 @@ gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
     if (pre->o_type != &st_free)
         debug_check_object(pre, cp, NULL);
 #endif
-    if_debug3('7', " [7](un)marking %s(%lu) 0x%lx\n",
-              struct_type_name_string(pre->o_type),
-              (ulong) size, (ulong) pre);
+    if_debug3m('7', (const gs_memory_t *)mem, " [7](un)marking %s(%lu) 0x%lx\n",
+               struct_type_name_string(pre->o_type),
+               (ulong) size, (ulong) pre);
     o_set_unmarked(pre);
     if (proc != 0)
         (*proc) (mem, pre + 1, size, pre->o_type);
@@ -640,10 +658,10 @@ gc_init_mark_stack(gc_mark_stack * pms, uint count)
     pms->entries[0].is_refs = false;
 }
 
-/* Mark starting from all marked objects in the interval of a chunk */
+/* Mark starting from all marked objects in the interval of a clump */
 /* needing rescanning. */
 static int
-gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
+gc_rescan_clump(clump_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     byte *sbot = cp->rescan_bot;
     byte *stop = cp->rescan_top;
@@ -655,17 +673,17 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
     if (sbot > stop)
         return 0;
     root.p = &comp;
-    if_debug_chunk('6', "[6]rescanning chunk", cp);
+    if_debug_clump('6', mem, "[6]rescanning clump", cp);
     cp->rescan_bot = cp->cend;
     cp->rescan_top = cp->cbase;
-    SCAN_CHUNK_OBJECTS(cp)
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
         if ((byte *) (pre + 1) + size < sbot);
     else if ((byte *) (pre + 1) > stop)
         return more;		/* 'break' won't work here */
     else {
-        if_debug2('7', " [7]scanning/marking 0x%lx(%lu)\n",
-                  (ulong) pre, (ulong) size);
+        if_debug2m('7', mem, " [7]scanning/marking 0x%lx(%lu)\n",
+                   (ulong) pre, (ulong) size);
         if (pre->o_type == &st_refs) {
             ref_packed *rp = (ref_packed *) (pre + 1);
             char *end = (char *)rp + size;
@@ -706,11 +724,11 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
         return more;
 }
 
-/* Mark starting from all the objects in a chunk. */
+/* Mark starting from all the objects in a clump. */
 /* We assume that pstate->min_collect > avm_system, */
 /* so we don't have to trace names. */
 static int
-gc_trace_chunk(const gs_memory_t *mem, chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
+gc_trace_clump(const gs_memory_t *mem, clump_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     gs_gc_root_t root;
     void *comp;
@@ -718,12 +736,12 @@ gc_trace_chunk(const gs_memory_t *mem, chunk_t * cp, gc_state_t * pstate, gc_mar
     int min_trace = pstate->min_collect;
 
     root.p = &comp;
-    if_debug_chunk('6', "[6]marking from chunk", cp);
-    SCAN_CHUNK_OBJECTS(cp)
+    if_debug_clump('6', mem, "[6]marking from clump", cp);
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
     {
-        if_debug2('7', " [7]scanning/marking 0x%lx(%lu)\n",
-                  (ulong) pre, (ulong) size);
+        if_debug2m('7', mem, " [7]scanning/marking 0x%lx(%lu)\n",
+                   (ulong) pre, (ulong) size);
         if (pre->o_type == &st_refs) {
             ref_packed *rp = (ref_packed *) (pre + 1);
             char *end = (char *)rp + size;
@@ -786,7 +804,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
   BEGIN\
     if (names_mark_index(nt, nidx)) {\
         new |= 1;\
-        if_debug2('8', "  [8]marked name 0x%lx(%u)\n",\
+        if_debug2m('8', gcst_get_memory_ptr(pstate), "  [8]marked name 0x%lx(%u)\n",\
                   (ulong)names_index_ptr(nt, nidx), nidx);\
     }\
   END
@@ -832,17 +850,17 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
                 continue;
             }
             debug_check_object(ptr - 1, NULL, NULL);
-          ts:if_debug4('7', " [7]%smarking %s 0x%lx[%u]",
-                      depth_dots(sp, pms),
-                      struct_type_name_string(ptr[-1].o_type),
-                      (ulong) ptr, sp->index);
+          ts:if_debug4m('7', pstate->heap, " [7]%smarking %s 0x%lx[%u]",
+                        depth_dots(sp, pms),
+                        struct_type_name_string(ptr[-1].o_type),
+                        (ulong) ptr, sp->index);
             mproc = ptr[-1].o_type->enum_ptrs;
             if (mproc == gs_no_struct_enum_ptrs ||
                 (ptp = (*mproc)
                  (gcst_get_memory_ptr(pstate), ptr, pre_obj_contents_size(ptr - 1),
                   sp->index, &nep, ptr[-1].o_type, pstate)) == 0
                 ) {
-                if_debug0('7', " - done\n");
+                if_debug0m('7', pstate->heap, " - done\n");
                 sp--;
                 continue;
             }
@@ -851,7 +869,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
             /* template for pointer enumeration work. */
             nptr = (void *)nep.ptr;
             sp->index++;
-            if_debug1('7', " = 0x%lx\n", (ulong) nptr);
+            if_debug1m('7', pstate->heap, " = 0x%lx\n", (ulong) nptr);
             /* Descend into nep.ptr, whose pointer type is ptp. */
             if (ptp == ptr_struct_type) {
                 sp[1].index = 0;
@@ -873,9 +891,9 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
                 goto do_refs;
             } else {		/* We assume this is some non-pointer- */
                 /* containing type. */
-                if ((*ptp->mark) (&nep, pstate))
-                    new |= 1;
-                goto ts;
+                /* This was the only formulation of this call/condition/assignment that valgrind did not complain about */
+                new |= ((*ptp->mark) (&nep, pstate)) ? 1 : 0;
+               goto ts;
             }
         }
 
@@ -891,7 +909,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
                 continue;
             }
             --(sp->index);
-            if_debug3('8', "  [8]%smarking refs 0x%lx[%u]\n",
+            if_debug3m('8', pstate->heap, "  [8]%smarking refs 0x%lx[%u]\n",
                       depth_dots(sp, pms), (ulong) pptr, sp->index);
             if (r_is_packed(pptr)) {
                 if (!r_has_pmark(pptr)) {
@@ -1018,7 +1036,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
     return new;
 }
 /* Link to, attempting to allocate if necessary, */
-/* another chunk of mark stack. */
+/* another clump of mark stack. */
 static int
 gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 {
@@ -1037,7 +1055,7 @@ gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
         if (pms->next == 0) {	/* The mark stack overflowed. */
             ms_entry *sp = pms->entries + pms->count - 1;
             byte *cptr = sp->ptr;	/* container */
-            chunk_t *cp = gc_locate(cptr, pstate);
+            clump_t *cp = gc_locate(cptr, pstate);
             int new = 1;
 
             if (cp == 0) {	/* We were tracing outside collectible */
@@ -1081,6 +1099,13 @@ ptr_string_mark(enum_ptr_t *pep, gc_state_t * gcst)
     return gc_string_mark(pep->ptr, pep->size, true, gcst);
 }
 
+/* Mark a name.  Return true if new mark. */
+static bool
+ptr_name_index_mark(enum_ptr_t *pep, gc_state_t * gcst)
+{
+    return names_mark_index(gcst->heap->gs_lib_ctx->gs_name_table, pep->size);
+}
+
 /* Finish tracing by marking names. */
 static bool
 gc_trace_finish(gc_state_t * pstate)
@@ -1112,27 +1137,27 @@ gc_trace_finish(gc_state_t * pstate)
 
 /* ------ Relocation planning phase ------ */
 
-/* Initialize the relocation information in the chunk header. */
+/* Initialize the relocation information in the clump header. */
 static void
-gc_init_reloc(chunk_t * cp)
+gc_init_reloc(clump_t * cp)
 {
-    chunk_head_t *chead = cp->chead;
+    clump_head_t *chead = cp->chead;
 
     chead->dest = cp->cbase;
     chead->free.o_back =
-        offset_of(chunk_head_t, free) >> obj_back_shift;
+        offset_of(clump_head_t, free) >> obj_back_shift;
     chead->free.o_size = sizeof(obj_header_t);
     chead->free.o_nreloc = 0;
 }
 
-/* Set marks and clear relocation for chunks that won't be compacted. */
+/* Set marks and clear relocation for clumps that won't be compacted. */
 static void
-gc_clear_reloc(chunk_t * cp)
+gc_clear_reloc(clump_t * cp)
 {
     byte *pfree = (byte *) & cp->chead->free;
 
     gc_init_reloc(cp);
-    SCAN_CHUNK_OBJECTS(cp)
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
         const struct_shared_procs_t *procs =
     pre->o_type->shared;
@@ -1146,18 +1171,18 @@ gc_clear_reloc(chunk_t * cp)
     gc_strings_clear_reloc(cp);
 }
 
-/* Set the relocation for the objects in a chunk. */
-/* This will never be called for a chunk with any o_untraced objects. */
+/* Set the relocation for the objects in a clump. */
+/* This will never be called for a clump with any o_untraced objects. */
 static void
-gc_objects_set_reloc(gc_state_t * gcst, chunk_t * cp)
+gc_objects_set_reloc(gc_state_t * gcst, clump_t * cp)
 {
-    uint reloc = 0;
-    chunk_head_t *chead = cp->chead;
+    size_t reloc = 0;
+    clump_head_t *chead = cp->chead;
     byte *pfree = (byte *) & chead->free;	/* most recent free object */
 
-    if_debug_chunk('6', "[6]setting reloc for chunk", cp);
+    if_debug_clump('6', gcst->heap, "[6]setting reloc for clump", cp);
     gc_init_reloc(cp);
-    SCAN_CHUNK_OBJECTS(cp)
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
         struct_proc_finalize((*finalize));
     const struct_shared_procs_t *procs =
@@ -1168,39 +1193,39 @@ gc_objects_set_reloc(gc_state_t * gcst, chunk_t * cp)
         ) {			/* Free object */
         reloc += sizeof(obj_header_t) + obj_align_round(size);
         if ((finalize = pre->o_type->finalize) != 0) {
-            if_debug2('u', "[u]GC finalizing %s 0x%lx\n",
-                      struct_type_name_string(pre->o_type),
-                      (ulong) (pre + 1));
+            if_debug2m('u', gcst->heap, "[u]GC finalizing %s 0x%lx\n",
+                       struct_type_name_string(pre->o_type),
+                       (ulong) (pre + 1));
             (*finalize) (gcst->cur_mem, pre + 1);
         }
         pfree = (byte *) pre;
         pre->o_back = (pfree - (byte *) chead) >> obj_back_shift;
         pre->o_nreloc = reloc;
-        if_debug3('7', " [7]at 0x%lx, unmarked %lu, new reloc = %u\n",
-                  (ulong) pre, (ulong) size, reloc);
+        if_debug3m('7', gcst->heap, " [7]at 0x%p, unmarked %lu, new reloc = %u\n",
+                   pre, (ulong) size, (unsigned int)reloc);
     } else {			/* Useful object */
         debug_check_object(pre, cp, gcst);
         pre->o_back = ((byte *) pre - pfree) >> obj_back_shift;
     }
     END_OBJECTS_SCAN
 #ifdef DEBUG
-        if (reloc != 0) {
-        if_debug1('6', "[6]freed %u", reloc);
-        if_debug_chunk('6', " in", cp);
+    if (reloc != 0) {
+        if_debug1m('6', gcst->heap, "[6]freed %u", (unsigned int)reloc);
+        if_debug_clump('6', gcst->heap, " in", cp);
     }
 #endif
 }
 
 /* ------ Relocation phase ------ */
 
-/* Relocate the pointers in all the objects in a chunk. */
+/* Relocate the pointers in all the objects in a clump. */
 static void
-gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
+gc_do_reloc(clump_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
 {
-    chunk_head_t *chead = cp->chead;
+    clump_head_t *chead = cp->chead;
 
-    if_debug_chunk('6', "[6]relocating in chunk", cp);
-    SCAN_CHUNK_OBJECTS(cp)
+    if_debug_clump('6', (const gs_memory_t *)mem, "[6]relocating in clump", cp);
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
 #ifdef DEBUG
         pstate->container = cp;
@@ -1208,17 +1233,17 @@ gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
     /* We need to relocate the pointers in an object iff */
     /* it is o_untraced, or it is a useful object. */
     /* An object is free iff its back pointer points to */
-    /* the chunk_head structure. */
+    /* the clump_head structure. */
         if (o_is_untraced(pre) ||
             pre->o_back << obj_back_shift != (byte *) pre - (byte *) chead
             ) {
             struct_proc_reloc_ptrs((*proc)) =
                 pre->o_type->reloc_ptrs;
 
-            if_debug3('7',
-                      " [7]relocating ptrs in %s(%lu) 0x%lx\n",
-                      struct_type_name_string(pre->o_type),
-                      (ulong) size, (ulong) pre);
+            if_debug3m('7', (const gs_memory_t *)mem,
+                       " [7]relocating ptrs in %s(%lu) 0x%lx\n",
+                       struct_type_name_string(pre->o_type),
+                       (ulong) size, (ulong) pre);
             if (proc != 0)
                 (*proc) (pre + 1, size, pre->o_type, pstate);
         }
@@ -1260,7 +1285,7 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
         else {
 #ifdef DEBUG
             /* Do some sanity checking. */
-            chunk_t *cp = gcst->container;
+            clump_t *cp = gcst->container;
 
             if (cp != 0 && cp->cbase <= (byte *)obj && (byte *)obj <cp->ctop) {
                 if (back > (cp->ctop - cp->cbase) >> obj_back_shift) {
@@ -1269,14 +1294,14 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
                     gs_abort(NULL);
                 }
             } else {
-                /* Pointed to unknown chunk. Can't check it, sorry. */
+                /* Pointed to unknown clump. Can't check it, sorry. */
             }
 #endif
             {
                 const obj_header_t *pfree = (const obj_header_t *)
                 ((const char *)(optr - 1) -
                  (back << obj_back_shift));
-                const chunk_head_t *chead = (const chunk_head_t *)
+                const clump_head_t *chead = (const clump_head_t *)
                 ((const char *)pfree -
                  (pfree->o_back << obj_back_shift));
 
@@ -1297,27 +1322,27 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 
 /* ------ Compaction phase ------ */
 
-/* Compact the objects in a chunk. */
-/* This will never be called for a chunk with any o_untraced objects. */
+/* Compact the objects in a clump. */
+/* This will never be called for a clump with any o_untraced objects. */
 static void
-gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
+gc_objects_compact(clump_t * cp, gc_state_t * gcst)
 {
-    chunk_head_t *chead = cp->chead;
+    clump_head_t *chead = cp->chead;
     obj_header_t *dpre = (obj_header_t *) chead->dest;
     const gs_memory_t *cmem = gcst->spaces.memories.named.system->stable_memory;
 
-    SCAN_CHUNK_OBJECTS(cp)
+    SCAN_CLUMP_OBJECTS(cp)
         DO_ALL
     /* An object is free iff its back pointer points to */
-    /* the chunk_head structure. */
+    /* the clump_head structure. */
         if (pre->o_back << obj_back_shift != (byte *) pre - (byte *) chead) {
         const struct_shared_procs_t *procs = pre->o_type->shared;
 
         debug_check_object(pre, cp, gcst);
-        if_debug4('7',
-                  " [7]compacting %s 0x%lx(%lu) to 0x%lx\n",
-                  struct_type_name_string(pre->o_type),
-                  (ulong) pre, (ulong) size, (ulong) dpre);
+        if_debug4m('7', cmem,
+                   " [7]compacting %s 0x%lx(%lu) to 0x%lx\n",
+                   struct_type_name_string(pre->o_type),
+                   (ulong) pre, (ulong) size, (ulong) dpre);
         if (procs == 0) {
             if (dpre != pre)
                 memmove(dpre, pre,
@@ -1329,7 +1354,7 @@ gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
     }
     END_OBJECTS_SCAN
         if (cp->outer == 0 && chead->dest != cp->cbase)
-        dpre = (obj_header_t *) cp->cbase;	/* compacted this chunk into another */
+        dpre = (obj_header_t *) cp->cbase;	/* compacted this clump into another */
     gs_alloc_fill(dpre, gs_alloc_fill_collected, cp->cbot - (byte *) dpre);
     cp->cbot = (byte *) dpre;
     cp->rcur = 0;
@@ -1338,26 +1363,28 @@ gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 
 /* ------ Cleanup ------ */
 
-/* Free empty chunks. */
-static void
-gc_free_empty_chunks(gs_ref_memory_t * mem)
+static splay_app_result_t
+free_if_empty(clump_t *cp, void *arg)
 {
-    chunk_t *cp;
-    chunk_t *csucc;
+    gs_ref_memory_t * mem = (gs_ref_memory_t *)arg;
 
-    /* Free the chunks in reverse order, */
-    /* to encourage LIFO behavior. */
-    for (cp = mem->clast; cp != 0; cp = csucc) {	/* Make sure this isn't an inner chunk, */
-        /* or a chunk that has inner chunks. */
-        csucc = cp->cprev;	/* save before freeing */
-        if (cp->cbot == cp->cbase && cp->ctop == cp->climit &&
-            cp->outer == 0 && cp->inner_count == 0
-            ) {
-            alloc_free_chunk(cp, mem);
-            if (mem->pcc == cp)
-                mem->pcc = 0;
-        }
+    if (cp->cbot == cp->cbase && cp->ctop == cp->climit &&
+        cp->outer == 0 && cp->inner_count == 0)
+    {
+        alloc_free_clump(cp, mem);
+        if (mem->pcc == cp)
+            mem->pcc = 0;
     }
+    return SPLAY_APP_CONTINUE;
+}
+
+/* Free empty clumps. */
+static void
+gc_free_empty_clumps(gs_ref_memory_t * mem)
+{
+    /* NOTE: Not in reverse order any more, so potentially
+     * not quite as good for crap allocators. */
+    clump_splay_app(mem->root, mem, free_if_empty, mem);
 }
 
 const gs_memory_t * gcst_get_memory_ptr(gc_state_t *gcst)
