@@ -248,7 +248,7 @@ context_reclaim(vm_spaces * pspaces, bool global)
     gs_context_t *pctx = 0;	/* = 0 is bogus to pacify compilers */
     gs_scheduler_t *psched = 0;
     gs_ref_memory_t *lmem = 0;	/* = 0 is bogus to pacify compilers */
-    chunk_locator_t loc;
+    clump_locator_t loc;
 
     for (i = countof(pspaces->memories.indexed) - 1; psched == 0 && i > 0; --i) {
         gs_ref_memory_t *mem = pspaces->memories.indexed[i];
@@ -264,6 +264,10 @@ context_reclaim(vm_spaces * pspaces, bool global)
         }
     }
 
+    /* Ensure psched is not NULL before we try to use it, exit if it is (should be impossible) */
+    if (!psched)
+        return;
+
     /* Hide all contexts in other (local) VMs. */
     /*
      * See context_create below for why we look for the context
@@ -273,7 +277,7 @@ context_reclaim(vm_spaces * pspaces, bool global)
     loc.cp = 0;
     for (i = 0; i < CTX_TABLE_SIZE; ++i)
         for (pctx = psched->table[i]; pctx; pctx = pctx->table_next)
-            pctx->visible = chunk_locate_ptr(pctx, &loc);
+            pctx->visible = clump_locate_ptr(pctx, &loc);
 
 #ifdef DEBUG
     if (!psched->current->visible) {
@@ -328,8 +332,12 @@ add_last(const gs_scheduler_t *psched, ctx_list_t *pl, gs_context_t *pc)
     pc->next_index = 0;
     if (pl->head_index == 0)
         pl->head_index = pc->index;
-    else
-        index_context(psched, pl->tail_index)->next_index = pc->index;
+    else {
+        gs_context_t *ready = index_context(psched, pl->tail_index);
+        if (ready)
+            ready->next_index = pc->index;
+        /* Other cases return fatal error if there is no context, but we can't do that */
+    }
     pl->tail_index = pc->index;
 }
 
@@ -407,15 +415,19 @@ ctx_reschedule(i_ctx_t **pi_ctx_p)
     /* take care of that now. */
     while (psched->dead_index != 0) {
         gs_context_t *dead = index_context(psched, psched->dead_index);
-        long next_index = dead->next_index;
+        long next_index;
 
-        if (current == dead) {
-            if_debug1('"', "[\"]storing dead %ld\n", current->index);
-            context_state_store(&current->state);
-            current = 0;
+        if (dead) {
+            next_index = dead->next_index;
+
+            if (current == dead) {
+                if_debug1('"', "[\"]storing dead %ld\n", current->index);
+                context_state_store(&current->state);
+                current = 0;
+            }
+            context_destroy(dead);
+            psched->dead_index = next_index;
         }
-        context_destroy(dead);
-        psched->dead_index = next_index;
     }
     /* Update saved_local_vm.  See above for the invariant. */
     if (current != 0)
@@ -617,8 +629,8 @@ do_fork(i_ctx_t *i_ctx_p, os_ptr op, const ref * pstdin, const ref * pstdout,
             return_error(gs_error_Fatal);
         old_userdict = *puserdict;
         userdict_size = dict_maxlength(&old_userdict);
-        lmem = ialloc_alloc_state(parent, iimemory_local->chunk_size);
-        lmem_stable = ialloc_alloc_state(parent, iimemory_local->chunk_size);
+        lmem = ialloc_alloc_state(parent, iimemory_local->clump_size);
+        lmem_stable = ialloc_alloc_state(parent, iimemory_local->clump_size);
         if (lmem == 0 || lmem_stable == 0) {
             gs_free_object(parent, lmem_stable, "do_fork");
             gs_free_object(parent, lmem, "do_fork");
@@ -656,12 +668,12 @@ do_fork(i_ctx_t *i_ctx_p, os_ptr op, const ref * pstdin, const ref * pstdout,
          */
         {
             int n;
-            const gs_state *old;
-            gs_state *new;
+            const gs_gstate *old;
+            gs_gstate *new;
 
-            for (n = 0, old = igs; old != 0; old = gs_state_saved(old))
+            for (n = 0, old = igs; old != 0; old = gs_gstate_saved(old))
                 ++n;
-            for (old = pctx->state.pgs; old != 0; old = gs_state_saved(old))
+            for (old = pctx->state.pgs; old != 0; old = gs_gstate_saved(old))
                 --n;
             for (; n > 0 && code >= 0; --n)
                 code = gs_gsave(pctx->state.pgs);
@@ -671,7 +683,7 @@ do_fork(i_ctx_t *i_ctx_p, os_ptr op, const ref * pstdin, const ref * pstdout,
             }
             for (old = igs, new = pctx->state.pgs;
                  old != 0 /* (== new != 0) */  && code >= 0;
-                 old = gs_state_saved(old), new = gs_state_saved(new)
+                 old = gs_gstate_saved(old), new = gs_gstate_saved(new)
                 )
                 code = gs_setgstate(new, old);
             if (code < 0) {
