@@ -99,7 +99,7 @@ sfnts_next_elem(sfnts_reader *r)
     do {
     	r->index++;
     	code = array_get(r->memory, r->sfnts, r->index, &s);
-    	if (code == e_rangecheck) {
+    	if (code == gs_error_rangecheck) {
     		r->error |= 2;
     	}
     	else if (code < 0) {
@@ -127,22 +127,36 @@ sfnts_reader_rbyte(sfnts_reader *r)
     return (sfnts_reader_rbyte_inline(r));
 }
 
+#define SFNTS_READER_RBYTE_TO_USHORT(r) ((ulong)sfnts_reader_rbyte_inline(r))
+
 static ushort
 sfnts_reader_rword(sfnts_reader *r)
 {
-    return ((sfnts_reader_rbyte_inline(r) << 8) +
-            sfnts_reader_rbyte_inline(r));
+    ushort retval;
+
+    retval = SFNTS_READER_RBYTE_TO_USHORT(r) << 8;
+    retval += SFNTS_READER_RBYTE_TO_USHORT(r);
+
+    return retval;
 }
 
+#undef SFNTS_READER_RBYTE_TO_USHORT
+
+#define SFNTS_READER_RBYTE_TO_ULONG(r) ((ulong)sfnts_reader_rbyte_inline(r))
+
 static ulong
 sfnts_reader_rlong(sfnts_reader *r)
 {
-    return ((sfnts_reader_rbyte_inline(r) << 24) +
-            (sfnts_reader_rbyte_inline(r) << 16) +
-            (sfnts_reader_rbyte_inline(r) << 8) +
-            sfnts_reader_rbyte_inline(r));
+    ulong retval;
+    retval = SFNTS_READER_RBYTE_TO_ULONG(r) << 24;
+    retval += SFNTS_READER_RBYTE_TO_ULONG(r) << 16;
+    retval += SFNTS_READER_RBYTE_TO_ULONG(r) << 8;
+    retval += SFNTS_READER_RBYTE_TO_ULONG(r);
+    return retval;
 }
 
+#undef SFNTS_READER_RWORD_TO_LONG
+
 static int
 sfnts_reader_rstring(sfnts_reader *r, byte *v, int length)
 {
@@ -676,6 +690,7 @@ FAPI_FF_get_float(gs_fapi_font *ff, gs_fapi_font_feature var_id, int index)
                 }
                 switch (index) {
                     case 0:
+                    default:
                         return (mptr->xx / FontMatrix_div);
                     case 1:
                         return (mptr->xy / FontMatrix_div);
@@ -978,6 +993,39 @@ FAPI_FF_get_charstring(gs_fapi_font *ff, int index, byte *buf,
     return (r_size(&eltp[1]));
 }
 
+static int
+sfnt_get_sfnt_length(ref *pdr, ulong *len)
+{
+    int code = 0;
+    ref *sfnts, sfnt_elem;
+    const gs_memory_t *mem = dict_mem(pdr->value.pdict);
+
+    *len = 0;
+    if (r_type(pdr) != t_dictionary ||
+        dict_find_string(pdr, "sfnts", &sfnts) <= 0) {
+        code = gs_error_invalidfont;
+    }
+    else {
+        if (r_type(sfnts) != t_array && r_type(sfnts) != t_string) {
+            code = gs_error_invalidfont;
+        }
+        else {
+            if (r_type(sfnts) == t_string) {
+                *len = r_size(sfnts);
+            }
+            else {
+                int i;
+                for (i = 0; i < r_size(sfnts); i++) {
+                    code = array_get(mem, sfnts, i, &sfnt_elem);
+                    if (code < 0) break;
+                    *len += r_size(&sfnt_elem);
+                }
+            }
+        }
+    }
+    return code;
+}
+
 static bool
 sfnt_get_glyph_offset(ref *pdr, gs_font_type42 *pfont42, int index,
                       ulong *offset0)
@@ -1184,14 +1232,38 @@ FAPI_FF_get_glyph(gs_fapi_font *ff, int char_code, byte *buf,
                            min(glyph_length, buf_length) /* safety */ );
             }
             else {
-                gs_font_type42 *pfont42 =
-                    (gs_font_type42 *) ff->client_font_data;
+                gs_font_type42 *pfont42 = (gs_font_type42 *) ff->client_font_data;
                 ulong offset0, length_read;
-                bool error =
-                    sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
+                bool error = sfnt_get_glyph_offset(pdr, pfont42, char_code, &offset0);
 
-                glyph_length =
-                    (error ? -1 : pfont42->data.len_glyphs[char_code]);
+                if (error != 0) {
+                    glyph_length = -1;
+                }
+                else if (pfont42->data.len_glyphs) {
+                    glyph_length = pfont42->data.len_glyphs[char_code];
+                }
+                else {
+                    ulong noffs;
+                    /* If we haven't got a len_glyphs array, try using the offset of the next glyph offset
+                     * to work out the length
+                     */
+                    error = sfnt_get_glyph_offset(pdr, pfont42, char_code + 1, &noffs);
+                    if (error == 0) {
+                        glyph_length = noffs - offset0;
+                    }
+                    else {
+                        /* And if we can't get the next glyph offset, use the end of the sfnt data
+                         * to work out the length.
+                         */
+                        int code = sfnt_get_sfnt_length(pdr, &noffs);
+                        if (code < 0) {
+                            glyph_length = -1;
+                        }
+                        else {
+                            glyph_length = noffs - offset0;
+                        }
+                    }
+                }
 
                 if (buf != 0 && !error) {
                     sfnts_reader r;
@@ -1311,7 +1383,7 @@ FAPI_get_xlatmap(i_ctx_t *i_ctx_p, char **xlatmap)
     if ((code = dict_find_string(systemdict, ".xlatmap", &pref)) < 0)
         return code;
     if (r_type(pref) != t_string)
-        return_error(e_typecheck);
+        return_error(gs_error_typecheck);
     *xlatmap = (char *)pref->value.bytes;
     /*  Note : this supposes that xlatmap doesn't move in virtual memory.
        Garbager must not be called while plugin executes get_scaled_font, get_decodingID.
@@ -1328,7 +1400,7 @@ renderer_retcode(gs_memory_t *mem, gs_fapi_server *I, gs_fapi_retcode rc)
     emprintf2(mem,
               "Error: Font Renderer Plugin ( %s ) return code = %d\n",
               I->ig.d->subtype, rc);
-    return rc < 0 ? rc : e_invalidfont;
+    return rc < 0 ? rc : gs_error_invalidfont;
 }
 
 /* <server name>/<null> object .FAPIavailable bool */
@@ -1345,7 +1417,7 @@ zFAPIavailable(i_ctx_t *i_ctx_p)
         serv_name =
             (char *) ref_to_string(&name_ref, imemory, "zFAPIavailable");
         if (!serv_name) {
-            return_error(e_VMerror);
+            return_error(gs_error_VMerror);
         }
     }
 
@@ -1410,7 +1482,7 @@ FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font *pfont,
         if (dict_find_string(op, "FontBBox", &v) > 0) {
             if (!r_has_type(v, t_array) && !r_has_type(v, t_shortarray)
                 && !r_has_type(v, t_mixedarray))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             make_real(&mat[0], pbfont->FontBBox.p.x);
             make_real(&mat[1], pbfont->FontBBox.p.y);
             make_real(&mat[2], pbfont->FontBBox.q.x);
@@ -1459,11 +1531,11 @@ FAPI_refine_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font *pfont,
 
             if (dict_find_string(pdr, "CIDSystemInfo", &CIDSystemInfo) <= 0
                 || !r_has_type(CIDSystemInfo, t_dictionary))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
 
             if (dict_find_string(CIDSystemInfo, "Ordering", &Ordering) <= 0
                 || !r_has_type(Ordering, t_string)) {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
 
             ordering_length =
@@ -1533,7 +1605,7 @@ zFAPIrebuildfont(i_ctx_t *i_ctx_p)
     if (pbfont->FAPI == NULL) {
         if (dict_find_string(op - 1, "FAPI", &v) <= 0
             || !r_has_type(v, t_name))
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         obj_string_data(imemory, v, &pchars, &len);
         len = min(len, sizeof(FAPI_ID) - 1);
         strncpy((char *)FAPI_ID, (const char *)pchars, len);
@@ -1547,7 +1619,7 @@ zFAPIrebuildfont(i_ctx_t *i_ctx_p)
                                 (gs_fapi_get_server_param_callback)
                                 ps_get_server_param);
         if (!pbfont->FAPI || code < 0) {
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         }
     }
 
@@ -1779,7 +1851,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         return code;
             }
             else {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
         }
         else {
@@ -1795,7 +1867,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
          * been constructed using the extended name....
          */
         if (!r_has_type(&char_name, t_name))
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         name_string_ref(imemory, &char_name, &cname_str);
         enc_char_name->data = cname_str.value.bytes;
         enc_char_name->size = r_size(&cname_str);
@@ -1807,16 +1879,16 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
         if (font_file_path != NULL) {
             ref *Decoding, *TT_cmap, *SubstNWP;
             ref src_type, dst_type;
-            uint c;
+            uint c = 0;
 
             is_glyph_index = true;
 
             if (dict_find_string(pdr, "Decoding", &Decoding) <= 0
                 || !r_has_type(Decoding, t_dictionary))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             if (dict_find_string(pdr, "SubstNWP", &SubstNWP) <= 0
                 || !r_has_type(SubstNWP, t_array))
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             if (dict_find_string(pdr, "TT_cmap", &TT_cmap) <= 0
                 || !r_has_type(TT_cmap, t_dictionary)) {
                 ref *DecodingArray, char_code, char_code1, ih;
@@ -1827,7 +1899,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 if (dict_find(Decoding, &ih, &DecodingArray) <= 0
                     || !r_has_type(DecodingArray, t_array)
                     || array_get(imemory, DecodingArray, i, &char_code) < 0) {
-                    return_error(e_invalidfont);
+                    return_error(gs_error_invalidfont);
                 }
 
                 /* Check the Decoding entry */
@@ -1840,13 +1912,13 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                     n = r_size(DecodingArray);
                 }
                 else {
-                    return_error(e_invalidfont);
+                    return_error(gs_error_invalidfont);
                 }
 
                 for (; n--; i++) {
                     if (array_get(imemory, DecodingArray, i, &char_code1) < 0
                         || !r_has_type(&char_code1, t_integer)) {
-                        return_error(e_invalidfont);
+                        return_error(gs_error_invalidfont);
                     }
 
                     c = char_code1.value.intval;
@@ -1862,14 +1934,14 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                 char *cmapnm = NULL;
                 int cmapnmlen = 0;
                 /* leave off the -H or -V */
-                char *utfcmap = "Identity-UTF16";
+                const char * const utfcmap = "Identity-UTF16";
                 int utfcmaplen = strlen(utfcmap);
 
                 fdict = pfont_dict(gs_rootfont(igs));
-                dict_find_string(fdict, "CMap", &CMapDict);
-                if (r_has_type(CMapDict, t_dictionary)) {
-                    dict_find_string(CMapDict, "CMapName", &CMapName);
-                    if (r_has_type(CMapName, t_name)) {
+                code = dict_find_string(fdict, "CMap", &CMapDict);
+                if (code >= 0 && r_has_type(CMapDict, t_dictionary)) {
+                    code = dict_find_string(CMapDict, "CMapName", &CMapName);
+                    if (code >= 0 && r_has_type(CMapName, t_name)) {
                         name_string_ref(imemory, CMapName, &CMapNameStr);
                         cmapnm = (char *)CMapNameStr.value.bytes;
                         cmapnmlen = r_size(&CMapNameStr);
@@ -1940,7 +2012,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                                                  (const byte **)&Map);
                 }
                 else {
-                    if (CIDMap->tas.rsize < c_code * gdb) {
+                    if (CIDMap->tas.rsize <= c_code * gdb) {
                         c_code = 0;
                     }
                     Map = &CIDMap->value.bytes[c_code * gdb];
@@ -1973,7 +2045,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
 
         if (dict_find_string(pdr, "CharStrings", &CharStrings) <= 0
             || !r_has_type(CharStrings, t_dictionary))
-            return_error(e_invalidfont);
+            return_error(gs_error_invalidfont);
         if ((dict_find(CharStrings, &char_name, &glyph_index) < 0)
             || r_has_type(glyph_index, t_null)) {
 #ifdef DEBUG
@@ -2025,7 +2097,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             ref_assign(esp, glyph_index);
             return o_push_estack;
 #else
-            return (e_invalidfont);
+            return (gs_error_invalidfont);
 #endif
         }
         cr->is_glyph_index = true;
@@ -2051,14 +2123,15 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
             ref *Encoding;
             ref glyph;
 
-            name_ref(pbfont->memory, name->data, name->size, &glyph, false);
+            if ((code = name_ref(pbfont->memory, name->data, name->size, &glyph, false)) < 0)
+                return code;
 
             if (dict_find_string(osp - 1, "Encoding", &Encoding) > 0) {
                 cr->char_codes[0] =
                     (uint) array_find(imemory, Encoding, &glyph);
             }
             else
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
         }
     }
     else {                      /* a non-embedded font, i.e. a disk font */
@@ -2071,8 +2144,9 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                            gx_extendeg_glyph_name_separator);
         if (p != NULL) {
             cr->char_name_length = p - cr->char_name;
-            name_ref(pbfont->memory, cr->char_name, cr->char_name_length,
-                     &char_name, true);
+            if ((code = name_ref(pbfont->memory, cr->char_name,
+                             cr->char_name_length, &char_name, true)) < 0)
+                return code;
         }
         if ((code =
              renderer_retcode(imemory, I,
@@ -2101,14 +2175,14 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
 
                         cr->char_codes_count = r_size(char_code);
                         if (cr->char_codes_count > count_of(cr->char_codes))
-                            code = gs_note_error(e_rangecheck);
+                            code = gs_note_error(gs_error_rangecheck);
                         if (code >= 0) {
                             for (i = 0; i < cr->char_codes_count; i++) {
                                 code = array_get(imemory, char_code, i, &v);
                                 if (code < 0)
                                     break;
                                 if (!r_has_type(char_code, t_integer)) {
-                                    code = gs_note_error(e_rangecheck);
+                                    code = gs_note_error(gs_error_rangecheck);
                                     break;
                                 }
                                 cr->char_codes[i] = v.value.intval;
@@ -2116,7 +2190,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         }
                     }
                     else {
-                        code = gs_note_error(e_rangecheck);
+                        code = gs_note_error(gs_error_rangecheck);
                     }
                     if (code < 0) {
                         char buf[16];
@@ -2130,7 +2204,7 @@ ps_get_glyphname_or_cid(gs_font_base *pbfont, gs_string *charstring,
                         emprintf1(imemory,
                                   "Wrong decoding entry for the character '%s'.\n",
                                   buf);
-                        return_error(e_rangecheck);
+                        return_error(gs_error_rangecheck);
                     }
                 }
             }
@@ -2209,11 +2283,11 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
                  * here, the out of range CID is handled later
                  */
                 if ((dict_find_string(op - 1, "CharStrings", &chstrs)) <= 0) {
-                    return_error(e_undefined);
+                    return_error(gs_error_undefined);
                 }
 
                 if ((dict_find_string(chstrs, ".notdef", &chs)) <= 0) {
-                    return_error(e_undefined);
+                    return_error(gs_error_undefined);
                 }
                 ref_assign_inline(op, chs);
             }
@@ -2254,11 +2328,11 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
          */
         if (code == gs_error_unregistered) {
             os_ptr op = osp;
-            ref *proc, gname;
+            ref *proc = NULL, gname;
 
             if (I->ff.is_type1
                 && (get_charstring(&I->ff, cindex, &proc, &gname) >= 0)
-                && (r_has_type(proc, t_array)
+                && proc != NULL && (r_has_type(proc, t_array)
                     || r_has_type(proc, t_mixedarray))) {
                 push(2);
                 ref_assign(op - 1, &gname);
@@ -2266,7 +2340,7 @@ FAPI_char(i_ctx_t *i_ctx_p, bool bBuildGlyph, ref *charstring)
                 return (zchar_exec_char_proc(i_ctx_p));
             }
             else {
-                return_error(e_invalidfont);
+                return_error(gs_error_invalidfont);
             }
         }
     }
@@ -2303,10 +2377,10 @@ zFAPIBuildGlyph9(i_ctx_t *i_ctx_p)
     font_index = op[0].value.intval;
     if (dict_find_string(&font9, "FDArray", &rFDArray) <= 0
         || r_type(rFDArray) != t_array)
-        return_error(e_invalidfont);
+        return_error(gs_error_invalidfont);
     if (array_get(imemory, rFDArray, font_index, &f) < 0
         || r_type(&f) != t_dictionary)
-        return_error(e_invalidfont);
+        return_error(gs_error_invalidfont);
 
     op[0] = op[-2];
     op[-2] = op[-1];            /* Keep the charstring on ostack for the garbager. */
@@ -2424,7 +2498,7 @@ zFAPIpassfont(i_ctx_t *i_ctx_p)
     if (fapi_request != NULL)
         gs_free_string(imemory, (byte *) fapi_request,
                        strlen(fapi_request) + 1, "do_FAPIpassfont");
-    if (code != 0)
+    if (code < 0 && code != gs_error_invalidaccess)
         return code;
 
     if (code >= 0 && fapi_id != NULL) {
