@@ -82,7 +82,7 @@ $user=""    if (!$user);
 
 unlink "cluster_command.run";
 
-my $host="casper.ghostscript.com";
+my $host="cluster.ghostscript.com";
 my $dir="/home/regression/cluster/users";
 
 # To cater for those whose cluster user name doesn't match the user name
@@ -137,7 +137,12 @@ print "$user $directory $product\n" if ($verbose);
 
 
 if ($directory eq 'gs') {
-  $directory='ghostpdl/gs';
+  if (-e 'gpdl') {
+    print "new directory structure\n";
+    $directory='ghostpdl';
+  } else {
+    $directory='ghostpdl/gs';
+  }
 }
 
 
@@ -177,13 +182,15 @@ my $cmd="rsync -avxcz ".
 " --exclude config.log --exclude .png".
 " --exclude .ppm --exclude .pkm --exclude .pgm --exclude .pbm".
 " --exclude .tif --exclude .bmp".
-" --exclude debug --exclude release --exclude generated".  # we cannot just exclude build, since tiff/build/Makefile.in, etc. is needed
-" --exclude tiff-config".  # we cannot just exclude build, since tiff/build/Makefile.in, etc. is needed
-# " --exclude Makefile". We can't just exclude Makefile, since the GhostPDL top Makefile is not a derived file.
+" --exclude debug --exclude release --exclude generated --exclude sanitize".  # we cannot just exclude build, since tiff/build/Makefile.in, etc. is needed
+" --exclude tiff-config".
+# " --exclude Makefile". We can't just exclude Makefile, since the MuPDF Makefile is not a derived file.
 " -e \"$ssh\" ".
 " .".
 " $hostpath";
 
+#print "$cmd\n";  exit;
+
 if ($product ne "abort" ) { #&& $product ne "bmpcmp") {
   print STDERR "syncing\n";
   print "$cmd\n" if ($verbose);
